Package: nparLD
Version: 2.2
Date: 2022-08-06
Title: Nonparametric Analysis of Longitudinal Data in Factorial
        Experiments
Author: Kimihiro Noguchi <noguchk@wwu.edu>, Mahbub Latif
        <mlatif@isrt.ac.bd>, Karthinathan Thangavelu,
        Frank Konietschke <frank.konietschke@charite.de>, Yulia R. Gel
        <yxg142030@utdallas.edu>, Edgar Brunner
        <ebrunne1@gwdg.de>
Maintainer: Frank Konietschke <frank.konietschke@charite.de>
Depends: R (>= 2.6.0), MASS
License: GPL (>= 2)
Description: Performs nonparametric
        analysis of longitudinal data in factorial experiments.
        Longitudinal data are those which are collected from the same
        subjects over time, and they frequently arise in biological
        sciences. Nonparametric methods do not require distributional
        assumptions, and are applicable to a variety of data types
        (continuous, discrete, purely ordinal, and dichotomous).  Such
        methods are also robust with respect to outliers and for small
        sample sizes.
NeedsCompilation: no
Packaged: 2022-08-06 14:39:07 UTC; Matthias
Repository: CRAN
Date/Publication: 2022-08-07 00:00:02 UTC
Built: R 4.6.0; ; 2026-04-26 23:09:00 UTC; unix

Package: nparLD
Type: Package
Title: Nonparametric Analysis of Longitudinal Data in Factorial
        Experiments
Version: 2.3.0
Authors@R: c(
    person(given = "Frank",
           family = "Konietschke",
           role = c("aut", "cre"),
           email = "frank.konietschke@charite.de"),
    person(given = "Kimihiro",
           family = "Noguchi",
           role = "ctb",
           comment = "Original package author"),
    person(given = "Mahbub",
           family = "Latif",
           role = "ctb",
           comment = "Original package author"),
    person(given = "Karthinathan",
           family = "Thangavelu",
           role = "ctb",
           comment = "Original package author"),
    person(given = c("Yulia", "R."),
           family = "Gel",
           role = "ctb",
           comment = "Original package author"),
    person(given = "Edgar",
           family = "Brunner",
           role = "ctb")
 )
Description: Provides nonparametric procedures for the analysis of longitudinal data in factorial experiments. The package implements hypothesis tests on marginal distribution functions and unweighted relative marginal effects. It supports arbitrary crossed factorial designs with longitudinal or repeated-measures factors, missing observations, dependent replicates, rank- and pseudo-rank-based inference, Wald-type and ANOVA-type statistics, multiple contrast tests, and simultaneous confidence intervals.
License: GPL (>= 2)
Encoding: UTF-8
LazyData: true
Imports: MASS, Matrix, ggplot2, mvtnorm, multcomp, rlang
Config/roxygen2/version: 8.0.0
Suggests: knitr, rmarkdown, spelling, testthat (>= 3.0.0)
Config/testthat/edition: 3
Depends: R (>= 3.5)
VignetteBuilder: knitr
Language: en-US
NeedsCompilation: no
Packaged: 2026-08-22 18:54:53 UTC; konietsf
Author: Frank Konietschke [aut, cre],
  Kimihiro Noguchi [ctb] (Original package author),
  Mahbub Latif [ctb] (Original package author),
  Karthinathan Thangavelu [ctb] (Original package author),
  Yulia R. Gel [ctb] (Original package author),
  Edgar Brunner [ctb]
Maintainer: Frank Konietschke <frank.konietschke@charite.de>
Repository: CRAN
Date/Publication: 2026-08-22 19:20:02 UTC

# Generated by roxygen2: do not edit by hand

S3method(plot,nparld_fit)
S3method(plot,nparld_mctp)
S3method(print,nparld_covarianceinfo)
S3method(print,nparld_factorinfo)
S3method(print,nparld_fit)
S3method(print,nparld_mctp)
S3method(print,summary.nparld_fit)
S3method(summary,nparld_fit)
export(nparLD)
importFrom(graphics,abline)
importFrom(graphics,axis)
importFrom(graphics,points)
importFrom(graphics,segments)
importFrom(rlang,.data)
importFrom(stats,aggregate)
importFrom(stats,as.formula)
importFrom(stats,ave)
importFrom(stats,cov2cor)
importFrom(stats,model.frame)
importFrom(stats,na.pass)
importFrom(stats,pchisq)
importFrom(stats,pf)
importFrom(stats,pt)
importFrom(stats,qnorm)
importFrom(stats,qt)
importFrom(stats,quantile)
importFrom(stats,rbinom)
importFrom(stats,reshape)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,terms)
importFrom(stats,var)

nparLD
================

# nparLD

`nparLD` provides nonparametric methods for the analysis of longitudinal
and repeated-measures data in factorial experiments. The package
implements procedures for hypotheses in marginal distribution functions
and in unweighted relative marginal effects. It supports crossed
factorial designs, missing observations, dependent replicate
measurements, rank- and pseudo-rank-based inference, Wald-type and
ANOVA-type statistics, multiple contrast procedures, and simultaneous
confidence intervals.

## Installation

You can install the development version locally with:

``` r
devtools::install()
```

## Basic example

``` r
library(nparLD)

set.seed(123)

T <- 4
n <- c(10, 10, 10)
N <- sum(T * n)

y <- rnorm(N)
times <- c(rep(1:T, n[1]), rep(1:T, n[2]), rep(1:T, n[3]))
grp <- c(rep(1, n[1] * T), rep(2, n[2] * T), rep(3, n[3] * T))
subjects <- c(
  sort(rep(1:n[1], T)),
  sort(rep((n[1] + 1):(n[1] + n[2]), T)),
  sort(rep((n[1] + n[2] + 1):(n[1] + n[2] + n[3]), T))
)

data <- data.frame(times, grp, subjects, y)

fit <- nparLD(
  y ~ grp * times,
  data = data,
  subject = "subjects",
  hypothesis = "H0p",
  contrast = list("grp:times")
)

fit
plot(fit)
plot(fit$MCTP)
```

## Weighted and unweighted relative effects

The package distinguishes between weighted and unweighted relative
marginal effects.

Weighted effects are based on the sample-size weighted reference
distribution. They describe relative positions with respect to the
empirical distribution obtained by pooling the marginal samples
according to their observed sample sizes. Weighted effects are useful as
descriptive summaries and correspond to ordinary rank-based estimators.

Unweighted effects are based on an equally weighted reference
distribution over the factorial cells. Each cell contributes equally to
the reference distribution, independently of its sample size. These
effects are the inferential target for hypotheses in relative marginal
effects, selected by `hypothesis = "H0p"`, and are estimated using
pseudo-ranks.

Consequently, tests of hypotheses in relative marginal effects are
performed for unweighted effects. Weighted effects are reported
descriptively, but they are not used as the target of `H0p` tests
because their interpretation depends on the sample-size allocation
across cells.

\`\`\`\`markdown The following example illustrates how the two effect
concepts are requested.

``` r
data(panic2)

fit_weighted <- nparLD(
  resp ~ group * time,
  data = panic2,
  subject = "subject",
  effect = "weighted",
  hypothesis = "H0F"
)

fit_unweighted <- nparLD(
  resp ~ group * time,
  data = panic2,
  subject = "subject",
  effect = "unweighted",
  hypothesis = "H0F"
)

fit_weighted
fit_unweighted
```

The first analysis reports weighted effects as descriptive summaries
together with tests in marginal distribution functions. The second
analysis targets unweighted relative marginal effects and therefore
provides the appropriate effect scale for `H0p`.

## Hypothesis types

Use `hypothesis = "H0F"` to test hypotheses in marginal distribution
functions. These tests compare marginal distribution functions either
using classical ranks with the argument `effect="weighted"` or using
pseudo-ranks using the argument `effect = "unweighted"`.

Use `hypothesis = "H0p"` to test hypotheses in unweighted relative
marginal effects. These effects describe the relative position of each
marginal distribution with respect to a common unweighted reference
distribution and are useful for interpretation, contrasts, simultaneous
confidence intervals, and plots.

## Dependent replicates

Dependent replicate measurements can be specified with the `replicate`
argument.

``` r
datensatz2 <- datensatz[rep(seq_len(nrow(datensatz)), each = 3), ]
datensatz2$rep <- rep(1:3, times = nrow(datensatz))
datensatz2$y <- rnorm(nrow(datensatz2))

fit_rep <- nparLD(
  y ~ grp * times,
  data = datensatz2,
  subject = "subjekte",
  replicate = "rep",
  hypothesis = "H0p",
  cell.weights = "subjects"
)

fit_rep
```

For relative marginal effects, `cell.weights = "subjects"` targets a
typical subject-condition cell, while `cell.weights = "observations"`
targets a typical replicate observation.

## Classical longitudinal designs

Earlier versions of `nparLD` used design-specific functions such as
`ld.f1()`, `f1.ld.f1()`, or `f2.ld.f1()`. In the redesigned package,
these designs are specified through one general formula interface.

Typical examples are:

| Classical design | Formula in the redesigned interface | Example |
|----|----|----|
| `LD-F1` | `resp ~ time` | one repeated-measures factor |
| `F1-LD-F1` | `resp ~ group * time` | one whole-plot and one repeated-measures factor |
| `LD-F2` | `resp ~ time1 * time2` | two repeated-measures factors |
| `F1-LD-F2` | `resp ~ group * time1 * time2` | one whole-plot and two repeated-measures factors |
| `F2-LD-F1` | `resp ~ group1 * group2 * time` | two whole-plot and one repeated-measures factor |

For example, the shoulder data correspond to an `F2-LD-F1` design:

``` r
data(shoulder)

fit <- nparLD(
  resp ~ group1 * group2 * time,
  data = shoulder,
  subject = "subject",
  hypothesis = "H0p",
  contrast = list("group1:time")
)

fit
plot(fit)
plot(fit$MCTP)
```

#*******************************************************************************
# Helpers: design + ordering
#*******************************************************************************


.nparld_build_design <- function(formula, data, subject, replicate = NULL) {

  dat.Model0 <- model.frame(formula, data = data, na.action = na.pass)

  # subject column
  if (!(subject %in% names(data))) {
    stop("Subject variable '", subject, "' not found in data.", call. = FALSE)
  }
  dat.Model0$subject <- as.factor(data[[subject]])

  # optional replicate column (NOT part of the design factors)
  if (!is.null(replicate)) {
    if (!(replicate %in% names(data))) {
      stop("replicate variable '", replicate, "' not found in data.", call. = FALSE)
    }
    dat.Model0$replicate <- as.factor(data[[replicate]])
  }

  N <- nlevels(dat.Model0$subject)

  # design variables = all columns except response, subject, replicate
  response_name <- names(dat.Model0)[1]
  design_cols <- setdiff(names(dat.Model0), c(response_name, "subject", "replicate"))

  for (nm in design_cols) dat.Model0[[nm]] <- as.factor(dat.Model0[[nm]])

  # safety: no missing values in design factors
  bad_design <- design_cols[vapply(dat.Model0[design_cols], function(z) any(is.na(z)), logical(1))]
  if (length(bad_design) > 0) {
    stop(
      "Missing values in design factors are not allowed: ",
      paste(bad_design, collapse = ", "),
      call. = FALSE
    )
  }

  bysubjects <- split(dat.Model0, dat.Model0$subject)

  # helper to create a subject-specific schedule signature
  .schedule_signature <- function(df, cols) {
    if (length(cols) == 0L) return("")
    sch <- unique(df[cols])
    sch[] <- lapply(sch, as.character)
    sch <- sch[do.call(order, sch), , drop = FALSE]
    row_sig <- do.call(paste, c(sch, sep = "\r"))
    paste(row_sig, collapse = "\n")
  }

  # determine which factors vary within subject
  factor_info <- lapply(design_cols, function(nm) {
    nuniq <- vapply(bysubjects, function(df) length(unique(df[[nm]])), integer(1))
    list(nuniq = nuniq)
  })
  names(factor_info) <- design_cols

  WP.names <- names(Filter(function(info) all(info$nuniq == 1L), factor_info))
  SP.names <- names(Filter(function(info) all(info$nuniq > 1L), factor_info))

  # factors that vary within some subjects but not others -> invalid schedule
  bad_mix <- names(Filter(function(info) {
    !(all(info$nuniq == 1L) || all(info$nuniq > 1L))
  }, factor_info))

  if (length(bad_mix) > 0) {
    stop(
      "Inconsistent within-subject variation for factor(s): ",
      paste(bad_mix, collapse = ", "),
      ". Some subjects are missing subplot levels.",
      call. = FALSE
    )
  }

  n.wholeplots <- length(WP.names)
  n.subplots <- length(SP.names)

  if (n.subplots == 0L) {
    stop("There is no subplot factor provided. Please check the model! Otherwise, please use the package rankFD for the analysis of independent observations.", call. = FALSE)
  }

  # strict schedule check, but ignoring replicate replication:
  # every subject must have the same unique subplot combinations
  sp_sig <- vapply(bysubjects, .schedule_signature, character(1), cols = SP.names)

  if (length(unique(sp_sig)) != 1L) {
    ref <- sp_sig[1]
    bad <- names(sp_sig)[sp_sig != ref]
    stop(
      "Subjects do not share the same subplot schedule. Please check subject(s): ",
      paste(bad, collapse = ", "),
      ". Unequal replicate sizes are allowed, but missing subplot combinations are not.",
      call. = FALSE
    )
  }

  if (n.wholeplots == 0) {
    Design <- paste("LD-F", n.subplots, "-Design \n", sep = "")
  }
  if (n.wholeplots > 0 && n.subplots > 0) {
    Design <- paste("F", n.wholeplots, "-LD-F", n.subplots, "-Design \n", sep = "")
  }

  # sort order: replicate may be used only for stable ordering
  sort_order <- c(WP.names, "subject", SP.names, if (!is.null(replicate)) "replicate" else NULL)
  dat.Model.sorted <- dat.Model0[do.call(order, dat.Model0[sort_order]), , drop = FALSE]

  factor_cols <- c(WP.names, SP.names)
  n.levels <- sapply(factor_cols, function(x) nlevels(dat.Model.sorted[[x]]))
  names.levels <- lapply(factor_cols, function(x) levels(dat.Model.sorted[[x]]))
  names(names.levels) <- factor_cols

  # rewrite formula in correct factor order (WP first, SP second)
  new_formula <- as.formula(
    paste(response_name, "~", paste(factor_cols, collapse = "*"))
  )

  list(
    dat = dat.Model.sorted,
    formula = new_formula,
    WP.names = WP.names,
    SP.names = SP.names,
    factor_cols = factor_cols,
    n.levels = n.levels,
    names.levels = names.levels,
    N = N,
    N.info = N,
    Design = Design,
    wholeplots = WP.names,
    subplots = SP.names
  )
}

.nparld_make_keys <- function(df, fac_names){
  if(length(fac_names)==0)
    return(rep("1", nrow(df)))

  do.call(paste,
          c(lapply(fac_names, function(v) as.character(df[[v]])),
            sep=":"))
}

.nparld_cell_keys <- function(dat, WP.names, SP.names){

  WP.names <- as.character(WP.names)
  SP.names <- as.character(SP.names)

  fac_names <- c(WP.names, SP.names)

  if(length(fac_names)==0)
    return("1")

  # expand.grid trick: reverse order so that the last factor varies fastest
  grid <- expand.grid(
    lapply(rev(fac_names), function(v) levels(dat[[v]])),
    KEEP.OUT.ATTRS = FALSE,
    stringsAsFactors = FALSE
  )

  names(grid) <- rev(fac_names)

  # restore original factor order
  grid <- grid[fac_names]

  # build keys
  cell_keys <- do.call(
    paste,
    c(lapply(grid, as.character), sep=":")
  )

  cell_keys
}

#' nparLD: Nonparametric Longitudinal Data Analysis
#'
#' The \pkg{nparLD} package provides nonparametric methods for the analysis of
#' longitudinal and repeated-measures data in factorial experiments. It is
#' especially useful for settings in which response distributions may be non-normal,
#' ordinal, skewed, heteroscedastic, or affected by ties. The procedures do not 
#' require distributional assumptions, and are applicable to a variety of data types 
#'(continuous, discrete, purely ordinal, and dichotomous). The methods are also robust 
#' with respect to outliers and for small sample sizes.
#'
#' The main function is [nparLD()], which implements inference for hypotheses in
#' marginal distribution functions and in unweighted relative marginal effects.
#' The package supports crossed factorial designs with whole-plot and sub-plot
#' factors, missing observations, dependent replicate measurements, rank- and
#' pseudo-rank-based estimation, confidence intervals, Wald-type and ANOVA-type statistics, multiple
#' contrast procedures, and simultaneous confidence intervals.
#'
#' @section Main function:
#' \itemize{
#'   \item [nparLD()] performs the nonparametric analysis.
#' }
#'
#' @section Hypotheses:
#' The argument \code{hypothesis = "H0F"} tests hypotheses in marginal
#' distribution functions. The argument \code{hypothesis = "H0p"} tests
#' hypotheses in unweighted relative marginal Mann-Whitney effects and thus addresses 
#' the nonparametric Behrens-Fisher problem in factorial longitudinal designs.
#'
#' @section Effects:
#' The argument \code{effect = "weighted"} estimates weighted relative marginal Mann-Whitney effects
#' using classical ranks (mid-ranks) of the observations. The argument \code{effect = "unweighted"} 
#' estimates unweighted relative marginal Mann-Whitney effects using pseudo-ranks of the data. The weighted relative marginal
#' effect depends on sample sizes and their allocations, whereas the unweighted relative marginal effect does not.
#'
#' @section Contrast:
#' The argument \code{contrast = list()} estimates and tests contrasts on the given factor levels or their interaction effects
#' using multiple contrast tests. If the null hypothesis H0p is tested, then simultaneous confidence intervals are computed.
#'
#'
#' @section Replicates:
#' Dependent replicates can be specified by the \code{replicate} argument.
#' For relative marginal effects, the \code{cell.weights} argument determines
#' whether subject-condition cells or individual replicate observations define
#' the target of estimation.
#'
#' @keywords internal
#' @importFrom graphics abline axis points segments
#' @importFrom stats aggregate as.formula ave cov2cor model.frame na.pass pchisq pf pt qnorm qt quantile rbinom reshape sd setNames terms var
#' @importFrom rlang .data
"_PACKAGE"

#' Nonparametric Analysis of Longitudinal Data
#'
#' Performs nonparametric inference for longitudinal or repeated-measures data
#' from crossed factorial experiments. The function can be used to test hypotheses in
#' marginal distribution functions or hypotheses in unweighted relative
#' marginal effects. It allows crossed whole-plot and sub-plot factors,
#' missing values, and dependent replicate measurements.
#'
#' @param formula A model formula of the form \code{response ~ factors}. Factors
#'   on the right-hand side may contain crossed whole-plot and subplot factors
#'   and their interactions.
#' @param data A data frame containing the response, subject variable, design
#'   factors, and optionally a replicate variable.
#' @param subject Character string specifying the subject identifier.
#' @param replicate Optional character string specifying the replicate identifier.
#'   Replicates can be dependent within subject-condition cells.
#' @param cell.weights Character string specifying how dependent replicates are
#'   weighted. Use \code{"subjects"} for subject-level cell weighting and
#'   \code{"observations"} for observation-level cell weighting. 
#'   For dependent replicates under \code{hypothesis = "H0F"},
#' \code{cell.weights = "subjects"} averages rank scores within
#' subject-condition cells before covariance estimation, whereas
#' \code{cell.weights = "observations"} uses the sums of the observed rank
#' scores within subject-condition cells and therefore gives more weight to
#' subjects with more observed replicate measurements.The default is
#'   \code{"subjects"}.
#' @param hypothesis Character string specifying the hypothesis type. Use
#'   \code{"H0F"} for hypotheses in marginal distribution functions and
#'   \code{"H0p"} for hypotheses in unweighted relative marginal effects.
#'   The latter permits unequal variances or higher-order distributional
#'   differences under the null hypothesis and is related to the nonparametric
#'   Behrens-Fisher problem.
#' @param effect Character string specifying whether weighted or unweighted
#'   relative effects are used. Use \code{"weighted"} for effects based on
#'   ordinary ranks and a sample-size weighted reference distribution. Use
#'   \code{"unweighted"} for effects based on pseudo-ranks and an equally
#'   weighted factorial-cell reference distribution. Weighted effects depend on
#'   the sample-size allocation and are mainly descriptive. Unweighted effects
#'   are the inferential target for \code{hypothesis = "H0p"}. In balanced
#'   designs, both effect definitions coincide. The default is
#'   \code{effect = "unweighted"}.
#' @param contrast Optional list specifying a factor or interaction term for
#'   simultaneous inference. A one-element specification such as
#'   \code{list("time")} uses the standard hypothesis matrix for the selected
#'   model term, that is, the same contrast structure underlying the WTS and ATS
#'   for that term. A two-element specification such as
#'   \code{list("time", "Tukey")} or \code{list("time", "Dunnett")} applies the
#'   named multiple contrast procedure to the marginal relative effects of the
#'   selected term. A numeric vector or matrix may be supplied as the second
#'   element for user-defined contrasts.
#' @param sci.method Character string specifying the method for simultaneous
#'   confidence intervals. Available choices are \code{"fisher"} and \code{"multi.t"}. 
#' The default is \code{"fisher"}.
#' @param Factor.Information Logical. If \code{TRUE}, factor-specific relative
#'   effects, standard errors, and confidence intervals are returned. These
#'   summaries can also be displayed with \code{plot(fit, term = ...)}.
#' @param alpha Significance level for tests and confidence intervals. The
#'   default is \code{alpha = 0.05}.
#' @param CI.method Character string specifying the method for confidence intervals for 
#' relative effects. Available choices are \code{"logit"} and \code{"normal"}. The default is \code{"logit"}.
#' @param covariance Logical. If \code{TRUE}, the estimated covariance matrix
#'   used for the selected hypothesis is included in the output. The default is
#'   \code{FALSE}.
#' @param perm.test Logical. Should the paired two-time-point permutation test be
#'   used when applicable? This option is only available for paired designs with
#'   two time points.
#' @param B Number of permutation samples used for the paired permutation test.
#'
#' @details
#' The function provides rank- and pseudo-rank-based procedures for factorial
#' longitudinal data. The argument \code{hypothesis = "H0F"} specifies
#' hypotheses in marginal distribution functions. These hypotheses compare the
#' complete marginal distributions and are tested by rank-based procedures.
#'
#' The argument \code{hypothesis = "H0p"} specifies hypotheses in unweighted
#' relative marginal effects. These effects describe the relative position of
#' each marginal distribution with respect to a common unweighted reference
#' distribution and are particularly useful for effect interpretation, multiple
#' contrast procedures, simultaneous confidence intervals, and graphical
#' summaries.
#'
#' The \code{contrast} argument computes a multiple contrast test along with simultaneous
#' confidence intervals for the selected model terms. A one-element specification such as 
#' \code{list("time")} uses the same hypothesis matrix as the global test procedures (Wald-
#' and ANOVA-type statistics) for the selected term. This
#' provides simultaneous inference for the components of the corresponding
#' global null hypothesis. A two-element specification such as
#' \code{list("time", "Tukey")} or \code{list("time", "Dunnett")} first forms
#' the marginal relative effects for the selected term and then applies the
#' requested multiple contrast procedure. Thus, \code{list("time", "Tukey")}
#' gives pairwise comparisons of the marginal time effects, whereas
#' \code{list("time", "Dunnett")} compares the marginal time effects with the
#' first time point. User-defined contrast vectors or matrices can be supplied
#' as the second list element.
#'
#' Missing observations are allowed. Incomplete subject-condition cells
#' contribute where observations are available, and covariance estimation is
#' based on the independent subject-level units. Dependent replicate
#' measurements can be specified using the \code{replicate} argument. For
#' hypotheses in relative marginal effects, \code{cell.weights = "subjects"}
#' targets a typical subject-condition cell, whereas
#' \code{cell.weights = "observations"} targets a typical replicate observation.
#' For hypotheses in marginal distribution functions, dependent replicates are
#' handled by averaging rank or pseudo-rank scores within subject-condition
#' cells.

#' In case of bivariate data (e.g., before and after measurements), the function implements
#' a studentized permutation test for testing either \code{hypothesis = "H0F"} or 
#' \code{hypothesis = "H0p"}.
#'
#'
#' @return
#' An object of class \code{"nparld_fit"}. The object is a list containing the
#' results of the nonparametric longitudinal analysis. The main components are:
#' \itemize{
#'   \item \code{Design}: character string describing the detected longitudinal
#'     factorial design.
#'   \item \code{wholeplots}: names of the whole-plot factors.
#'   \item \code{subplots}: names of the subplot or repeated-measures factors.
#'   \item \code{text.ranks}: character string describing the type of ranks used.
#'   \item \code{text.hypotheses}: character string describing the type of
#'     hypotheses tested.
#'   \item \code{N.info}: information on the number of subjects and observations.
#'   \item \code{effects}: a data frame with estimated relative effects. This
#'     includes the factor-level combinations, the number of contributing
#'     subjects and observations, the number of missing observations, the mean
#'     rank or pseudo-rank score, the estimated relative effect, and its
#'     standard error. For \code{hypothesis = "H0p"}, confidence limits are also
#'     returned.
#'   \item \code{factor.info}: optional factor-specific relative effects,
#'     standard errors, and confidence limits for main effects and interactions,
#'     returned when \code{Factor.Information = TRUE}. These summaries can be
#'     displayed with \code{plot(fit, term = ...)}.
#'   \item \code{WTS}: Wald-type test statistics, degrees of freedom, and
#'     p-values for the tested main effects and interactions.
#'   \item \code{ATS}: ANOVA-type test statistics, denominator degrees of
#'     freedom, and p-values for the tested main effects and interactions.
#'   \item \code{MCTP}: multiple contrast test results, returned when
#'     \code{contrast} is specified. These include the selected factor or
#'     interaction, the simultaneous confidence interval method, the global
#'     multiple contrast test, local contrast estimates with standard errors,
#'     simultaneous confidence limits, test statistics, p-values, degrees of
#'     freedom, and the corresponding contrast matrix.
#'   \item \code{covariance.info}: estimated covariance matrix used for the
#'     selected hypothesis, returned when \code{covariance = TRUE}.
#'   \item \code{perm}: permutation test results, returned when a permutation
#'     test is requested and applicable.
#'   \item \code{hypothesis}: the type of hypothesis tested, either \code{"H0F"}
#'     for hypotheses in marginal distribution functions or \code{"H0p"} for
#'     hypotheses in unweighted relative marginal effects.
#'   \item \code{CI.method}: method used for confidence intervals.
#'   \item \code{.internal}: internal objects used for computation and advanced
#'     post-processing.
#' }
#'
#' Some components may be \code{NULL} depending on the selected hypothesis,
#' contrast specification, and output options.
#'
#'
#'
#' @examples
#' ## One repeated-measures factor: dental data
#' data(dental)
#'
#' fit_dental <- nparLD(
#'   resp ~ time,
#'   data = dental,
#'   subject = "subject",
#'   hypothesis = "H0p"
#' )
#'
#' fit_dental
#'
#' ## Crossed factorial longitudinal design: shoulder data
#' data(shoulder)
#'
#' \dontrun{
#' fit_shoulder <- nparLD(
#'   resp ~ group1 * group2 * time,
#'   data = shoulder,
#'   subject = "subject",
#'   hypothesis = "H0p"
#' )
#'
#' fit_shoulder
#' }
#'
#' ## Multiple contrast procedure for an interaction
#'
#' \dontrun{
#' fit_contrast <- nparLD(
#'   resp ~ group1 * group2 * time,
#'   data = shoulder,
#'   subject = "subject",
#'   hypothesis = "H0p",
#'   contrast = list("group1:time")
#' )
#'
#' fit_contrast$MCTP
#' print(fit_contrast$MCTP, show.matrix = TRUE)
#' }
#' 
#' ## Missing response values
#' \dontrun{
#' dat_miss <- dental
#' dat_miss$resp[c(2, 7, 12)] <- NA
#'
#' fit_miss <- nparLD(
#'   resp ~ time,
#'   data = dat_miss,
#'   subject = "subject",
#'   hypothesis = "H0p"
#' )
#'
#' fit_miss
#' }
#'
#' ## Dependent replicate measurements: BrdU data
#' data(brdu)
#'
#' \dontrun{
#' fit_brdu <- nparLD(
#'   resp ~ dose,
#'   data = brdu,
#'   subject = "culture",
#'   replicate = "replicate",
#'   hypothesis = "H0p",
#'   cell.weights = "subjects"
#' )
#'
#' fit_brdu
#'}
#'
#' \dontrun{
#' ## Graphical displays
#' plot(fit_dental)
#' plot(fit_shoulder)
#' plot(fit_contrast$MCTP)
#' }
#'
#'
#' @references 
#'
#' Akritas, M. G., & Brunner, E. (1997). A unified approach to rank tests for mixed models. 
#' Journal of Statistical Planning and Inference, 61(2), 249-277.
#'
#' Brunner, E., Bathke, A.C., Konietschke, F. Rank and Pseudo-Rank Procedures 
#' for Independent Observations in Factorial Designs. Springer International Publishing, 2018.
#'
#' Brunner, E., Domhof, S., & Langer, F. (2002). Nonparametric analysis of longitudinal data in factorial experiments. Wiley, New York
#'
#' Brunner, E., Konietschke, F., Pauly, M., Puri, M. L. (2017). Rank-based procedures in factorial designs: 
#' Hypotheses about non-parametric treatment effects. Journal of the Royal Statistical Society: Series B 
#' (Statistical Methodology), 79(5), 1463-1485.
#' 
#' Domhof, S., Brunner, E., Osgood, D. W. (2002). Rank procedures for repeated measures with missing values. 
#' Sociological methods & research, 30(3), 367-393.
#' 
#' Konietschke, F., Bathke, A. C., Hothorn, L. A., & Brunner, E. (2010). 
#' Testing and estimation of purely nonparametric effects in repeated measures designs. 
#' Computational Statistics & Data Analysis, 54(8), 1895-1905.
#'
#' Konietschke, F., Hothorn, L. A., Brunner, E. (2012). Rank-based multiple test procedures and simultaneous confidence intervals. Electronic Journal of Statistics,
#' 6, 738-759.
#' 
#' Rubarth, K., Pauly, M., & Konietschke, F. (2022). Ranking procedures for repeated measures designs with missing data: 
#' estimation, testing and asymptotic theory. 
#' Statistical Methods in Medical Research, 31(1), 105-118.
#'
#' Rubarth, K., Sattler, P., Zimmermann, H. G., & Konietschke, F. (2021). 
#' Estimation and testing of Wilcoxon–Mann–Whitney effects in factorial clustered data designs. 
#' Symmetry, 14(2), 244.
#' 
#'
#'
#' @export 

nparLD <- function(formula, data, subject, replicate = NULL,
                    cell.weights = c("subjects", "observations"),
                   effect = c("unweighted", "weighted"),
                   hypothesis = c("H0F", "H0p"),
                   contrast = NULL,
                   sci.method = c("fisher", "multi.t"),
                   Factor.Information = FALSE,
                   CI.method = c("logit", "normal"),
                   alpha = 0.05,
                   covariance = FALSE,
                   perm.test = FALSE,
                   B = 1000) {

effect    <- match.arg(effect)
cell.weights <- match.arg(cell.weights)
hypothesis   <- match.arg(hypothesis)
CI.method    <- match.arg(CI.method)
sci.method   <- match.arg(sci.method)
if (cell.weights %in% c("nu1", "subjects")) cell.weights <- "subjects"
if (cell.weights %in% c("nu2", "observations")) cell.weights <- "observations"

  #*******************************************************************************
  # Factor Design
  #*******************************************************************************
  design <- .nparld_build_design(formula, data, subject, replicate)

  dat <- design$dat
  formula <- design$formula
  replicate_internal <- if (!is.null(replicate)) "replicate" else NULL

.nparld_check_complete_within_schedule(
  dat,
  design$SP.names,
  replicate = replicate_internal
)

.check_replicate_structure(
  dat = dat,
  WP.names = design$WP.names,
  SP.names = design$SP.names,
  replicate = replicate_internal
)
  #*******************************************************************************
  # Add Ranks or Pseudo-Ranks to the data set
  #*******************************************************************************
  split_obj <- .nparld_split_by_wp_subject_sp(
    dat,
    design$WP.names,
    design$SP.names
  )

  Fhat <- NULL
  Ghat <- NULL

  if (effect == "weighted") {
    dat$Ranks <- rank(dat[[1]], na.last = "keep")
    text.ranks <- "Classical ranks (mid-ranks)"
  }

  if (effect == "unweighted" || hypothesis == "H0p") {
    Fhat <- .nparld_build_Fhat(
      split_obj,
      cell.weights = cell.weights
    )

Ghat <- .nparld_build_Ghat(
  split_obj = split_obj,
  Fhat = Fhat,
  effect = effect,
  cell.weights = cell.weights
)

  Ntotal <- sum(!is.na(dat[[1]]))
dat <- .nparld_compute_pseudoranksN(dat, Ghat, Ntotal)
    text.ranks <- "Pseudo-ranks"
  }
est <- .nparld_estimate_theta_rankmeans(
  dat = dat,
  formula = formula,
  WP.names = design$WP.names,
  SP.names = design$SP.names,
  replicate = replicate_internal,
  effect = effect,
  cell.weights = cell.weights
)

  dat <- est$dat
  effects <- est$effects
  theta_hat <- est$theta_hat
  Ntotal <- est$Ntotal

  #*******************************************************************************
  # Compute the covariance matrix accordingly
  #*******************************************************************************
  hy <- .nparld_build_hypotheses(
    formula,
    design$factor_cols,
    design$n.levels
  )

  res.mctp <- NULL
  Sigma <- NULL
  df_satt <- NULL

if (hypothesis == "H0p") {

  if (effect == "weighted") {
    stop("Testing H0p with weighted reference distribution is not implemented.",
         call. = FALSE)
  }

  if (!is.null(contrast)) {

    if (!any(hy$Output.names == contrast[[1]])) {
      stop("Factor not found in model.", call. = FALSE)
    }

    pos <- which(hy$Output.names == contrast[[1]])

    if (length(pos) != 1L) {
      stop("Requested contrast term is ambiguous.", call. = FALSE)
    }

    if (length(contrast) == 1L) {
      ## Use the same hypothesis matrix as WTS/ATS
      H <- hy$Hypotheses[[pos]]
      C <- diag(nrow(H))
      rownames(C) <- rownames(H)
    } else {
      ## Use marginal-effect matrix and then apply named/user contrast
      H <- hy$CI.Matrices[[pos]]
      C <- .nparld_build_contrast_matrix(contrast, H)
    }

    CH <- C %*% H

    if (cell.weights == "subjects") {
      covp <- .nparld_cov_H0p_ref_unweighted_subjects(
        split_obj,
        Fhat,
        C = CH
      )
    } else {
      covp <- .nparld_cov_H0p_ref_unweighted_observations(
        split_obj,
        Fhat,
        C = CH
      )
    }

  } else {

    if (cell.weights == "subjects") {
      covp <- .nparld_cov_H0p_ref_unweighted_subjects(
        split_obj,
        Fhat,
        C = NULL
      )
    } else {
      covp <- .nparld_cov_H0p_ref_unweighted_observations(
        split_obj,
        Fhat,
        C = NULL
      )
    }
  }

  Sigma   <- covp$Sigma
  df_satt <- covp$df_satt

  if (is.null(Sigma)) {
    stop("Sigma could not be computed.", call. = FALSE)
  }

  var_theta <- diag(Sigma)

  eps_effect <- apply(covp$eps_theta, 1, function(x) {
    x <- x[is.finite(x)]
    if (length(x) == 0L) NA_real_ else min(x)
  })

  var_theta <- .nparld_regularize_theta_variance(
    var_theta = var_theta,
    eps_theta = eps_effect
  )

  Sigma.reg <- Sigma
  diag(Sigma.reg) <- var_theta

  effects$std.error <- sqrt(diag(Sigma.reg))

  effects <- cbind(
    effects,
    .nparld_limits(theta_hat, Sigma.reg, alpha, design$N, CI.method)
  )

  Sigma <- Sigma.reg

  text.hypotheses <- "Tested in marginal Mann-Whitney effects (relative marginal effects)"
}

if (hypothesis == "H0F") {

  Sigma <- .nparld_estimate_sigma(
    dat = dat,
    WP.names = design$WP.names,
    SP.names = design$SP.names,
    N = design$N,
    Ntotal = Ntotal,
    replicate = replicate,
    cell.weights = cell.weights
  )

  effects$std.error <- sqrt(diag(Sigma))
  text.hypotheses <- "Tested in distribution functions"
}

  #*******************************************************************************
  # Compute the test statistics
  #*******************************************************************************
  tests <- .nparld_run_tests(
    theta_hat,
    Sigma,
    hy$Hypotheses,
    design$N,
    hy$Output.names
  )

if (!is.null(contrast)) {
  pos <- which(hy$Output.names == contrast[[1]])

  if (length(contrast) == 1L) {
    H <- hy$Hypotheses[[pos]]
    C <- diag(nrow(H))
    rownames(C) <- rownames(H)
  } else {
    H <- hy$CI.Matrices[[pos]]
    C <- .nparld_build_contrast_matrix(contrast, H)
  }

  res.mctp <- .nparld_run_contrasts(
    theta_hat = theta_hat,
    Sigma = Sigma,
    H = H,
    C = C,
    N = design$N,
    alpha = alpha,
    sci.method = sci.method,
    hypothesis = hypothesis,
    df_satt = df_satt,
    eps_theta = if (hypothesis == "H0p") covp$eps_theta else NULL,
    factor_name = contrast[[1]]
  )
}

  #*******************************************************************************
  # Factor information
  #*******************************************************************************
  factor.info <- NULL

  if (Factor.Information) {
    factor.info <- .nparld_run_factor.information(
      theta_hat = theta_hat,
      Sigma = Sigma,
      CIMatrices = hy$CI.Matrices,
      Output.names = hy$Output.names,
      names.levels = design$names.levels,
      alpha = alpha,
      N = design$N,
      CI.method = CI.method,
      include.CI = (hypothesis == "H0p")
    )
  }

  #*****************************************************************************
  # Return the Covariance Matrix estimator
  #*****************************************************************************

covariance.info <- NULL


if (covariance) {
  covariance.info <- Sigma
  class(covariance.info) <- c("nparld_covarianceinfo", class(Sigma))
}
  

  #*******************************************************************************
  # Two time points
  #*******************************************************************************
  perm_res <- list(available = FALSE)

  if (perm.test) {
    design_obj <- .nparld_build_design(formula, data, subject, replicate)

    if (.can_use_perm_test(design_obj)) {
      samp <- .extract_paired_samples(design_obj)

      res_perm_raw <- studentized_perm_test_paired(
        samp$x, samp$y,
        B = B,
        alpha = alpha,
        hypothesis = hypothesis
      )

      table_out <- .format_perm_output(
        res = res_perm_raw,
        alpha = alpha,
        hypothesis = hypothesis
      )

      perm_res <- list(
        available = TRUE,
        method = "Studentized permutation test (paired)",
        table = table_out,
        raw = res_perm_raw,
        levels = samp$lev
      )

    } else {
      perm_res <- list(
        available = FALSE,
        message = "Permutation test not applicable (requires 1 subplot factor with 2 levels and complete data)."
      )
    }
  }

  #*******************************************************************************
  # Final result object
  #*******************************************************************************
  result <- list(
    Design = design$Design,
    wholeplots = design$wholeplots,
    subplots = design$subplots,
     text.ranks = text.ranks,
      text.hypotheses = text.hypotheses,
    N.info = design$N.info,
    effects = effects,
    factor.info = factor.info,
    WTS = tests$WTS,
    ATS = tests$ATS,
    MCTP = res.mctp,
    perm = perm_res,
    hypothesis = hypothesis,
    CI.method = CI.method,
    covariance.info = covariance.info,
    .internal = list(
      formula = formula,
      WP.names = design$WP.names,
      SP.names = design$SP.names,
      factor_cols = design$factor_cols,
      sci.method = sci.method,
      n.levels = design$n.levels,
      names.levels = design$names.levels,
      N = design$N,
      Sigma = Sigma,
      Hypotheses = hy$Hypotheses,
      CI.Matrices = hy$CI.Matrices,
      Output.names = hy$Output.names,
      effect = effect,
      cell.weights = cell.weights,
      hypothesis = hypothesis,
      CI.method = CI.method,
      alpha = alpha,
      Factor.Information = Factor.Information
    )
  )

  class(result) <- "nparld_fit"
  result
}

test_that("nparLD runs for unbalanced group-by-time design with H0p and contrast", {
  set.seed(123)

  T <- 4
  n <- c(10, 20, 30)
  N <- sum(T * n)

  y <- rnorm(N)
  times <- c(rep(1:T, n[1]), rep(1:T, n[2]), rep(1:T, n[3]))
  grp <- c(rep(1, n[1] * T), rep(2, n[2] * T), rep(3, n[3] * T))
  subjekte <- c(
    sort(rep(1:n[1], T)),
    sort(rep((n[1] + 1):(n[1] + n[2]), T)),
    sort(rep((n[1] + n[2] + 1):(n[1] + n[2] + n[3]), T))
  )

  datensatz <- data.frame(times, grp, subjekte, y)

  fit <- nparLD(
    y ~ grp * times,
    data = datensatz,
    subject = "subjekte",
    hypothesis = "H0p",
    contrast = list("grp:times")
  )

  expect_s3_class(fit, "nparld_fit")
  expect_true("effects" %in% names(fit))
  expect_true("WTS" %in% names(fit))
  expect_true("ATS" %in% names(fit))
})


test_that("nparLD runs for unbalanced group-by-time design with missing values and H0F", {
  set.seed(123)

  T <- 4
  n <- c(10, 20, 30)
  N <- sum(T * n)

  y <- rnorm(N)
  times <- c(rep(1:T, n[1]), rep(1:T, n[2]), rep(1:T, n[3]))
  grp <- c(rep(1, n[1] * T), rep(2, n[2] * T), rep(3, n[3] * T))
  subjekte <- c(
    sort(rep(1:n[1], T)),
    sort(rep((n[1] + 1):(n[1] + n[2]), T)),
    sort(rep((n[1] + n[2] + 1):(n[1] + n[2] + n[3]), T))
  )

  y[c(1:2, 17, 28:30)] <- NA

  datensatz <- data.frame(times, grp, subjekte, y)

  fit <- nparLD(
    y ~ grp * times,
    data = datensatz,
    subject = "subjekte",
    hypothesis = "H0F",
    effect = "unweighted"
  )

  expect_s3_class(fit, "nparld_fit")
  expect_true("WTS" %in% names(fit))
  expect_true("ATS" %in% names(fit))
})


test_that("nparLD runs for balanced group-by-time design with replicates", {
  set.seed(123)

  T <- 4
  n <- c(10, 10, 10)
  N <- sum(T * n)

  y <- rnorm(N)
  times <- c(rep(1:T, n[1]), rep(1:T, n[2]), rep(1:T, n[3]))
  grp <- c(rep(1, n[1] * T), rep(2, n[2] * T), rep(3, n[3] * T))
  subjekte <- c(
    sort(rep(1:n[1], T)),
    sort(rep((n[1] + 1):(n[1] + n[2]), T)),
    sort(rep((n[1] + n[2] + 1):(n[1] + n[2] + n[3]), T))
  )

  datensatz <- data.frame(times, grp, subjekte, y)

  datensatz$rep <- rep(1:3, length.out = nrow(datensatz))

  fit1 <- nparLD(
    y ~ grp * times,
    data = datensatz,
    subject = "subjekte",
    replicate = "rep"
  )

  fit2 <- nparLD(
    y ~ grp * times,
    data = datensatz,
    subject = "subjekte",
    replicate = "rep",
    hypothesis = "H0p",
    contrast = list("grp:times")
  )

  expect_s3_class(fit1, "nparld_fit")
  expect_s3_class(fit2, "nparld_fit")
  expect_true("effects" %in% names(fit1))
  expect_true("effects" %in% names(fit2))
})


test_that("nparLD runs for balanced group-by-time design with missing values", {
  set.seed(123)

  T <- 4
  n <- c(10, 10, 10)
  N <- sum(T * n)

  y <- rnorm(N)
  times <- c(rep(1:T, n[1]), rep(1:T, n[2]), rep(1:T, n[3]))
  grp <- c(rep(1, n[1] * T), rep(2, n[2] * T), rep(3, n[3] * T))
  subjekte <- c(
    sort(rep(1:n[1], T)),
    sort(rep((n[1] + 1):(n[1] + n[2]), T)),
    sort(rep((n[1] + n[2] + 1):(n[1] + n[2] + n[3]), T))
  )

  datensatz <- data.frame(times, grp, subjekte, y)
  datensatz$y[1:3] <- NA

  fit <- nparLD(
    y ~ times * grp,
    data = datensatz,
    subject = "subjekte"
  )

  expect_s3_class(fit, "nparld_fit")
  expect_true("effects" %in% names(fit))
})


test_that("plot method returns a plot object for nparLD fit", {
  set.seed(123)

  T <- 4
  n <- c(10, 10, 10)
  N <- sum(T * n)

  y <- rnorm(N)
  times <- c(rep(1:T, n[1]), rep(1:T, n[2]), rep(1:T, n[3]))
  grp <- c(rep(1, n[1] * T), rep(2, n[2] * T), rep(3, n[3] * T))
  subjekte <- c(
    sort(rep(1:n[1], T)),
    sort(rep((n[1] + 1):(n[1] + n[2]), T)),
    sort(rep((n[1] + n[2] + 1):(n[1] + n[2] + n[3]), T))
  )

  datensatz <- data.frame(times, grp, subjekte, y)
  datensatz$y[1:3] <- NA

  fit <- nparLD(
    y ~ times * grp,
    data = datensatz,
    subject = "subjekte"
  )

  p <- plot(fit)

  expect_s3_class(fit, "nparld_fit")
  expect_s3_class(p, "ggplot")
})


test_that("nparLD runs for factorial design with two whole-plot and two subplot factors", {
  set.seed(123)

  dat <- data.frame()
  n_subjects <- 10
  subject_id <- 1

  for (a in c("A1", "A2")) {
    for (b in c("B1", "B2")) {
      for (subject in 1:n_subjects) {
        subject_effect <- rnorm(1, 0, 1.5)

        for (c in c("C1", "C2")) {
          for (d in c("D1", "D2")) {
            base_response <- 50
            a_effect <- ifelse(a == "A2", 5, 0)
            b_effect <- ifelse(b == "B2", 3, 0)
            c_effect <- ifelse(c == "C2", 4, 0)
            d_effect <- ifelse(d == "D2", 2, 0)

            response <- base_response +
              a_effect + b_effect + c_effect + d_effect +
              subject_effect + rnorm(1, 0, 0.5)

            dat <- rbind(
              dat,
              data.frame(
                ID = subject_id,
                WholePlot = paste0("WP_", a, "_", b),
                A = a,
                B = b,
                C = c,
                D = d,
                Response = round(response, 2)
              )
            )
          }
        }

        subject_id <- subject_id + 1
      }
    }
  }

  fit <- nparLD(
    Response ~ B * C * D,
    data = dat,
    subject = "ID"
  )

  expect_s3_class(fit, "nparld_fit")
  expect_true("effects" %in% names(fit))
  expect_true("WTS" %in% names(fit))
  expect_true("ATS" %in% names(fit))
})


test_that("nparLD runs for factorial design with missing values", {
  set.seed(123)

  dat <- data.frame()
  n_subjects <- 10
  subject_id <- 1

  for (a in c("A1", "A2")) {
    for (b in c("B1", "B2")) {
      for (subject in 1:n_subjects) {
        subject_effect <- rnorm(1, 0, 1.5)

        for (c in c("C1", "C2")) {
          for (d in c("D1", "D2")) {
            base_response <- 50
            a_effect <- ifelse(a == "A2", 5, 0)
            b_effect <- ifelse(b == "B2", 3, 0)
            c_effect <- ifelse(c == "C2", 4, 0)
            d_effect <- ifelse(d == "D2", 2, 0)

            response <- base_response +
              a_effect + b_effect + c_effect + d_effect +
              subject_effect + rnorm(1, 0, 0.5)

            dat <- rbind(
              dat,
              data.frame(
                ID = subject_id,
                WholePlot = paste0("WP_", a, "_", b),
                A = a,
                B = b,
                C = c,
                D = d,
                Response = round(response, 2)
              )
            )
          }
        }

        subject_id <- subject_id + 1
      }
    }
  }

  dat$Response[c(23, 51, 100)] <- NA

  fit <- nparLD(
    Response ~ B * C * D,
    data = dat,
    subject = "ID"
  )

  expect_s3_class(fit, "nparld_fit")
  expect_true("effects" %in% names(fit))
})



test_that("nparLD runs with true dependent replicate observations", {
  set.seed(123)

  T <- 4
  n <- c(10, 10, 10)
  N <- sum(T * n)

  y <- rnorm(N)
  times <- c(rep(1:T, n[1]), rep(1:T, n[2]), rep(1:T, n[3]))
  grp <- c(rep(1, n[1] * T), rep(2, n[2] * T), rep(3, n[3] * T))
  subjekte <- c(
    sort(rep(1:n[1], T)),
    sort(rep((n[1] + 1):(n[1] + n[2]), T)),
    sort(rep((n[1] + n[2] + 1):(n[1] + n[2] + n[3]), T))
  )

  datensatz <- data.frame(times, grp, subjekte, y)

  datensatz2 <- datensatz[rep(seq_len(nrow(datensatz)), each = 3), ]
  datensatz2$rep <- rep(1:3, times = nrow(datensatz))
  datensatz2$y <- rnorm(nrow(datensatz2))

  fit_subjects <- nparLD(
    y ~ grp * times,
    data = datensatz2,
    subject = "subjekte",
    replicate = "rep",
    hypothesis = "H0p",
    cell.weights = "subjects",
    contrast = list("grp:times")
  )

  fit_observations <- nparLD(
    y ~ grp * times,
    data = datensatz2,
    subject = "subjekte",
    replicate = "rep",
    hypothesis = "H0p",
    cell.weights = "observations",
    contrast = list("grp:times")
  )

  expect_s3_class(fit_subjects, "nparld_fit")
  expect_s3_class(fit_observations, "nparld_fit")

  expect_true("effects" %in% names(fit_subjects))
  expect_true("effects" %in% names(fit_observations))


  expect_equal(nrow(fit_subjects$effects), nrow(fit_observations$effects))
})

test_that("brdu replicate data can be analyzed", {
  fit <- nparLD(
    resp ~ dose,
    data = brdu,
    subject = "culture",
    replicate = "replicate",
    hypothesis = "H0p",
    cell.weights = "subjects"
  )

  expect_s3_class(fit, "nparld_fit")
  expect_true("effects" %in% names(fit))
  expect_true(nrow(fit$effects) > 0)
})

test_that("user-defined contrast matrix is accepted", {
  C <- rbind(
    "2 - 1" = c(-1, 1, 0, 0),
    "3 - 1" = c(-1, 0, 1, 0),
    "4 - 1" = c(-1, 0, 0, 1)
  )

  fit <- nparLD(
    resp ~ time,
    data = dental,
    subject = "subject",
    hypothesis = "H0p",
    contrast = list("time", C)
  )

  expect_equal(nrow(fit$MCTP$Local.Results), 3)
  expect_equal(nrow(fit$MCTP$Contrast.Matrix), 3)
})

test_that("MCTP print supports show.matrix", {
  fit <- nparLD(
    resp ~ time,
    data = dental,
    subject = "subject",
    hypothesis = "H0p",
    contrast = list("time", "Dunnett")
  )

  expect_output(print(fit$MCTP, show.matrix = TRUE), "Contrast matrix")
})

test_that("factor-information plots can be requested by term", {
  fit <- nparLD(
    resp ~ group1 * group2 * time,
    data = shoulder,
    subject = "subject",
    hypothesis = "H0p",
    Factor.Information = TRUE
  )

  expect_s3_class(plot(fit, term = "time"), "ggplot")
  expect_s3_class(plot(fit, term = "group1:time"), "ggplot")
})

test_that("H0F supports observation-weighted dependent replicates", {

  fit <- nparLD(
    resp ~ dose,
    data = brdu,
    subject = "culture",
    replicate = "replicate",
    hypothesis = "H0F",
    cell.weights = "observations"
  )

  expect_s3_class(fit, "nparld_fit")
  expect_equal(fit$hypothesis, "H0F")
  expect_false(is.null(fit$WTS))
  expect_false(is.null(fit$ATS))
})

test_that("H0F replicate subject and observation weights both run", {

  fit_subjects <- nparLD(
    resp ~ dose,
    data = brdu,
    subject = "culture",
    replicate = "replicate",
    hypothesis = "H0F",
    cell.weights = "subjects"
  )

  fit_observations <- nparLD(
    resp ~ dose,
    data = brdu,
    subject = "culture",
    replicate = "replicate",
    hypothesis = "H0F",
    cell.weights = "observations"
  )

  expect_s3_class(fit_subjects, "nparld_fit")
  expect_s3_class(fit_observations, "nparld_fit")

  expect_false(is.null(fit_subjects$WTS))
  expect_false(is.null(fit_observations$WTS))
})

test_that("factor-information plots can be requested by term", {
  fit <- nparLD(
    resp ~ group1 * group2 * time,
    data = shoulder,
    subject = "subject",
    hypothesis = "H0p",
    Factor.Information = TRUE
  )

  p1 <- plot(fit, term = "time")
  p2 <- plot(fit, term = c("time", "group1:time"))

  expect_s3_class(p1, "ggplot")
  expect_s3_class(p2, "ggplot")
})

test_that("factor-information plot requires Factor.Information", {
  fit <- nparLD(
    resp ~ group1 * group2 * time,
    data = shoulder,
    subject = "subject",
    hypothesis = "H0p"
  )

  expect_error(
    plot(fit, term = "time"),
    "Factor.Information = TRUE"
  )
})

test_that("multiplication works", {
  expect_equal(2 * 2, 4)
})

test_that("nparLD runs for simple one-factor longitudinal design", {
  set.seed(123)
  dat <- data.frame(
    subject = rep(1:8, each = 3),
    time = rep(1:3, times = 8),
    y = rnorm(24)
  )

  fit <- nparLD(y ~ time, data = dat, subject = "subject", hypothesis = "H0p")

  expect_s3_class(fit, "nparld_fit")
  expect_true("effects" %in% names(fit))
  expect_true("WTS" %in% names(fit))
  expect_true("ATS" %in% names(fit))
})



test_that("nparLD recognizes replicate variable", {
  dat <- data.frame(
    CultureNo = rep(1:5, each = 12),
    Replic = rep(rep(1:3, times = 4), times = 5),
    Dose = rep(rep(c(0, 0.1, 1, 10), each = 3), times = 5),
    Brdu = rnorm(60)
  )

  fit <- nparLD(
    Brdu ~ Dose,
    data = dat,
    subject = "CultureNo",
    replicate = "Replic",
    hypothesis = "H0p"
  )

  expect_s3_class(fit, "nparld_fit")
})


test_that("nparLD recognizes replicate variable", {
  dat <- data.frame(
    CultureNo = rep(1:5, each = 12),
    Replic = rep(rep(1:3, times = 4), times = 5),
    Dose = rep(rep(c(0, 0.1, 1, 10), each = 3), times = 5),
    Brdu = rnorm(60)
  )

  fit <- nparLD(
    Brdu ~ Dose,
    data = dat,
    subject = "CultureNo",
    replicate = "Replic",
    hypothesis = "H0p"
  )

  expect_s3_class(fit, "nparld_fit")
})

################################################################################
# Functions for general computations
################################################################################


.nparld_logit <- function(x){log(x/(1-x))}
.nparld_expit <- function(y){exp(y)/(1+exp(y))}

.nparld_limits <- function(p,V,alpha,N,CI.method){

  switch(CI.method,
                   normal={
  lower <- p - qnorm(1-alpha/2)/sqrt(N)*sqrt(c(diag(V)))
  upper <- p + qnorm(1-alpha/2)/sqrt(N)*sqrt(c(diag(V)))
               },
  logit = {Psi <- diag(1/(p*(1-p)))
  VLogit <- Psi%*%V%*%t(Psi)
  lower <- .nparld_expit(.nparld_logit(p)- qnorm(1-alpha/2)/sqrt(N)*sqrt(c(diag(VLogit))))
  upper <- .nparld_expit(.nparld_logit(p)+ qnorm(1-alpha/2)/sqrt(N)*sqrt(c(diag(VLogit))))}
  )

  res=cbind(lower=lower,upper=upper)
  res
}


.nparld_rankH <- function(A) {
  if (!requireNamespace("MASS", quietly = TRUE)) {
    stop("Package 'MASS' is required for ginv().")
  }
  sum(diag(MASS::ginv(A) %*% A))
}

.nparld_wald <- function(M, H, V, N) {
  if (!requireNamespace("MASS", quietly = TRUE)) {
    stop("Package 'MASS' is required for ginv().")
  }
  WTS <- N * t(H %*% M) %*% MASS::ginv(H %*% V %*% t(H)) %*% H %*% M
  dfWTS <- .nparld_rankH(H %*% V %*% t(H))
  pv.WTS <- 1 - pchisq(WTS, dfWTS)
  c(Statistic = as.numeric(WTS), df = as.numeric(dfWTS), `p-value` = as.numeric(pv.WTS))
}

.nparld_ats <- function(M, H, V, N) {
  if (!requireNamespace("MASS", quietly = TRUE)) {
    stop("Package 'MASS' is required for ginv().")
  }
  C <- t(H) %*% MASS::ginv(H %*% t(H)) %*% H
  spur <- sum(diag(C %*% V))
  ATS <- N / spur * t(M) %*% C %*% M
  df_ATS1 <- spur^2 / sum(diag(C %*% V %*% C %*% V))
  pv.ATS <- 1 - pf(ATS, df_ATS1, 1e10)
  c(Statistic = as.numeric(ATS), df = as.numeric(df_ATS1), `p-value` = as.numeric(pv.ATS))
}

.nparld_run_tests <- function(theta_hat, Sigma, Hypotheses, N, output_names) {
  n.hypotheses <- length(Hypotheses)

  WTS <- matrix(0, n.hypotheses, 3)
  ATS <- matrix(0, n.hypotheses, 3)

  for (i in seq_len(n.hypotheses)) {
    WTS[i, ] <- .nparld_wald(theta_hat, Hypotheses[[i]], Sigma, N)
    ATS[i, ] <- .nparld_ats(theta_hat, Hypotheses[[i]], Sigma, N)
  }

  rownames(WTS) <- output_names
  rownames(ATS) <- output_names
  colnames(WTS) <- colnames(ATS) <- c("Statistic", "df", "p-value")

  list(WTS = WTS, ATS = ATS)
}





.nparld_run_factor.information <- function(theta_hat,
                                           Sigma,
                                           CIMatrices,
                                           Output.names,
                                           names.levels,
                                           alpha,
                                           N,
                                           CI.method,
                                           include.CI = FALSE) {

  n.factors <- length(CIMatrices)
  descriptives <- vector("list", n.factors)

  for (i in seq_len(n.factors)) {

    # effect estimates
    est <- as.vector(CIMatrices[[i]] %*% theta_hat)

    # covariance of estimates
    V <- CIMatrices[[i]] %*% Sigma %*% t(CIMatrices[[i]])

    se <- sqrt(diag(V))

    df <- data.frame(
      Rel.Effect = est,
      Std.Error  = se
    )

    # compute CIs if requested
    if (include.CI) {

      CI <- .nparld_limits(
        p = est,
        V = V,
        alpha = alpha,
        N = N,
        CI.method = CI.method
      )

      df$Lower <- CI[, 1]
      df$Upper <- CI[, 2]
    }
    # attach factor-level names when possible
    rn <- NULL
    fac_name <- Output.names[i]

    if (!is.null(fac_name)) {

      if (!is.null(names.levels[[fac_name]])) {
        rn <- names.levels[[fac_name]]
      } else if (grepl(":", fac_name, fixed = TRUE)) {
        facs <- strsplit(fac_name, ":", fixed = TRUE)[[1]]

        if (all(facs %in% names(names.levels))) {
          levs <- names.levels[facs]
          rn <- do.call(
            interaction,
            c(
              expand.grid(levs, KEEP.OUT.ATTRS = FALSE, stringsAsFactors = FALSE),
              list(sep = ":")
            )
          )
          rn <- as.character(rn)
        }
      }
    }

    if (!is.null(rn) && length(rn) == nrow(df)) {
      rownames(df) <- rn
    } else {
      rownames(df) <- seq_len(nrow(df))
    }

    descriptives[[i]] <- df
  }

  names(descriptives) <- Output.names
  class(descriptives) <- "nparld_factorinfo"

  descriptives
}




#*******************************************************************************
# Function for Hypotheses Matrices

HC <- function(fl, art = c("Hyp", "CI"), perm_names, names) {

  art <- match.arg(art)
  nf <- length(fl)

  if (art == "Hyp") {
    P <- function(x) {
      diag(x) - matrix(1 / x, ncol = x, nrow = x)
    }
  } else {
    P <- function(x) {
      diag(x)
    }
  }

  One <- function(x) {
    matrix(1 / x, nrow = 1, ncol = x)
  }

  kp <- function(A) {
    out <- A[[1]]
    if (length(A) > 1L) {
      for (i in 2:length(A)) {
        out <- out %x% A[[i]]
      }
    }
    out
  }

  if (is.null(dim(perm_names))) {
    perm_names <- matrix(perm_names, nrow = 1)
  }

  perm_names <- as.matrix(perm_names)

  fac_names <- apply(perm_names, 1, function(x) {
    paste(names[which(x == 1)], collapse = ":")
  })

  hypo <- vector("list", nrow(perm_names))

  for (i in seq_len(nrow(perm_names))) {

    mats <- vector("list", nf)

    for (j in seq_len(nf)) {
      if (perm_names[i, j] == 1) {
        mats[[j]] <- P(fl[j])
      } else {
        mats[[j]] <- One(fl[j])
      }
    }

    hypo[[i]] <- kp(mats)
  }

  list(Matrix = hypo, Namen = fac_names)
}


.nparld_build_hypotheses <- function(formula, factor_cols, n.levels) {
  if (!exists("HC", mode = "function")) {
    stop("Function 'HC' not found. Make sure HC() is available in the package namespace.")
  }

  response_name <- all.vars(formula)[1]
  if (length(factor_cols) == 0) {
    stop("No factors found in the model (besides subject/response). Cannot build hypotheses.")
  }
  formula.new <- as.formula(paste(response_name, "~", paste(factor_cols, collapse = "*")))

  tf <- attr(terms(formula.new), "factors")
  if (nrow(tf) < 2) stop("Unexpected terms() structure; cannot extract factors for hypotheses.")

  fac_names <- rownames(tf)[-1]
  if (is.null(fac_names) || any(fac_names == "")) fac_names <- factor_cols

  perm_names <- t(tf[-1, , drop = FALSE])  # terms x factors (0/1)

  # Align perm_names columns to factor_cols if possible
  if (!is.null(colnames(perm_names)) && all(factor_cols %in% colnames(perm_names))) {
    perm_names <- perm_names[, factor_cols, drop = FALSE]
    fac_names <- factor_cols
  } else {
    fac_names <- factor_cols
  }

  Hyp <- HC(n.levels, "Hyp", perm_names, fac_names)
  CI  <- HC(n.levels, "CI",  perm_names, fac_names)

  list(
    Hypotheses   = Hyp$Matrix,
    CI.Matrices  = CI$Matrix,
    Output.names = Hyp$Namen,
    perm_names   = perm_names,
    fac_names    = fac_names
  )
}


# -----------------------------
# Helpers: theta (rank means) + effects table
.nparld_estimate_theta_rankmeans <- function(dat, formula, WP.names, SP.names,
                                             replicate = NULL,
                                             effect = c("unweighted", "weighted"),
                                             cell.weights = c("subjects", "observations")) {

  effect <- match.arg(effect)
  cell.weights <- match.arg(cell.weights)

  if ("PseudoRank" %in% names(dat)) {
    score_var <- "PseudoRank"
  } else {
    score_var <- "Ranks"
  }

  if (!(score_var %in% names(dat))) {
    dat[[score_var]] <- rank(dat[[1]], na.last = "keep")
  }

  if (!("subject" %in% names(dat))) {
    stop("Internal column 'subject' not found in data.", call. = FALSE)
  }

  factor_cols <- c(WP.names, SP.names)

  # canonical cell grid
  if (length(factor_cols) == 0L) {
    effects <- data.frame(dummy = 1)
    factor_cols <- character(0)
    cell_id <- rep("1", nrow(dat))
    cell_levels <- "1"
  } else {
    grid <- do.call(
      expand.grid,
      c(
        lapply(rev(factor_cols), function(v) levels(dat[[v]])),
        KEEP.OUT.ATTRS = FALSE,
        stringsAsFactors = FALSE
      )
    )
    names(grid) <- rev(factor_cols)
    grid <- grid[factor_cols]
    effects <- grid

    cell_id <- interaction(dat[, factor_cols, drop = FALSE], drop = TRUE, lex.order = TRUE)
    cell_levels <- interaction(grid[, factor_cols, drop = FALSE], drop = TRUE, lex.order = TRUE)
  }

  # number of subjects assigned to the cell
  n_subj <- tapply(dat$subject, cell_id, function(x) length(unique(x)))

  # number of subjects with at least one observed value in the cell
  lambda_cell <- tapply(seq_len(nrow(dat)), cell_id, function(idx) {
    df <- dat[idx, , drop = FALSE]
    by_subj <- split(df[[1]], df$subject)
    sum(vapply(by_subj, function(x) any(!is.na(x)), logical(1)))
  })

  # number of observed / missing responses per cell
  n_obs  <- tapply(!is.na(dat[[1]]), cell_id, sum)
  n_miss <- tapply(is.na(dat[[1]]),  cell_id, sum)

  effects$Nsubj <- 0L
  effects$lambda <- 0L
  effects$Nobs  <- 0L
  effects$Nmiss <- 0L

  pos_subj   <- match(names(n_subj), cell_levels)
  pos_lambda <- match(names(lambda_cell), cell_levels)
  pos_obs    <- match(names(n_obs), cell_levels)
  pos_miss   <- match(names(n_miss), cell_levels)

  if (length(pos_subj)   > 0) effects$Nsubj[pos_subj]     <- as.integer(n_subj)
  if (length(pos_lambda) > 0) effects$lambda[pos_lambda]  <- as.integer(lambda_cell)
  if (length(pos_obs)    > 0) effects$Nobs[pos_obs]       <- as.integer(n_obs)
  if (length(pos_miss)   > 0) effects$Nmiss[pos_miss]     <- as.integer(n_miss)

  # total number of observed responses
  Ntotal <- sum(effects$Nobs)

  # ---------------------------------------------------------------------------
  # cell score means
  # ---------------------------------------------------------------------------
  if (is.null(replicate) || cell.weights == "observations") {

    # no replicates OR observation-weighted within cell
    score_means <- tapply(dat[[score_var]], cell_id, function(x) {
      x <- x[!is.na(x)]
      if (length(x) == 0L) return(NA_real_)
      mean(x)
    })

  } else {

    # replicates present + subjects: equal subject weighting within cell
    if (!(replicate %in% names(dat))) {
      stop("Replicate variable '", replicate, "' not found in data.", call. = FALSE)
    }

    group_cols <- c("subject", factor_cols)

    subj_means <- aggregate(
      dat[[score_var]],
      by = dat[group_cols],
      FUN = function(x) {
        x <- x[!is.na(x)]
        if (length(x) == 0L) return(NA_real_)
        mean(x)
      }
    )
    colnames(subj_means)[ncol(subj_means)] <- "subj_score_mean"

    subj_cell_id <- if (length(factor_cols) == 0L) {
      rep("1", nrow(subj_means))
    } else {
      interaction(subj_means[, factor_cols, drop = FALSE], drop = TRUE, lex.order = TRUE)
    }

    score_means <- tapply(subj_means$subj_score_mean, subj_cell_id, function(x) {
      x <- x[!is.na(x)]
      if (length(x) == 0L) return(NA_real_)
      mean(x)
    })
  }

  effects$ScoreMean <- NA_real_
  pos_score <- match(names(score_means), cell_levels)
  if (length(pos_score) > 0) {
    effects$ScoreMean[pos_score] <- as.numeric(score_means)
  }

  effects$RTE <- 1 / Ntotal * (effects$ScoreMean - 1 / 2)

  if (length(factor_cols) > 0) {
    effects <- effects[do.call(order, effects[factor_cols]), , drop = FALSE]
  }

  list(
    dat = dat,
    effects = effects,
    theta_hat = effects$RTE,
    Ntotal = Ntotal
  )
}

# -----------------------------
# Helpers: covariance estimation (your block diagonal construction)
# -----------------------------
.nparld_estimate_sigma <- function(dat, WP.names, SP.names, N, Ntotal,
                                   replicate = NULL,
                                   cell.weights = c("subjects", "observations")) {

  cell.weights <- match.arg(cell.weights)

  if (!requireNamespace("Matrix", quietly = TRUE)) {
    stop("Package 'Matrix' is required for covariance estimation (bdiag).")
  }

  if (!("Ranks" %in% names(dat)) && !("PseudoRank" %in% names(dat))) {
    stop("Neither Ranks nor PseudoRank found in data.")
  }

  score_var <- if ("PseudoRank" %in% names(dat)) "PseudoRank" else "Ranks"

  cell_keys <- .nparld_cell_keys(dat, WP.names, SP.names)

  # subplot keys in canonical order
  if (length(SP.names) == 0) {
    sp_keys <- "1"
  } else {
    sp_grid <- do.call(
      expand.grid,
      c(
        lapply(rev(SP.names), function(v) levels(dat[[v]])),
        KEEP.OUT.ATTRS = FALSE,
        stringsAsFactors = FALSE
      )
    )
    names(sp_grid) <- rev(SP.names)
    sp_grid <- sp_grid[SP.names]
    sp_keys <- .nparld_make_keys(sp_grid, SP.names)
  }

  # whole-plot keys in canonical order
  if (length(WP.names) == 0) {
    wp_keys <- "1"
  } else {
    wp_grid <- do.call(
      expand.grid,
      c(
        lapply(rev(WP.names), function(v) levels(dat[[v]])),
        KEEP.OUT.ATTRS = FALSE,
        stringsAsFactors = FALSE
      )
    )
    names(wp_grid) <- rev(WP.names)
    wp_grid <- wp_grid[WP.names]
    wp_keys <- .nparld_make_keys(wp_grid, WP.names)
  }

  cov_from_units <- function(YY) {
    lambdas <- !is.na(YY)
    lambda_i <- colSums(lambdas)

    Cov_block <- matrix(0, ncol(YY), ncol(YY))
    Rbar <- colSums(YY, na.rm = TRUE) / lambda_i

    for (s in seq_len(ncol(YY))) {
      for (ss in seq_len(ncol(YY))) {
        if (s == ss) {
          Cov_block[s, ss] <- sum((YY[, s] - Rbar[s])^2, na.rm = TRUE) /
            (lambda_i[s] * (lambda_i[s] - 1))
        } else {
          Cov_block[s, ss] <- sum(
            (YY[, s] - Rbar[s]) *
              (YY[, ss] - Rbar[ss]),
            na.rm = TRUE
          ) /
            ((lambda_i[s] - 1) * (lambda_i[ss] - 1) +
               sum(lambdas[, s] * lambdas[, ss]) - 1)
        }
      }
    }

    Cov_block
  }

  Cov.blocks <- vector("list", length(wp_keys))

  for (g in seq_along(wp_keys)) {

    wp_key <- wp_keys[g]

    if (length(WP.names) == 0) {
      groupdata <- dat
    } else {
      dat_wp_key <- .nparld_make_keys(dat, WP.names)
      groupdata <- dat[dat_wp_key == wp_key, , drop = FALSE]
    }

    # Independent units are SUBJECTS, not subject:replicate.
    groupdata$subject <- droplevels(groupdata$subject)
    subjdat <- split(groupdata, groupdata$subject)

    n_i <- length(subjdat)

    if (n_i < 2L) {
      stop("Too few subjects in whole-plot block.")
    }

    # Observation counts N_is for the observation-weighted replicate case.
    # These are the numbers of observed replicate measurements per subplot cell
    # within the current whole-plot group.
    if (!is.null(replicate) && cell.weights == "observations") {

      if (length(SP.names) == 0) {
        group_sp_keys <- rep("1", nrow(groupdata))
      } else {
        group_sp_keys <- .nparld_make_keys(groupdata, SP.names)
      }

      N_is <- tapply(
        !is.na(groupdata[[score_var]]),
        factor(group_sp_keys, levels = sp_keys),
        sum
      )

      N_is <- as.numeric(N_is)
      names(N_is) <- sp_keys

      if (any(is.na(N_is)) || any(N_is <= 0L)) {
        stop("A whole-plot by subplot cell contains no observed rank scores.",
             call. = FALSE)
      }
    }

    subj_vecs <- lapply(subjdat, function(df) {

      score <- df[[score_var]]

      if (length(SP.names) == 0) {
        subj_keys <- rep("1", nrow(df))
      } else {
        subj_keys <- .nparld_make_keys(df, SP.names)
      }

      yvec <- rep(NA_real_, length(sp_keys))
      names(yvec) <- sp_keys

      if (!is.null(replicate) && cell.weights == "observations") {

        ## H0F with dependent replicates and observation-level weighting:
        ##
        ##   Z_iks = sum_l R_iksl
        ##
        ## The subject-level contribution is represented on the mean-rank
        ## scale by n_i / N_is * Z_iks. Subjects without observations in a
        ## cell have Z_iks = 0. This centers correctly on the sum scale and
        ## yields
        ##
        ##   mean_k { n_i / N_is * Z_iks } = Rbar_is.
        ##
        ## The subtraction of 1/2 and division by Ntotal put the contribution
        ## on the relative-effect scale.

        cell_sums <- tapply(
          score,
          factor(subj_keys, levels = sp_keys),
          sum,
          na.rm = TRUE
        )

        cell_sums[is.na(cell_sums)] <- 0

        yvec[] <- 1 / Ntotal * ((n_i / N_is) * as.numeric(cell_sums) - 1 / 2)

      } else {

        ## Default H0F case, including dependent replicates with
        ## cell.weights = "subjects":
        ##
        ## each subject contributes the mean of its observed rank scores
        ## within the subject-condition cell.

        cell_means <- tapply(
          score,
          subj_keys,
          mean,
          na.rm = TRUE
        )

        if (!is.null(cell_means)) {
          pos <- match(names(cell_means), sp_keys)
          ok <- !is.na(pos)

          if (any(ok)) {
            yvec[pos[ok]] <- 1 / Ntotal * (cell_means[ok] - 1 / 2)
          }
        }
      }

      yvec
    })

    subj_vecs <- Filter(Negate(is.null), subj_vecs)

    if (length(subj_vecs) == 0) {
      stop("No valid observations found in this whole-plot block.")
    }

    YY_subj <- do.call(rbind, subj_vecs)
    YY_subj <- as.matrix(YY_subj)
    rownames(YY_subj) <- names(subj_vecs)
    colnames(YY_subj) <- sp_keys

    if (nrow(YY_subj) < 2) {
      stop("Too few subjects in whole-plot block.")
    }

    Cov.blocks[[g]] <- N * cov_from_units(YY_subj)

    rownames(Cov.blocks[[g]]) <- sp_keys
    colnames(Cov.blocks[[g]]) <- sp_keys
  }

  Sigma <- as.matrix(Matrix::bdiag(Cov.blocks))
  rownames(Sigma) <- cell_keys
  colnames(Sigma) <- cell_keys

  Sigma
}

.nparld_check_complete_within_schedule <- function(dat, SP.names, replicate = NULL) {
  if (length(SP.names) == 0) return(invisible(TRUE))

  if (!"subject" %in% names(dat)) {
    stop("Column 'subject' is required in 'dat'.", call. = FALSE)
  }

  # Missing design-factor values are not allowed here
  bad_sp <- SP.names[vapply(dat[SP.names], function(z) any(is.na(z)), logical(1))]
  if (length(bad_sp) > 0) {
    stop(
      "Missing values in subplot factor(s) are not allowed: ",
      paste(bad_sp, collapse = ", "),
      call. = FALSE
    )
  }

  # If there is no replicate variable, duplicates within subject are still suspicious
  if (is.null(replicate)) {
    within_key <- interaction(dat[, SP.names, drop = FALSE], drop = TRUE, lex.order = TRUE)
    dup <- ave(rep.int(1L, length(within_key)), dat$subject, within_key, FUN = sum)

    if (any(dup > 1L)) {
      bad <- dat[dup > 1L, c("subject", SP.names), drop = FALSE]
      bad <- unique(bad)
      stop(
        "Duplicate within-subject rows detected (same subplot combination occurs >1 time).\n",
        "Examples:\n",
        paste(utils::capture.output(print(utils::head(bad, 10))), collapse = "\n"),
        call. = FALSE
      )
    }
  }

  # Compare exact unique subplot schedule across subjects, ignoring replicate replication
  by_subj <- split(dat, dat$subject)

  schedule_signature <- function(df, cols) {
    sch <- unique(df[, cols, drop = FALSE])
    sch[] <- lapply(sch, as.character)
    sch <- sch[do.call(order, sch), , drop = FALSE]
    if (nrow(sch) == 0L) return("")
    paste(do.call(paste, c(sch, sep = "\r")), collapse = "\n")
  }

  sigs <- vapply(by_subj, schedule_signature, character(1), cols = SP.names)

  if (length(unique(sigs)) != 1L) {
    ref_subj <- names(sigs)[1]
    ref_sched <- unique(by_subj[[ref_subj]][, SP.names, drop = FALSE])
    ref_sched[] <- lapply(ref_sched, as.character)

    msg <- character(0)
    bad_subj <- names(sigs)[sigs != sigs[1]]

    for (s in utils::head(bad_subj, 5)) {
      cur_sched <- unique(by_subj[[s]][, SP.names, drop = FALSE])
      cur_sched[] <- lapply(cur_sched, as.character)

      ref_rows <- do.call(paste, c(ref_sched, sep = "\r"))
      cur_rows <- do.call(paste, c(cur_sched, sep = "\r"))

      missing <- setdiff(ref_rows, cur_rows)
      extra   <- setdiff(cur_rows, ref_rows)

      one <- paste0("Subject ", s, ":")
      if (length(missing) > 0) {
        one <- paste0(
          one, " missing ",
          paste(utils::head(missing, 10), collapse = ", "),
          if (length(missing) > 10) ", ..." else ""
        )
      }
      if (length(extra) > 0) {
        one <- paste0(
          one, if (length(missing) > 0) "; " else " ",
          "extra ",
          paste(utils::head(extra, 10), collapse = ", "),
          if (length(extra) > 10) ", ..." else ""
        )
      }
      msg <- c(msg, one)
    }

    stop(
      "Incomplete or inconsistent within-subject subplot schedule detected.\n",
      "Unequal replicate sizes are allowed, but subjects must share the same unique subplot combinations.\n",
      paste(msg, collapse = "\n"),
      call. = FALSE
    )
  }

  invisible(TRUE)
}

.check_replicate_structure <- function(dat, WP.names, SP.names, replicate = NULL) {
  factor_cols <- c(WP.names, SP.names)
  key_cols <- c("subject", factor_cols)

  if (length(factor_cols) == 0L) return(invisible(TRUE))

  cell_id <- interaction(dat[, key_cols, drop = FALSE], drop = TRUE, lex.order = TRUE)
  n_per_cell <- table(cell_id)

  has_reps <- any(n_per_cell > 1L)

  if (has_reps && is.null(replicate)) {
    bad_ids <- names(n_per_cell)[n_per_cell > 1L]
    bad <- dat[cell_id %in% utils::head(bad_ids, 10), c(key_cols), drop = FALSE]
    bad <- unique(bad)

    stop(
      "Repeated rows within subject-by-cell combinations detected, but no 'replicate' variable was specified.\n",
      "If these are intended replicates, please provide replicate = <column_name>.\n",
      "Examples:\n",
      paste(utils::capture.output(print(utils::head(bad, 10))), collapse = "\n"),
      call. = FALSE
    )
  }

  if (!is.null(replicate)) {
    if (!(replicate %in% names(dat))) {
      stop("Replicate variable '", replicate, "' not found in data.", call. = FALSE)
    }

    rep_key_cols <- c(key_cols, replicate)
    rep_id <- interaction(dat[, rep_key_cols, drop = FALSE], drop = TRUE, lex.order = TRUE)
    n_per_rep <- table(rep_id)

    if (any(n_per_rep > 1L)) {
      bad_ids <- names(n_per_rep)[n_per_rep > 1L]
      bad <- dat[rep_id %in% utils::head(bad_ids, 10), rep_key_cols, drop = FALSE]
      bad <- unique(bad)

      stop(
        "Duplicate replicate IDs detected within subject-by-cell combinations.\n",
        "Examples:\n",
        paste(utils::capture.output(print(utils::head(bad, 10))), collapse = "\n"),
        call. = FALSE
      )
    }
  }

  invisible(TRUE)
}

.nparld_cov_H0F_replicates_observations <- function(score.data,
                                                    subject,
                                                    wholeplot,
                                                    subplot,
                                                    score,
                                                    H = NULL,
                                                    Nstar) {

  ## H0F with dependent replicates and observation-level cell weighting.
  ##
  ## For each whole-plot group i and repeated-measures cell s:
  ##
  ##   Z_iks = sum_l R_iksl
  ##
  ## where R_iksl are the ordinary ranks or pseudo-ranks used for H0F.
  ## The covariance is computed over independent subjects. The centering is
  ## performed on the sum scale:
  ##
  ##   Z_iks - (N_is / n_i) * Rbar_is
  ##
  ## and the resulting covariance is transformed back to the mean-rank scale
  ## by diag(1 / N_is), and finally to the relative-effect scale by 1 / Nstar^2.

  if (!all(c(subject, wholeplot, subplot, score) %in% names(score.data))) {
    stop("Internal score data do not contain the required variables.",
         call. = FALSE)
  }

  dat <- score.data
  dat <- dat[!is.na(dat[[score]]), , drop = FALSE]

  wp.lev <- levels(factor(dat[[wholeplot]]))
  sp.lev <- levels(factor(dat[[subplot]]))

  blocks <- vector("list", length(wp.lev))
  names(blocks) <- wp.lev

  for (ii in seq_along(wp.lev)) {

    gi <- wp.lev[ii]
    di <- dat[dat[[wholeplot]] == gi, , drop = FALSE]

    subj.lev <- levels(factor(di[[subject]]))
    n_i <- length(subj.lev)

    if (n_i < 2L) {
      stop("At least two subjects are required in each whole-plot group.",
           call. = FALSE)
    }

    ## Z_i: rows = subjects, columns = subplot cells.
    ## Observation weighting means sum of rank scores per subject-cell.
    Z_i <- matrix(
      0,
      nrow = n_i,
      ncol = length(sp.lev),
      dimnames = list(subj.lev, sp.lev)
    )

    N_is <- numeric(length(sp.lev))
    names(N_is) <- sp.lev

    for (ss in seq_along(sp.lev)) {

      sj <- sp.lev[ss]
      dij <- di[di[[subplot]] == sj, , drop = FALSE]

      N_is[ss] <- nrow(dij)

      if (N_is[ss] == 0L) {
        stop("A whole-plot by subplot cell contains no observations.",
             call. = FALSE)
      }

      tmp <- tapply(
        dij[[score]],
        factor(dij[[subject]], levels = subj.lev),
        sum,
        na.rm = TRUE
      )

      tmp[is.na(tmp)] <- 0
      Z_i[, ss] <- as.numeric(tmp)
    }

    ## Cell mean ranks:
    ## Rbar_is = sum_k Z_iks / N_is
    Rbar_is <- colSums(Z_i) / N_is

    ## Center on the sum scale:
    ## center_is = (N_is / n_i) * Rbar_is
    center_i <- matrix(
      rep((N_is / n_i) * Rbar_is, each = n_i),
      nrow = n_i,
      ncol = length(sp.lev),
      byrow = FALSE
    )

    Zc_i <- Z_i - center_i

    ## Empirical covariance of subject-level sums.
    V_Z_i <- crossprod(Zc_i) / (n_i - 1)

    ## Transform from sum scale to cell mean-rank scale.
    D_i <- diag(1 / N_is, nrow = length(N_is))
    V_Rbar_i <- D_i %*% V_Z_i %*% D_i

    ## Transform from mean-rank scale to theta scale.
    blocks[[ii]] <- V_Rbar_i / (Nstar^2)
  }

  Sigma <- as.matrix(Matrix::bdiag(blocks))

  cell.names <- as.vector(
    outer(wp.lev, sp.lev, paste, sep = ":")
  )

  rownames(Sigma) <- colnames(Sigma) <- cell.names

  if (!is.null(H)) {
    Sigma <- H %*% Sigma %*% t(H)
  }

  Sigma
}

##############################################################################################
# Functions for testing H0p
#############################################################################################

.nparld_cfun <- function(x, y) (x > y) + 0.5 * (x == y)

.nparld_build_Ghat <- function(split_obj, Fhat,
                               effect = c("unweighted", "weighted"),
                               cell.weights = c("subjects", "observations")) {

  effect <- match.arg(effect)
  cell.weights <- match.arg(cell.weights)

  S <- split_obj$struct
  wp_levels <- split_obj$WP_levels
  sp_levels <- split_obj$SP_levels

  F_list <- list()
  w_list <- numeric(0)

  idx <- 1L

  for (wp in wp_levels) {
    subj_list <- S[[wp]]

    for (sp in sp_levels) {
      F_list[[idx]] <- Fhat[[wp]][[sp]]

      vals_by_subj <- lapply(subj_list, function(one_subj) {
        v <- one_subj[[sp]]
        v[!is.na(v)]
      })

      m_ijk <- vapply(vals_by_subj, length, integer(1))
      lambda_ij <- sum(m_ijk > 0L)
      m_ij <- sum(m_ijk)

      if (cell.weights == "subjects") {
        w_list[idx] <- lambda_ij
      } else {
        w_list[idx] <- m_ij
      }

      idx <- idx + 1L
    }
  }

  function(xvec) {
    vapply(xvec, function(x) {
      vals <- vapply(F_list, function(F) F(x), numeric(1))

      if (effect == "unweighted") {
        mean(vals, na.rm = TRUE)
      } else {
        ok <- !is.na(vals) & (w_list > 0)
        if (!any(ok)) {
          NA_real_
        } else {
          sum(w_list[ok] * vals[ok]) / sum(w_list[ok])
        }
      }
    }, numeric(1))
  }
}

.nparld_compute_pseudoranksN <- function(dat, Ghat, N) {

  y <- dat[[1]]  # response is first column in model.frame

  pseudo_rank <- N * Ghat(y)+1/2

  dat$PseudoRank <- pseudo_rank
  dat
}


.nparld_estimate_p_hat_pseudo <- function(dat, WP.names, SP.names) {

  factor_cols <- c(WP.names, SP.names)

  by_subj_cell <- aggregate(
    PseudoRank ~ subject + .,
    data = dat[, c("PseudoRank", "subject", factor_cols), drop = FALSE],
    FUN = mean
  )

  effects <- aggregate(
    PseudoRank ~ .,
    data = by_subj_cell[, c("PseudoRank", factor_cols), drop = FALSE],
    FUN = mean
  )

  # explizite Zielordnung
  target_keys <- .nparld_cell_keys(dat, WP.names, SP.names)
  effect_keys <- .nparld_make_keys(effects, factor_cols)

  pos <- match(target_keys, effect_keys)
  effects <- effects[pos, , drop = FALSE]

  ad <- nrow(effects)
  p_hat <- effects$PseudoRank / ad
  names(p_hat) <- target_keys

  list(
    p_hat = as.numeric(p_hat),
    effects = effects
  )
}


.nparld_Fcell_eval <- function(cell_subj_reps, x, drop_subject = NULL) {
  # cell_subj_reps: named list(subject -> numeric reps)
  if (!is.null(drop_subject) && drop_subject %in% names(cell_subj_reps)) {
    cell_subj_reps <- cell_subj_reps[names(cell_subj_reps) != drop_subject]
  }
  vals <- vapply(cell_subj_reps, function(reps) {
    reps <- reps[!is.na(reps)]
    if (length(reps) == 0) return(NA_real_)
    mean(.nparld_cfun(x, reps))
  }, numeric(1))
  mean(vals, na.rm = TRUE)
}

.nparld_Ghat_loo_eval <- function(all_cells, x, drop_subject = NULL) {
  # all_cells: named list(cell_name -> named list(subject -> reps))
  mean(vapply(all_cells, function(cell_subj_reps) {
    .nparld_Fcell_eval(cell_subj_reps, x, drop_subject = drop_subject)
  }, numeric(1)))
}

.nparld_pairwise_p_hat <- function(split_obj, Fhat, cell.weights = c("subjects", "observations")) {

  cell.weights <- match.arg(cell.weights)

  S <- split_obj$struct
  wp_levels <- split_obj$WP_levels
  sp_levels <- split_obj$SP_levels

  a <- length(wp_levels)
  d <- length(sp_levels)
  ad <- a * d

  cell_names <- as.vector(t(outer(wp_levels, sp_levels, paste, sep = ":")))

  p_pair <- matrix(
    NA_real_,
    nrow = ad,
    ncol = ad,
    dimnames = list(cell_names, cell_names)
  )

  F_list <- vector("list", ad)
  idx <- 1L
  for (i in wp_levels) {
    for (s in sp_levels) {
      F_list[[idx]] <- Fhat[[i]][[s]]
      idx <- idx + 1L
    }
  }

  col_idx <- 1L

  for (h in wp_levels) {
    subj_list <- S[[h]]

    for (t in sp_levels) {

      vals_ht_by_subj <- lapply(subj_list, function(one_subj) {
        v <- one_subj[[t]]
        v[!is.na(v)]
      })

      m_htk <- vapply(vals_ht_by_subj, length, integer(1))
      lambda_ht <- sum(m_htk > 0L)
      m_ht <- sum(m_htk)

      if (cell.weights == "observations") {
        if (m_ht == 0L) {
          p_pair[, col_idx] <- NA_real_
        } else {
          vals_ht <- unlist(vals_ht_by_subj, use.names = FALSE)
          p_pair[, col_idx] <- vapply(
            F_list,
            function(F_is) mean(F_is(vals_ht)),
            numeric(1)
          )
        }
      } else {  # subjects
        if (lambda_ht == 0L) {
          p_pair[, col_idx] <- NA_real_
        } else {
          p_pair[, col_idx] <- vapply(
            F_list,
            function(F_is) {
              subj_means <- vapply(vals_ht_by_subj, function(v) {
                if (length(v) == 0L) return(NA_real_)
                mean(F_is(v))
              }, numeric(1))
              mean(subj_means, na.rm = TRUE)
            },
            numeric(1)
          )
        }
      }

      col_idx <- col_idx + 1L
    }
  }

  list(
    p_pair = p_pair,
    cell_names = cell_names,
    a = a,
    d = d,
    ad = ad,
    wp_levels = wp_levels,
    sp_levels = sp_levels
  )
}

#************************************************************************************************
# Covariance Matrix Estimation with two different weights

#*******************Helper for Null Variances in H0theta****************************************#

.nparld_effective_cell_sizes <- function(split_obj,
                                         cell.weights = c("subjects", "observations")) {
  cell.weights <- match.arg(cell.weights)

  S <- split_obj$struct
  wp_levels <- names(S)
  sp_levels <- split_obj$SP_levels

  n_eff <- matrix(
    0,
    nrow = length(wp_levels),
    ncol = length(sp_levels),
    dimnames = list(wp_levels, sp_levels)
  )

  for (h in wp_levels) {
    subj_list <- S[[h]]

    for (t in sp_levels) {
      n_eff[h, t] <- sum(vapply(subj_list, function(one_subj) {
        vals <- one_subj[[t]]
        vals <- vals[!is.na(vals)]

        if (cell.weights == "subjects") {
          as.numeric(length(vals) > 0L)
        } else {
          length(vals)
        }
      }, numeric(1)))
    }
  }

  ## Same ordering as row_of()/col_of(): group first, then condition
  as.numeric(t(n_eff))
}

.nparld_theta_variance_floor <- function(split_obj,
                                         cell.weights = c("subjects", "observations"),
                                         cell_names = NULL) {
  cell.weights <- match.arg(cell.weights)

  n_eff <- .nparld_effective_cell_sizes(split_obj, cell.weights = cell.weights)

  eps <- outer(n_eff, n_eff, function(ni, nj) {
    ifelse(
      ni > 0 & nj > 0,
      1 / (4 * ni^2 * nj^2),
      NA_real_
    )
  })

  diag(eps) <- NA_real_

  if (!is.null(cell_names)) {
    dimnames(eps) <- list(cell_names, cell_names)
  }

  eps
}

.nparld_regularize_theta_variance <- function(var_theta, eps_theta, tol = 0) {
  zero <- !is.finite(var_theta) | var_theta <= tol
  use  <- zero & is.finite(eps_theta)

  var_theta[use] <- eps_theta[use]
  var_theta
}

#***********************************************************************************************************#
# First is nu=2 = WEIGHTED replicates

.nparld_cov_H0p_ref_unweighted_observations <- function(split_obj, Fhat, C = NULL) {
  S <- split_obj$struct
  wp_levels <- names(S)
  sp_levels <- split_obj$SP_levels
  a <- length(wp_levels)
  d <- length(sp_levels)
  ad <- a * d

  # Pairwise p_hat*(is,ht)
  pp <- .nparld_pairwise_p_hat(split_obj, Fhat, cell.weights = "observations")
  p_pair <- pp$p_pair
  cell_names <- pp$cell_names

  eps_theta <- .nparld_theta_variance_floor(
  split_obj,
  cell.weights = "observations",
  cell_names = cell_names
)

  # helper: map (group, time) -> position in p_pair / Sigma
  col_of <- function(h, t) (match(h, wp_levels) - 1L) * d + match(t, sp_levels)
  row_of <- function(i, s) (match(i, wp_levels) - 1L) * d + match(s, sp_levels)

  # group sizes n_h (subjects per group)
  n_h <- vapply(wp_levels, function(h) length(S[[h]]), integer(1))
  N <- sum(n_h)

  # final covariance accumulator
  Vhat_sum <- matrix(0, ad, ad)

  # optional Satterthwaite storage for contrasts
  if (!is.null(C)) {
    q <- nrow(C)
    var_contr_h <- matrix(0, nrow = q, ncol = length(wp_levels))
    colnames(var_contr_h) <- wp_levels
    h_index <- setNames(seq_along(wp_levels), wp_levels)
  }

  for (h in wp_levels) {
    subj_list <- S[[h]]
    nh <- length(subj_list)
    if (nh < 2L) {
      stop("Need at least 2 subjects in group ", h, " for covariance.", call. = FALSE)
    }

    # subject-by-component matrix of Psi-beta
    Z <- matrix(0, nrow = nh, ncol = ad)

    # total number of observed observations in group h, time t
    m_ht <- setNames(
      vapply(sp_levels, function(t) {
        sum(vapply(subj_list, function(one_subj) {
          sum(!is.na(one_subj[[t]]))
        }, numeric(1)))
      }, numeric(1)),
      sp_levels
    )

    for (k_idx in seq_along(subj_list)) {
      one_subj <- subj_list[[k_idx]]

      # observed values for subject k in group h
      X_htk <- setNames(lapply(sp_levels, function(t) {
        v <- one_subj[[t]]
        v[!is.na(v)]
      }), sp_levels)

      # subject-specific counts m_htk
      m_htk <- setNames(vapply(X_htk, length, integer(1)), sp_levels)

      Psi  <- numeric(ad)
      beta <- numeric(ad)

      for (i in wp_levels) for (s in sp_levels) {
        is_idx <- row_of(i, s)

        if (h != i) {
          # ------------------------------------------------------------
          # h != i:
          #  Psi_{hk}(is) = -(n_h/ad) * sum_t sum_u [ 1/m_ht * F_is(X_htku) ]
          #  beta_{hk}(is)= -(n_h/ad) * sum_t [ m_htk/m_ht * p_hat*(is,ht) ]
          # ------------------------------------------------------------
          tmpPsi  <- 0
          tmpBeta <- 0

          for (t in sp_levels) {
            if (m_ht[[t]] <= 0) next

            vals <- X_htk[[t]]
            if (length(vals) > 0) {
              for (u in seq_along(vals)) {
                tmpPsi <- tmpPsi + (1 / m_ht[[t]]) * Fhat[[i]][[s]](vals[u])
              }
            }

            ht_idx <- col_of(h, t)
            tmpBeta <- tmpBeta + (m_htk[[t]] / m_ht[[t]]) * p_pair[is_idx, ht_idx]
          }

          Psi[is_idx]  <- -(nh / ad) * tmpPsi
          beta[is_idx] <- -(nh / ad) * tmpBeta

        } else {
          # ------------------------------------------------------------
          # h == i:
          #
          # Psi_{ik}(is) = (n_i/ad) * {
          #   sum_{j!=i} sum_t sum_u [ 1/m_is * F_jt(X_isku) ]
          # + sum_t sum_u [ 1/m_is * F_it(X_isku) ]
          # - sum_t sum_u [ 1/m_it * F_is(X_itku) ]
          # }
          #
          # beta_{ik}(is) = (n_i/ad) * {
          #   sum_{j!=i} sum_t [ m_isk/m_is * p_hat*(jt,is) ]
          # + sum_t         [ m_isk/m_is * p_hat*(it,is) ]
          # - sum_t         [ m_itk/m_it * p_hat*(is,it) ]
          # }
          # ------------------------------------------------------------
          tmpPsi1  <- 0
          tmpBeta1 <- 0

          # terms involving observations from cell (i,s):
          if (m_ht[[s]] > 0 && m_htk[[s]] > 0) {
            vals_is <- X_htk[[s]]

            for (j in wp_levels) {
              if (j == i) next

              for (t in sp_levels) {
                if (length(vals_is) > 0) {
                  for (u in seq_along(vals_is)) {
                    tmpPsi1 <- tmpPsi1 + (1 / m_ht[[s]]) * Fhat[[j]][[t]](vals_is[u])
                  }
                }

                jt_idx <- row_of(j, t)
                is_col <- col_of(i, s)
                tmpBeta1 <- tmpBeta1 + (m_htk[[s]] / m_ht[[s]]) * p_pair[jt_idx, is_col]
              }
            }
          }

          tmpPsi2  <- 0
          tmpBeta2 <- 0

          for (t in sp_levels) {
            # + sum_u [1/m_is * F_it(X_isku)]
            if (m_ht[[s]] > 0 && m_htk[[s]] > 0) {
              vals_is <- X_htk[[s]]
              if (length(vals_is) > 0) {
                for (u in seq_along(vals_is)) {
                  tmpPsi2 <- tmpPsi2 + (1 / m_ht[[s]]) * Fhat[[i]][[t]](vals_is[u])
                }
              }

              it_idx <- row_of(i, t)
              is_col <- col_of(i, s)
              tmpBeta2 <- tmpBeta2 + (m_htk[[s]] / m_ht[[s]]) * p_pair[it_idx, is_col]
            }

            # - sum_u [1/m_it * F_is(X_itku)]
            if (m_ht[[t]] > 0 && m_htk[[t]] > 0) {
              vals_it <- X_htk[[t]]
              if (length(vals_it) > 0) {
                for (u in seq_along(vals_it)) {
                  tmpPsi2 <- tmpPsi2 - (1 / m_ht[[t]]) * Fhat[[i]][[s]](vals_it[u])
                }
              }

              is_row <- row_of(i, s)
              it_col <- col_of(i, t)
              tmpBeta2 <- tmpBeta2 - (m_htk[[t]] / m_ht[[t]]) * p_pair[is_row, it_col]
            }
          }

          Psi[is_idx]  <- (nh / ad) * (tmpPsi1 + tmpPsi2)
          beta[is_idx] <- (nh / ad) * (tmpBeta1 + tmpBeta2)
        }
      }

      Z[k_idx, ] <- Psi - beta
    }

    # optional contrast-specific variances for Satterthwaite df
    if (!is.null(C)) {
      U <- Z %*% t(C)
      U[!is.finite(U)] <- 0
      var_h <- apply(U, 2, function(x) var(x, na.rm = TRUE))
      var_contr_h[, h_index[h]] <- var_h
    }

    Vh <- crossprod(scale(Z, center = colMeans(Z), scale = FALSE)) / (nh - 1)
    Vhat_sum <- Vhat_sum + (N / nh) * Vh
  }

  # optional Satterthwaite df
  dfs <- NULL
  if (!is.null(C)) {
    df <- numeric(q)
    for (j in seq_len(q)) {
      num <- (sum((N / n_h) * var_contr_h[j, ]))^2
      denom <- sum(((N / n_h)^2) * (var_contr_h[j, ]^2) / (n_h - 1))
      dfs[j] <- if (denom > 1e-12) num / denom else 10000
    }
    df <- round(max(4,min(dfs)))
  }

  dimnames(Vhat_sum) <- list(cell_names, cell_names)
list(
  Sigma = Vhat_sum,
  p_pair = p_pair,
  eps_theta = eps_theta,
  cell_names = cell_names,
  df_satt = df
)
}

#*************************************************************************************
# Next is nu = 1 = unweighted = DEFAULT

.nparld_cov_H0p_ref_unweighted_subjects <- function(split_obj, Fhat, C = NULL) {
  S <- split_obj$struct
  wp_levels <- names(S)
  sp_levels <- split_obj$SP_levels
  a <- length(wp_levels)
  d <- length(sp_levels)
  ad <- a * d

  # Pairwise p_hat*(is,ht) under subjects
  pp <- .nparld_pairwise_p_hat(split_obj, Fhat, cell.weights = "subjects")
  p_pair <- pp$p_pair
  cell_names <- pp$cell_names

  eps_theta <- .nparld_theta_variance_floor(
  split_obj,
  cell.weights = "subjects",
  cell_names = cell_names
)

  col_of <- function(h, t) (match(h, wp_levels) - 1L) * d + match(t, sp_levels)
  row_of <- function(i, s) (match(i, wp_levels) - 1L) * d + match(s, sp_levels)

  n_h <- vapply(wp_levels, function(h) length(S[[h]]), integer(1))
  N <- sum(n_h)

  Vhat_sum <- matrix(0, ad, ad)

  if (!is.null(C)) {
    q <- nrow(C)
    var_contr_h <- matrix(0, nrow = q, ncol = length(wp_levels))
    colnames(var_contr_h) <- wp_levels
    h_index <- setNames(seq_along(wp_levels), wp_levels)
  }

  for (h in wp_levels) {
    subj_list <- S[[h]]
    nh <- length(subj_list)
    if (nh < 2L) {
      stop("Need at least 2 subjects in group ", h, " for covariance.", call. = FALSE)
    }

    Z <- matrix(0, nrow = nh, ncol = ad)

    # subject-specific observed values and counts for group h
    X_hk <- lapply(subj_list, function(one_subj) {
      setNames(lapply(sp_levels, function(t) {
        v <- one_subj[[t]]
        v[!is.na(v)]
      }), sp_levels)
    })

    m_hk <- lapply(X_hk, function(xx) {
      setNames(vapply(xx, length, integer(1)), sp_levels)
    })

    # lambda_ht = number of subjects in group h with at least one obs in time t
    lambda_ht <- setNames(
      vapply(sp_levels, function(t) {
        sum(vapply(m_hk, function(mm) mm[[t]] > 0L, logical(1)))
      }, integer(1)),
      sp_levels
    )

    for (k_idx in seq_along(subj_list)) {
      X_htk <- X_hk[[k_idx]]
      m_htk <- m_hk[[k_idx]]

      Psi  <- numeric(ad)
      beta <- numeric(ad)

      for (i in wp_levels) for (s in sp_levels) {
        is_idx <- row_of(i, s)

        if (h != i) {
          tmpPsi  <- 0
          tmpBeta <- 0

          for (t in sp_levels) {
            if (lambda_ht[[t]] <= 0L || m_htk[[t]] <= 0L) next

            vals <- X_htk[[t]]
            w_htk <- 1 / (lambda_ht[[t]] * m_htk[[t]])

            for (u in seq_along(vals)) {
              tmpPsi <- tmpPsi + w_htk * Fhat[[i]][[s]](vals[u])
            }

            ht_idx <- col_of(h, t)
            tmpBeta <- tmpBeta + (1 / lambda_ht[[t]]) * p_pair[is_idx, ht_idx]
          }

          Psi[is_idx]  <- -(nh / ad) * tmpPsi
          beta[is_idx] <- -(nh / ad) * tmpBeta

        } else {
          tmpPsi1  <- 0
          tmpBeta1 <- 0

          # terms involving observations from cell (i,s)
          if (lambda_ht[[s]] > 0L && m_htk[[s]] > 0L) {
            vals_is <- X_htk[[s]]
            w_isk <- 1 / (lambda_ht[[s]] * m_htk[[s]])

            for (j in wp_levels) {
              if (j == i) next

              for (t in sp_levels) {
                for (u in seq_along(vals_is)) {
                  tmpPsi1 <- tmpPsi1 + w_isk * Fhat[[j]][[t]](vals_is[u])
                }

                jt_idx <- row_of(j, t)
                is_col <- col_of(i, s)
                tmpBeta1 <- tmpBeta1 + (1 / lambda_ht[[s]]) * p_pair[jt_idx, is_col]
              }
            }
          }

          tmpPsi2  <- 0
          tmpBeta2 <- 0

          for (t in sp_levels) {
            # + sum_u [1/(lambda_is m_isk) * F_it(X_isku)]
            if (lambda_ht[[s]] > 0L && m_htk[[s]] > 0L) {
              vals_is <- X_htk[[s]]
              w_isk <- 1 / (lambda_ht[[s]] * m_htk[[s]])

              for (u in seq_along(vals_is)) {
                tmpPsi2 <- tmpPsi2 + w_isk * Fhat[[i]][[t]](vals_is[u])
              }

              it_idx <- row_of(i, t)
              is_col <- col_of(i, s)
              tmpBeta2 <- tmpBeta2 + (1 / lambda_ht[[s]]) * p_pair[it_idx, is_col]
            }

            # - sum_u [1/(lambda_it m_itk) * F_is(X_itku)]
            if (lambda_ht[[t]] > 0L && m_htk[[t]] > 0L) {
              vals_it <- X_htk[[t]]
              w_itk <- 1 / (lambda_ht[[t]] * m_htk[[t]])

              for (u in seq_along(vals_it)) {
                tmpPsi2 <- tmpPsi2 - w_itk * Fhat[[i]][[s]](vals_it[u])
              }

              is_row <- row_of(i, s)
              it_col <- col_of(i, t)
              tmpBeta2 <- tmpBeta2 - (1 / lambda_ht[[t]]) * p_pair[is_row, it_col]
            }
          }

          Psi[is_idx]  <- (nh / ad) * (tmpPsi1 + tmpPsi2)
          beta[is_idx] <- (nh / ad) * (tmpBeta1 + tmpBeta2)
        }
      }

      Z[k_idx, ] <- Psi - beta
    }

    if (!is.null(C)) {
      U <- Z %*% t(C)
      U[!is.finite(U)] <- 0
      var_h <- apply(U, 2, function(x) var(x, na.rm = TRUE))
      var_contr_h[, h_index[h]] <- var_h
    }

    Vh <- crossprod(scale(Z, center = colMeans(Z), scale = FALSE)) / (nh - 1)
    Vhat_sum <- Vhat_sum + (N / nh) * Vh
  }

  df <- NULL
  if (!is.null(C)) {
    dfs <- numeric(q)
    for (j in seq_len(q)) {
      num <- (sum((N / n_h) * var_contr_h[j, ]))^2
      denom <- sum(((N / n_h)^2) * (var_contr_h[j, ]^2) / (n_h - 1))
      dfs[j] <- if (denom > 1e-12) num / denom else 10000
    }
    df <- round(max(4, min(dfs)))
  }

  dimnames(Vhat_sum) <- list(cell_names, cell_names)

list(
  Sigma = Vhat_sum,
  p_pair = p_pair,
  eps_theta = eps_theta,
  cell_names = cell_names,
  df_satt = df
)
}

#*******************************************************************************************



.nparld_cfun <- function(x, y) {
  (x > y) + 0.5 * (x == y)
}

.nparld_split_by_wp_subject_sp <- function(dat, WP.names, SP.names) {

  dat$..WP <- .nparld_make_keys(dat, WP.names)
  dat$..SP <- .nparld_make_keys(dat, SP.names)

  wp_keys <- if(length(WP.names)==0) "1" else {
    wp_grid <- expand.grid(lapply(rev(WP.names), function(v) levels(dat[[v]])),
                           KEEP.OUT.ATTRS = FALSE, stringsAsFactors = FALSE)
    names(wp_grid) <- rev(WP.names)
    wp_grid <- wp_grid[WP.names]
    do.call(paste, c(lapply(wp_grid, as.character), sep=":"))
  }

  sp_keys <- if(length(SP.names)==0) "1" else {
    sp_grid <- expand.grid(lapply(rev(SP.names), function(v) levels(dat[[v]])),
                           KEEP.OUT.ATTRS = FALSE, stringsAsFactors = FALSE)
    names(sp_grid) <- rev(SP.names)
    sp_grid <- sp_grid[SP.names]
    do.call(paste, c(lapply(sp_grid, as.character), sep=":"))
  }

  by_wp <- split(dat, factor(dat$..WP, levels = wp_keys))

  out <- lapply(by_wp, function(dw) {
    dw$subject <- droplevels(dw$subject)
    by_subj <- split(dw, dw$subject)

    lapply(by_subj, function(ds) {
      by_sp <- split(ds[[1]], factor(ds$..SP, levels = sp_keys))
      by_sp[sapply(by_sp, is.null)] <- list(numeric(0))
      by_sp
    })
  })

  names(out) <- wp_keys

  list(
    struct = out,
    WP_levels = wp_keys,
    SP_levels = sp_keys
  )
}


.nparld_build_Fhat <- function(split_obj, cell.weights = c("subjects", "observations")) {
  cell.weights <- match.arg(cell.weights)

  S <- split_obj$struct
  wp_levels <- split_obj$WP_levels
  sp_levels <- split_obj$SP_levels

  Fhat <- lapply(wp_levels, function(wp) {
    subj_list <- S[[wp]]

    lapply(sp_levels, function(sp) {

      # observations per subject in this cell
      reps_by_subj <- lapply(subj_list, function(one_subj) {
        v <- one_subj[[sp]]
        v[!is.na(v)]
      })

      # pooled observations in this cell
      reps_all <- unlist(reps_by_subj, use.names = FALSE)

      function(xvec) {
        vapply(xvec, function(x) {

          if (cell.weights == "subjects") {
            # subject-unweighted
            vals <- vapply(reps_by_subj, function(reps) {
              if (length(reps) == 0L) return(NA_real_)
              mean(.nparld_cfun(x, reps))
            }, numeric(1))

            if (all(is.na(vals))) {
              NA_real_
            } else {
              mean(vals, na.rm = TRUE)
            }
          } else {
            # observations: observation-weighted
            if (length(reps_all) == 0L) {
              NA_real_
            } else {
              mean(.nparld_cfun(x, reps_all))
            }
          }

        }, numeric(1))
      }
    })
  })

  names(Fhat) <- wp_levels
  for (wp in wp_levels) names(Fhat[[wp]]) <- sp_levels

  Fhat
}


#*******************************************************************************
# Multiple Contrast Tests
#*******************************************************************************
.nparld_build_contrast_matrix <- function(contrast, H) {

  k <- nrow(H)

  if (!is.numeric(k) || length(k) != 1L || k < 2L) {
    stop("At least two marginal levels are required for contrasts.",
         call. = FALSE)
  }

  if (length(contrast) != 2L) {
    stop(
      "Named or user-defined contrasts require a second contrast argument.",
      call. = FALSE
    )
  }

  if (is.character(contrast[[2]])) {

    if (!requireNamespace("multcomp", quietly = TRUE)) {
      stop("Package 'multcomp' required for built-in contrasts.",
           call. = FALSE)
    }

    CC <- multcomp::contrMat(rep(1, k), type = contrast[[2]])
  }

  if (is.matrix(contrast[[2]])) {

    CC <- contrast[[2]]

    if (ncol(CC) != k) {
      stop(
        "Contrast matrix must have one column for each marginal level of the selected term. ",
        "Expected ", k, " columns, but got ", ncol(CC), ".",
        call. = FALSE
      )
    }

    if (is.null(rownames(CC))) {
      rownames(CC) <- paste("C", seq_len(nrow(CC)))
    }
  }

  if (is.numeric(contrast[[2]]) && !is.matrix(contrast[[2]])) {

    if (length(contrast[[2]]) != k) {
      stop(
        "Contrast length does not match the number of marginal levels of the selected term. ",
        "Expected ", k, " entries, but got ", length(contrast[[2]]), ".",
        call. = FALSE
      )
    }

    CC <- matrix(contrast[[2]], nrow = 1)
    rownames(CC) <- "Custom"
  }

  if (!exists("CC", inherits = FALSE)) {
    stop("Invalid contrast specification.", call. = FALSE)
  }

  rs <- rowSums(CC)
  rows <- which(abs(rs) > sqrt(.Machine$double.eps))

  if (length(rows) > 0L) {
    CC[rows, ] <- CC[rows, , drop = FALSE] -
      rowMeans(CC[rows, , drop = FALSE])
  }

  CC
}

.nparld_regularize_contrast_cov <- function(Vuse, Cfull, eps_theta) {
  vars <- diag(Vuse)

  eps_contrast <- apply(Cfull, 1, function(cc) {
    involved <- which(abs(cc) > 0)

    if (length(involved) < 2L) {
      return(NA_real_)
    }

    eps_sub <- eps_theta[involved, involved, drop = FALSE]
    eps_sub <- eps_sub[is.finite(eps_sub)]

    if (length(eps_sub) == 0L) {
      NA_real_
    } else {
      min(eps_sub)
    }
  })

  vars.reg <- .nparld_regularize_theta_variance(
    var_theta = vars,
    eps_theta = eps_contrast
  )

  diag(Vuse) <- vars.reg
  Vuse
}

.nparld_MCTP <- function(theta_hat, Sigma, H, C, N, alpha,
                         sci.method = c("fisher", "multi.t"),
                         hypothesis,
                         df_satt = NULL,
                         eps_theta = NULL) {

  if (!requireNamespace("mvtnorm", quietly = TRUE))
    stop("Package 'mvtnorm' required.")
  eps_theta = NULL
  sci.method <- match.arg(sci.method)

  pd.main <- H %*% theta_hat
  V.main  <- H %*% Sigma %*% t(H)

  CH.pd <- as.vector(C %*% pd.main)
  CH.V  <- C %*% V.main %*% t(C)



Cfull <- C %*% H

if (!is.null(eps_theta)) {
  CH.V <- .nparld_regularize_contrast_cov(
    Vuse = CH.V,
    Cfull = Cfull,
    eps_theta = eps_theta
  )
}

  # degrees of freedom
  df <- if (!is.null(df_satt)) min(df_satt) else max(4, N - 1)

  # ------------------------------------------------------------------
  # H0F: no simultaneous confidence intervals, no Fisher transformation
  # ------------------------------------------------------------------
  if (hypothesis == "H0F") {

    Est <- CH.pd
    Vuse <- CH.V
    SE <- sqrt(diag(Vuse))
    Tvec <- Est / (SE / sqrt(N))
    R <- cov2cor(Vuse)

    pv <- sapply(seq_along(Tvec), function(i) {
      1 - mvtnorm::pmvt(
        lower = -abs(Tvec[i]),
        upper =  abs(Tvec[i]),
        corr = R,
        delta = rep(0, nrow(R)),
        df = df
      )[1]
    })

    crit <- mvtnorm::qmvt(
      1 - alpha,
      delta = rep(0, nrow(R)),
      corr = R,
      tail = "both",
      df = df
    )$quantile

    res <- data.frame(
      Estimate  = Est,
      Std.Error = SE,
      Statistic = Tvec,
      p.value   = pv,
      df        = df
    )

    return(list(
      Contrast.Matrix = C %*% H,
      Local.Results   = res,
      Global.Result   = data.frame(
        Statistic = max(abs(Tvec)),
        p.value   = min(pv)
      ),
      DF = df,
      Quantile = crit,
      sci.method = NA_character_
    ))
  }

  # ------------------------------------------------------------------
  # H0p: simultaneous confidence intervals are computed
  # ------------------------------------------------------------------

  if (sci.method == "multi.t") {

    Est <- CH.pd
    Vuse <- CH.V
    SE <- sqrt(diag(Vuse))
    Tvec <- Est / (SE / sqrt(N))
    R <- cov2cor(Vuse)

    pv <- sapply(seq_along(Tvec), function(i) {
      1 - mvtnorm::pmvt(
        lower = -abs(Tvec[i]),
        upper =  abs(Tvec[i]),
        corr = R,
        delta = rep(0, nrow(R)),
        df = df
      )[1]
    })

    crit <- mvtnorm::qmvt(
      1 - alpha,
      delta = rep(0, nrow(R)),
      corr = R,
      tail = "both",
      df = df
    )$quantile

    lower <- Est - crit * SE / sqrt(N)
    upper <- Est + crit * SE / sqrt(N)

    res <- data.frame(
      Estimate  = Est,
      Std.Error = SE,
      lower     = lower,
      upper     = upper,
      Statistic = Tvec,
      p.value   = pv,
      df        = df
    )

    return(list(
      Contrast.Matrix = C %*% H,
      Local.Results   = res,
      Global.Result   = data.frame(
        Statistic = max(abs(Tvec)),
        p.value   = min(pv)
      ),
      DF = df,
      Quantile = crit,
      sci.method = sci.method
    ))
  }

  # ------------------------------------------------------------------
  # H0p + Fisher transformation + delta method
  # ------------------------------------------------------------------

  eps <- 1e-10
  if (any(abs(CH.pd) >= 1 - eps)) {
    stop("Fisher transformation requires all contrast estimates to lie strictly between -1 and 1.",
         call. = FALSE)
  }

  g <- function(x) 0.5 * log((1 + x) / (1 - x))
  ginv <- function(y) tanh(y)
  gp <- function(x) 1 / (1 - x^2)

  Est.raw <- CH.pd
  Est <- g(Est.raw)

  D <- diag(gp(Est.raw), nrow = length(Est.raw), ncol = length(Est.raw))
  Vuse <- D %*% CH.V %*% D

  SE <- sqrt(diag(Vuse))
  Tvec <- Est / (SE / sqrt(N))
  R <- cov2cor(Vuse)

  pv <- sapply(seq_along(Tvec), function(i) {
    1 - mvtnorm::pmvt(
      lower = -abs(Tvec[i]),
      upper =  abs(Tvec[i]),
      corr = R,
      delta = rep(0, nrow(R)),
      df = df
    )[1]
  })

  crit <- mvtnorm::qmvt(
    1 - alpha,
    delta = rep(0, nrow(R)),
    corr = R,
    tail = "both",
    df = df
  )$quantile

  lower.z <- Est - crit * SE / sqrt(N)
  upper.z <- Est + crit * SE / sqrt(N)

  lower <- ginv(lower.z)
  upper <- ginv(upper.z)

  res <- data.frame(
    Estimate  = Est.raw,
    Std.Error = SE,
    lower     = lower,
    upper     = upper,
    Statistic = Tvec,
    p.value   = pv,
    df        = df
  )

  list(
    Contrast.Matrix = C %*% H,
    Local.Results   = res,
    Global.Result   = data.frame(
      Statistic = max(abs(Tvec)),
      p.value   = min(pv)
    ),
    DF = df,
    Quantile = crit,
    sci.method = sci.method
  )
}

.nparld_run_contrasts <- function(theta_hat,
                                  Sigma,
                                  H,
                                  C,
                                  N,
                                  alpha,
                                  sci.method,
                                  hypothesis,
                                  df_satt = NULL,
                                  eps_theta = NULL,
                                  factor_name = NULL) {

  res <- .nparld_MCTP(
    theta_hat = theta_hat,
    Sigma = Sigma,
    H = H,
    C = C,
    N = N,
    alpha = alpha,
    sci.method = sci.method,
    hypothesis = hypothesis,
    df_satt = df_satt,
    eps_theta = eps_theta
  )

  # build readable contrast labels
  contrast_labels <- rownames(C)

  if (is.null(contrast_labels)) {
    contrast_labels <- paste("C", seq_len(nrow(C)))
  }

  rownames(res$Local.Results) <- contrast_labels
  rownames(res$Contrast.Matrix) <- contrast_labels

  if (!is.null(names(theta_hat)) &&
      length(names(theta_hat)) == ncol(res$Contrast.Matrix)) {
    colnames(res$Contrast.Matrix) <- names(theta_hat)
  }

  if (!is.null(factor_name)) {
    res$Factor <- factor_name
  }

  class(res) <- "nparld_mctp"
  res
}

#*******************************************************************************
# Two time points
#*******************************************************************************

.can_use_perm_test <- function(design_obj) {
  
  # exactly one subplot factor
  if (length(design_obj$SP.names) != 1) return(FALSE)
  
  sp <- design_obj$SP.names
  
  # exactly 2 levels
  if (design_obj$n.levels[sp] != 2) return(FALSE)
  
  # no missing values
  if (any(is.na(design_obj$dat))) return(FALSE)
  
  TRUE
}

.extract_paired_samples <- function(design_obj) {
  
  dat <- design_obj$dat
  response <- all.vars(design_obj$formula)[1]
  sp <- design_obj$SP.names
  
  lev <- design_obj$names.levels[[sp]]
  
  # reshape into wide format (safe because design is balanced)
  wide <- reshape(
    dat,
    idvar = "subject",
    timevar = sp,
    direction = "wide"
  )
  
  x <- wide[[paste0(response, ".", lev[1])]]
  y <- wide[[paste0(response, ".", lev[2])]]
  
  list(x = x, y = y, lev = lev)
}

studentized_perm_test_paired <- function(
  x, y, B = 1000, alpha = 0.05,
  hypothesis = c("H0p", "H0F")
){
  
  n <- length(x)
  
compute_stat <- function(x, y, hypothesis) {
  
  n <- length(x)
  
  pooled <- c(x, y)
  ranks <- rank(pooled, ties.method = "average")
  
  R1 <- ranks[1:n]
  R2 <- ranks[(n+1):(2*n)]
  
  # --- estimator p-hat ---
  p_hat <- (mean(R2) - mean(R1)) / (2*n) + 0.5
  
  # --- variance depending on hypothesis ---
  
  if (hypothesis == "H0p") {
    
    # original Munzel placements
    R1_within <- rank(x, ties.method = "average")
    R2_within <- rank(y, ties.method = "average")
    
    Z <- (R2 - R2_within - R1 + R1_within) / n
    
    sigma_hat <- sd(Z)
    
  } else if (hypothesis == "H0F") {
    
    D <- (R2 - R1) / (2*n)   # scaling optional but recommended
    
    sigma_hat <- sd(D)
    
  }
  
  # safeguard
  if (sigma_hat == 0) sigma_hat <- 1/n
  
  M <- sqrt(n) * (p_hat - 0.5) / sigma_hat
  
  list(M = M, p_hat = p_hat, sigma = sigma_hat)
}
  
  # observed
  obs <- compute_stat(x, y, hypothesis)
  M_obs <- obs$M
se <- obs$sigma/sqrt(n)

# after computing obs, se, etc.

df <- n - 1

# t-based p-value
p_t <- 2 * (1 - pt(abs(M_obs), df = df))

# t-based CI (only meaningful for H0p)
t_quant <- qt(1 - alpha/2, df = df)

CI_t_lower <- obs$p_hat - t_quant * se
CI_t_upper <- obs$p_hat + t_quant * se
  
  # permutation distribution
  M_perm <- numeric(B)
  
  for (b in seq_len(B)) {
    swap <- rbinom(n, 1, 0.5) == 1
    
    x_perm <- x
    y_perm <- y
    
    x_perm[swap] <- y[swap]
    y_perm[swap] <- x[swap]
    
    M_perm[b] <- compute_stat(x_perm, y_perm, hypothesis)$M
  }
  
  # p-value
  p1 <- mean(M_perm >= M_obs)
  p_value <- min(2*p1, 2 - 2*p1)
  
  # --- CI via permutation quantiles (Eq. 3.4) ---
  q_low  <- quantile(M_perm, probs = alpha/2, na.rm = TRUE)
  q_high <- quantile(M_perm, probs = 1 - alpha/2, na.rm = TRUE)
  
  se <- obs$sigma / sqrt(n)
  
  CI_lower <- obs$p_hat - q_high * se
  CI_upper <- obs$p_hat - q_low  * se
  
if (hypothesis == "H0F") {
  CI_lower <- NA
  CI_upper <- NA
}

list(
  statistic = M_obs,
  p.value = p_value,
  p.value.t = p_t,
  estimate = obs$p_hat,
  se = se,
  CI = c(lower = CI_lower, upper = CI_upper),
  CI.t = c(lower = CI_t_lower, upper = CI_t_upper),
  perm.dist = M_perm
)
}


.format_perm_output <- function(res, alpha = 0.05, hypothesis = "H0p") {
  
  if (hypothesis == "H0F") {
    
    out <- data.frame(
      Estimate = res$estimate,
      Std.Error = res$se,
      Statistic = res$statistic,
      p.t = res$p.value.t,
      p.perm = res$p.value
    )
    
    rownames(out) <- "Rank-based test"
    
  } else {
    
    out <- data.frame(
      Estimate = res$estimate,
      Std.Error = res$se,
      Lower.t = res$CI.t[1],
      Upper.t = res$CI.t[2],
      Lower.perm = res$CI[1],
      Upper.perm = res$CI[2],
      Statistic = res$statistic,
      p.t = res$p.value.t,
      p.perm = res$p.value
    )
    
    rownames(out) <- paste0("p (CI ", 100*(1-alpha), "%)")
  }
  
  out
}

#' Alpha-amylase Saliva Study
#'
#' Measurements of alpha-amylase levels in saliva from healthy volunteers.
#' The study illustrates a two-factor longitudinal design with two repeated
#' measures factors.
#'
#' @format A data frame with variables:
#' \describe{
#'   \item{resp}{Alpha-amylase measurement.}
#'   \item{time1}{First repeated-measures factor.}
#'   \item{time2}{Second repeated-measures factor.}
#'   \item{subject}{Subject identifier.}
#' }
#'
#' @examples
#' data(amylase)
#'
#' fit <- nparLD(resp ~ time1 * time2,
#'               data = amylase,
#'               subject = "subject",
#'               effect = "weighted",
#'               hypothesis = "H0F")
#' fit
#'
#' # Dunnett-type contrasts for the levels of time2
#' fit <- nparLD(resp ~ time1 * time2,
#'               data = amylase,
#'               subject = "subject",
#'               hypothesis = "H0p",
#'               contrast = list("time2", "Dunnett"),
#'               Factor.Information = TRUE)
#' fit
#' plot(fit)
#' plot(fit$MCTP)
#'
#' @docType data
#' @keywords datasets
"amylase"


#' Dental Growth Study
#'
#' Measurements of distances between the center of the pituitary and the
#' pterygomaxillary fissure in boys. The data illustrate a one-factor
#' longitudinal design.
#'
#' @format A data frame with variables:
#' \describe{
#'   \item{resp}{Distance measurement in millimeters.}
#'   \item{time}{Age at measurement.}
#'   \item{subject}{Subject identifier.}
#' }
#'
#' @examples
#' data(dental)
#'
#' dental$time <- factor(dental$time)
#'
#' \dontrun{
#' fit <- nparLD(resp ~ time,
#'               data = dental,
#'               subject = "subject",
#'               hypothesis = "H0F")
#' fit
#' }
#'
#' \dontrun{
#' fit <- nparLD(resp ~ time,
#'               data = dental,
#'               subject = "subject",
#'               hypothesis = "H0p",
#'               contrast = list("time", "Tukey"))
#' fit
#' plot(fit)
#' plot(fit$MCTP)
#'}
#'
#' @docType data
#' @keywords datasets
"dental"


#' Postoperative Edema Study
#'
#' Skin-temperature measurements from patients after hand surgery. The study
#' compares treatment groups and includes two repeated-measures factors.
#'
#' @format A data frame with variables:
#' \describe{
#'   \item{resp}{Skin temperature measurement.}
#'   \item{time1}{Hand factor.}
#'   \item{time2}{Day factor.}
#'   \item{group}{Treatment group.}
#'   \item{subject}{Subject identifier.}
#' }
#'
#' @examples
#' data(edema)
#'
#' \dontrun{
#' fit <- nparLD(resp ~ group * time1 * time2,
#'               data = edema,
#'               subject = "subject",
#'               hypothesis = "H0F")
#' fit
#' }
#'
#' \dontrun{
#' fit <- nparLD(resp ~ group * time1 * time2,
#'               data = edema,
#'               subject = "subject",
#'               hypothesis = "H0p",
#'               contrast = list("group", "Tukey"),
#'               Factor.Information = TRUE)
#' fit
#' plot(fit)
#' plot(fit$MCTP)
#' }
#'
#' @docType data
#' @keywords datasets
"edema"


#' Panic Disorder Study I
#'
#' Clinical Global Impression scores from patients with panic disorder and
#' agoraphobia. The data illustrate a one-factor longitudinal design with
#' ordinal responses.
#'
#' @format A data frame with variables:
#' \describe{
#'   \item{resp}{Clinical Global Impression score.}
#'   \item{time}{Visit or week of assessment.}
#'   \item{subject}{Patient identifier.}
#' }
#'
#' @examples
#' data(panic)
#'
#' \dontrun{
#' fit <- nparLD(resp ~ time,
#'               data = panic,
#'               subject = "subject",
#'               hypothesis = "H0F", contrast =list("time", "Dunnett"))
#' fit
#' }
#'
#' \dontrun{
#' fit <- nparLD(resp ~ time,
#'               data = panic,
#'               subject = "subject",
#'               hypothesis = "H0p",
#'               contrast = list("time", "Dunnett"))
#' fit
#' plot(fit)
#' plot(fit$MCTP)
#' }
#'
#' @docType data
#' @keywords datasets
"panic"


#' Panic Disorder Study II
#'
#' Panic and agoraphobia scores from patients with panic disorder, with or
#' without agoraphobia. The data illustrate a factorial longitudinal design with
#' one whole-plot factor and one repeated-measures factor.
#'
#' @format A data frame with variables:
#' \describe{
#'   \item{resp}{Panic and agoraphobia score.}
#'   \item{time}{Visit or week of assessment.}
#'   \item{group}{Agoraphobia group.}
#'   \item{subject}{Patient identifier.}
#' }
#'
#' @examples
#' data(panic2)
#'
#' \dontrun{
#' fit <- nparLD(resp ~ group * time,
#'               data = panic2,
#'               subject = "subject",
#'               hypothesis = "H0F", effect="unweighted")
#' fit
#' }
#'
#' \dontrun{
#' fit <- nparLD(resp ~ group * time,
#'               data = panic2,
#'               subject = "subject",
#'               hypothesis = "H0p",
#'               contrast = list("group:time"),
#'               Factor.Information = TRUE,
#'               covariance=TRUE)
#' fit
#' plot(fit)
#' plot(fit$MCTP)
#' }
#'
#' @docType data
#' @keywords datasets
"panic2"


#' Plasma-renin Activity Study
#'
#' Plasma-renin activity measurements from a randomized study of healthy
#' non-smokers. The data illustrate a factorial longitudinal design with one
#' whole-plot factor and one repeated-measures factor.
#'
#' @format A data frame with variables:
#' \describe{
#'   \item{resp}{Plasma-renin activity measurement.}
#'   \item{time}{Measurement time.}
#'   \item{group}{Drug group.}
#'   \item{subject}{Subject identifier.}
#' }
#'
#' @examples
#' data(plasma)
#'
#' \dontrun{
#' fit <- nparLD(resp ~ group * time,
#'               data = plasma,
#'               subject = "subject",
#'               hypothesis = "H0F", effect="weighted")
#' fit
#' }
#'
#' \dontrun{
#' fit <- nparLD(resp ~ group * time,
#'               data = plasma,
#'               subject = "subject",
#'               hypothesis = "H0p",
#'               contrast = list("group", "Tukey"))
#' fit
#' plot(fit)
#' plot(fit$MCTP)
#' }
#'
#' @docType data
#' @keywords datasets
"plasma"


#' Rat Growth Study
#'
#' Body-weight measurements from rats observed over a five-week period. The data
#' illustrate longitudinal growth curves in several treatment groups.
#'
#' @format A data frame with variables:
#' \describe{
#'   \item{resp}{Body weight in grams.}
#'   \item{time}{Week of measurement.}
#'   \item{group}{Treatment group.}
#'   \item{subject}{Rat identifier.}
#' }
#'
#' @examples
#' 
#' \dontrun{
#' data(rat)
#'
#' fit <- nparLD(resp ~ group * time,
#'               data = rat,
#'               subject = "subject",
#'               hypothesis = "H0F")
#' fit
#'}
#'
#' \dontrun{
#' fit <- nparLD(resp ~ group * time,
#'               data = rat,
#'               subject = "subject",
#'               hypothesis = "H0p",
#'               contrast = list("group:time"))
#' fit
#' plot(fit)
#' plot(fit$MCTP)
#' }
#'
#' @docType data
#' @keywords datasets
"rat"


#' Respiratory Disorder Study
#'
#' Ordinal health-status measurements from patients with a respiratory disorder.
#' The data include study center, treatment group, and repeated visits.
#'
#' @format A data frame with variables:
#' \describe{
#'   \item{resp}{Ordinal health-status response.}
#'   \item{time}{Visit number.}
#'   \item{center}{Study center.}
#'   \item{treatment}{Treatment group.}
#'   \item{patient}{Patient identifier.}
#' }
#'
#' @examples
#'
#' \dontrun{
#' data(respiration)
#'
#' fit <- nparLD(resp ~ center * treatment * time,
#'               data = respiration,
#'               subject = "patient",
#'               hypothesis = "H0F")
#' fit
#' }
#'
#' \dontrun{
#' fit <- nparLD(resp ~ center * treatment * time,
#'               data = respiration,
#'               subject = "patient",
#'               hypothesis = "H0p",
#'               contrast = list("treatment", "Tukey"))
#' fit
#' plot(fit)
#' plot(fit$MCTP)
#' }
#'
#' @docType data
#' @keywords datasets
"respiration"


#' Shoulder Tip Pain Study
#'
#' Shoulder pain scores from patients after laparoscopic abdominal surgery. The
#' study includes treatment group, gender group, and repeated pain measurements.
#'
#' @format A data frame with variables:
#' \describe{
#'   \item{resp}{Shoulder pain score.}
#'   \item{time}{Measurement occasion.}
#'   \item{group1}{Treatment group.}
#'   \item{group2}{Gender group.}
#'   \item{subject}{Patient identifier.}
#' }
#'
#' @examples
#' data(shoulder)
#'
#' \dontrun{
#' fit <- nparLD(resp ~ group1 * group2 * time,
#'               data = shoulder,
#'               subject = "subject",
#'               hypothesis = "H0F")
#' fit
#' }
#'
#' \dontrun{
#' fit <- nparLD(resp ~ group1 * group2 * time,
#'               data = shoulder,
#'               subject = "subject",
#'               hypothesis = "H0p",
#'               contrast = list("group1:time"),
#'               Factor.Information=TRUE)
#' fit
#' plot(fit)
#' plot(fit$MCTP)
#' }
#'
#' @docType data
#' @keywords datasets
"shoulder"


#' Vitality of Treetops
#'
#' Repeated vitality scores of treetops from three experimental areas. The data
#' illustrate an ordinal longitudinal response in several groups.
#'
#' @format A data frame with variables:
#' \describe{
#'   \item{resp}{Vitality score.}
#'   \item{time}{Year or measurement occasion.}
#'   \item{group}{Experimental area.}
#'   \item{subject}{Tree identifier.}
#' }
#'
#' @examples
#' data(tree)
#'
#' \dontrun{
#' fit <- nparLD(resp ~ group * time,
#'               data = tree,
#'               subject = "subject",
#'               hypothesis = "H0F")
#' fit
#'}
#'
#'
#' \dontrun{
#' fit <- nparLD(resp ~ group * time,
#'               data = tree,
#'               subject = "subject",
#'               hypothesis = "H0p",
#'               contrast = list("group:time"))
#' fit
#' plot(fit)
#' plot(fit$MCTP)
#'}
#'
#' @docType data
#' @keywords datasets
"tree"

#' BrdU incorporation in fibroblasts
#'
#' BrdU incorporation in fibroblast cultures measured under four dose
#' conditions. For each culture and dose condition, three replicate measurements
#' are available. The data set is provided in long format and can be used to
#' illustrate dependent replicate measurements in `nparLD()`.
#'
#' @format A data frame with 60 rows and 4 variables:
#' \describe{
#'   \item{resp}{BrdU incorporation response.}
#'   \item{dose}{Dose condition, with levels `0`, `0.1`, `1`, and `10`.}
#'   \item{culture}{Culture identifier. This is the independent experimental unit.}
#'   \item{replicate}{Replicate identifier within each culture-dose condition.}
#' }
#'
#' @details
#' The original variables were `CultureNo`, `Replic`, `Dose`, and `Brdu`.
#' In the package data set, the variables were renamed to `culture`,
#' `replicate`, `dose`, and `resp` for consistency with the examples.
#'
#' @examples
#' \dontrun{
#' data(brdu)
#'
#' fit <- nparLD(
#'   resp ~ dose,
#'   data = brdu,
#'   subject = "culture",
#'   replicate = "replicate",
#'   hypothesis = "H0p",
#'   cell.weights = "subjects"
#' )
#'
#' fit
#' }
#'
#' @name brdu
#' @docType data
#' @keywords datasets
"brdu"

#*******************************************************************************
# Output Functions
#*******************************************************************************

utils::globalVariables(c(".data", "RTE", "lower", "upper"))

#' Print nparLD fit
#'
#' Prints the main results of a fitted \code{nparLD()} model, including the
#' detected design, hypothesis type, ranking method, estimated relative effects,
#' and global WTS and ATS results. Optional components such as factor-specific
#' information, multiple contrast results, covariance matrices, and permutation
#' tests are displayed when available.
#'
#' @param x An object of class \code{"nparld_fit"}.
#'
#' @param x An object of class \code{"nparld_fit"}.
#' @param digits Number of digits used for printed numerical results.
#' @param ... Further arguments.
#'
#' @details
#' The print method displays the detected design, sample size information,
#' hypothesis type, ranking method, estimated relative effects, and global WTS
#' and ATS results. Optional components are printed when available, including
#' factor-specific information, multiple contrast results, covariance matrices,
#' and permutation test results.
#' @export
print.nparld_fit <- function(x, digits = 4, ...) {

  cat("\nNonparametric Longitudinal Analysis\n")
  cat("-------------------------------------------------\n")

  if (!is.null(x$Design)) {
    cat("Design: ", x$Design, "\n", sep = "")

    if (!is.null(x$wholeplots) && length(x$wholeplots) > 0L) {
      cat("Whole-plot factors: ", paste(x$wholeplots, collapse = ", "), "\n", sep = "")
    }

    if (!is.null(x$subplots) && length(x$subplots) > 0L) {
      cat("Subplot factors: ", paste(x$subplots, collapse = ", "), "\n", sep = "")
    }
  }

  if (!is.null(x$N.info)) {
    cat("Number of subjects: ", x$N.info, "\n", sep = "")
  }

  if (!is.null(x$text.hypotheses)) {
    cat("Hypotheses: ", x$text.hypotheses, "\n", sep = "")
  }

  if (!is.null(x$text.ranks)) {
    cat("Ranking: ", x$text.ranks, "\n", sep = "")
  }

  if (!is.null(x$hypothesis) && identical(x$hypothesis, "H0p") &&
      !is.null(x$CI.method)) {
    cat("Confidence interval method: ", x$CI.method, "\n", sep = "")
  }

  cat("\nEstimated relative effects\n")
  cat("-------------------------------------------------\n")
  print(x$effects, digits = digits)

  if (!is.null(x$factor.info)) {
    cat("\nFactor-specific relative effects\n")
    cat("-------------------------------------------------\n")
    print(x$factor.info, digits = digits)
  }

  if (!is.null(x$WTS) || !is.null(x$ATS)) {
    cat("\nGlobal test results\n")
    cat("-------------------------------------------------\n")
  }

  if (!is.null(x$WTS)) {
    cat("\nWald-type statistic (WTS)\n")
    print(round(x$WTS, digits))
  }

  if (!is.null(x$ATS)) {
    cat("\nANOVA-type statistic (ATS)\n")
    print(round(x$ATS, digits))
  }

  if (!is.null(x$MCTP)) {
    print(x$MCTP, digits = digits)
  }

  if (!is.null(x$covariance.info)) {
    print(x$covariance.info, digits = digits)
  }

  if (!is.null(x$perm) && isTRUE(x$perm$available)) {
    cat("\nPermutation test (studentized)\n")
    cat("-------------------------------------------------\n")
    print(x$perm$table, digits = digits)
  }

  invisible(x)
}

#' Print nparLD covariance matrix
#'
#' Displays the estimated covariance matrix of the relative effect estimator. 
#'
#' @param x An object of class \code{"nparld_covarianceinfo"}.
#' @param digits Number of digits used for printing numerical results.
#' @param ... Further arguments.
#'
#' @details
#' The printed matrix is the estimated covariance matrix used for the selected
#' hypothesis. It is returned when \code{covariance = TRUE}.
#' @export
print.nparld_covarianceinfo <- function(x, digits = 4, ...) {

  cat("\nEstimated covariance matrix\n")
  cat("-------------------------------------------------\n")

  print(round(unclass(x), digits = digits))

  invisible(x)
}

#' Print nparLD factor information
#'
#' Prints factor-specific relative effects, standard errors, and confidence
#' limits for the main effects and interactions of an \code{nparLD()} fit.
#' Factor information is returned when the model is fitted with
#' \code{Factor.Information = TRUE} and can also be displayed graphically with
#' \code{plot(fit, term = ...)}.
#'
#' @param x An object of class \code{"nparld_factorinfo"}.
#' @param digits Number of digits used for printing numerical results.
#' @param ... Further arguments.
#'
#' @details
#' Factor information consists of term-specific relative effects, standard
#' errors, and confidence limits for main effects and interactions. These
#' summaries are returned when \code{Factor.Information = TRUE} and can be
#' visualized with \code{plot(fit, term = ...)}.
#'
#' @export
print.nparld_factorinfo <- function(x, digits = 4, ...) {

  cat("\nFactor-specific relative effects\n")
  cat("-------------------------------------------------\n")

  for (i in seq_along(x)) {

    cat("\nTerm: ", names(x)[i], "\n", sep = "")

    tab <- as.data.frame(x[[i]])

    num_cols <- vapply(tab, is.numeric, logical(1))

    tab[num_cols] <- lapply(tab[num_cols], round, digits = digits)

    print(tab)
  }

  invisible(x)
}

#' Summarize an nparLD fit
#'
#' Extracts the main components of a fitted \code{nparLD()} object into a
#' structured summary object. The summary is useful for inspecting, printing, or
#' programmatically accessing the main results without working directly with all
#' internal components of the fitted object.
#'
#' @param object An object of class \code{"nparld_fit"}.
#'
#' @param object An object of class \code{"nparld_fit"}.
#' @param ... Further arguments passed to or from other methods.
#'
#' @details
#' The summary method returns a structured list containing the main components
#' of the fitted object. In contrast to \code{print()}, which is mainly intended
#' for console display, \code{summary()} is useful for storing or extracting the
#' main results programmatically.
#'
#' @return An object of class \code{"summary.nparld_fit"} containing selected
#'   components of the fitted model.
#'
#' @export
summary.nparld_fit <- function(object, ...) {

  res <- list(
    Design = object$Design,
    wholeplots = object$wholeplots,
    subplots = object$subplots,
    N.info = object$N.info,
    hypothesis = object$hypothesis,
    effects = object$effects,
    factor.info = object$factor.info,
    WTS = object$WTS,
    ATS = object$ATS
  )

  if (!is.null(object$MCTP)) {
    res$MCTP <- object$MCTP
  }

  if (!is.null(object$covariance.info)) {
    res$covariance.info <- object$covariance.info
  }

  if (!is.null(object$perm) && isTRUE(object$perm$available)) {
    res$Permutation <- object$perm
  }

  res <- Filter(Negate(is.null), res)

  class(res) <- "summary.nparld_fit"
  res
}

#' Print summary of an nparLD fit
#'
#' Prints the main components of a summarized \code{nparLD()} fit, including
#' design information, hypothesis type, estimated relative effects, and global
#' WTS and ATS results. Optional components such as factor-specific information,
#' multiple contrast results, covariance matrices, and permutation tests are
#' displayed when available.
#'
#' @param x An object of class \code{"summary.nparld_fit"}.
#' @param digits Number of digits used for printing numerical results.
#' @param ... Further arguments.
#'
#' @export
print.summary.nparld_fit <- function(x, digits = 4, ...) {

  cat("\nSummary of Nonparametric Longitudinal Analysis\n")
  cat("-------------------------------------------------\n")

  if (!is.null(x$Design)) {
    cat("Design: ", x$Design, "\n", sep = "")
  }

  if (!is.null(x$wholeplots) && length(x$wholeplots) > 0L) {
    cat("Whole-plot factors: ",
        paste(x$wholeplots, collapse = ", "), "\n", sep = "")
  }

  if (!is.null(x$subplots) && length(x$subplots) > 0L) {
    cat("Subplot factors: ",
        paste(x$subplots, collapse = ", "), "\n", sep = "")
  }

  if (!is.null(x$hypothesis)) {
    cat("Hypothesis: ", x$hypothesis, "\n", sep = "")
  }

if (!is.null(x$N.info)) {
  cat("Number of subjects: ", x$N.info, "\n", sep = "")
}

  if (!is.null(x$effects)) {
    cat("\nEstimated relative effects\n")
    cat("-------------------------------------------------\n")
    print(x$effects, digits = digits)
  }

  if (!is.null(x$factor.info)) {
    print(x$factor.info, digits = digits)
  }

  if (!is.null(x$WTS)) {
    cat("\nWald-type statistic (WTS)\n")
    cat("-------------------------------------------------\n")
    print(round(x$WTS, digits))
  }

  if (!is.null(x$ATS)) {
    cat("\nANOVA-type statistic (ATS)\n")
    cat("-------------------------------------------------\n")
    print(round(x$ATS, digits))
  }

  if (!is.null(x$MCTP)) {
    print(x$MCTP, digits = digits)
  }

  if (!is.null(x$covariance.info)) {
    print(x$covariance.info, digits = digits)
  }

  if (!is.null(x$Permutation) && !is.null(x$Permutation$table)) {
    cat("\nPermutation test\n")
    cat("-------------------------------------------------\n")
    print(x$Permutation$table, digits = digits)
  }

  invisible(x)
}

#' Print nparLD multiple contrast results
#'
#' Prints the global and local results of a multiple contrast test procedure.
#' The local results include contrast estimates, standard errors, simultaneous
#' confidence limits, test statistics, adjusted p-values, and degrees of freedom.
#' The contrast matrix can optionally be displayed with
#' \code{show.matrix = TRUE}.
#'
#' @param x An object of class \code{"nparld_mctp"}.
#' @param x An object of class \code{"nparld_mctp"}.
#' @param digits Number of digits used for printing numerical results.
#' @param show.matrix Logical. If \code{TRUE}, print the contrast matrix used in the
#'   multiple contrast procedure.
#' @param ... Further arguments.
#'
#' @details
#' The method prints the global multiple contrast test and the local contrast
#' results, including estimates, standard errors, simultaneous confidence
#' limits, test statistics, adjusted p-values, and degrees of freedom. The
#' contrast matrix is stored in \code{x$Contrast.Matrix} and is printed only
#' when \code{show.matrix = TRUE}.
#' @export
print.nparld_mctp <- function(x, digits = 4, show.matrix = FALSE, ...) {

  cat("\nMultiple contrast test procedure\n")
  cat("-------------------------------------------------\n")

  if (!is.null(x$Factor)) {
    cat("Factor or interaction: ", x$Factor, "\n", sep = "")
  }

  if (!is.null(x$sci.method) && !is.na(x$sci.method)) {
    cat("SCI method: ", x$sci.method, "\n", sep = "")
  }

  if (!is.null(x$Global.Result)) {
    cat("\nGlobal result\n")
    cat("-------------------------------------------------\n")
    print(round(x$Global.Result, digits = digits))
  }

  if (!is.null(x$Local.Results)) {
    cat("\nLocal contrast results\n")
    cat("-------------------------------------------------\n")
    print(round(x$Local.Results, digits = digits))
  }

  if (!is.null(x$Contrast.Matrix)) {
    if (isTRUE(show.matrix)) {
      cat("\nContrast matrix\n")
      cat("-------------------------------------------------\n")
      print(round(x$Contrast.Matrix, digits = digits))
    } else {
      cat("\nContrast matrix: available in component `Contrast.Matrix`.\n")
      cat("Use `print(x, show.matrix = TRUE)` to display it.\n")
    }
  }

  invisible(x)
}

#' Plot nparLD results
#'
#' Displays graphical summaries of estimated relative effects from an
#' \code{nparLD()} fit. By default, the method plots the cell-level relative
#' effects stored in \code{x$effects}. If \code{term} is supplied, it plots
#' factor-specific relative effects and confidence intervals for selected main
#' effects or interactions. Term-specific plots require that the model was fitted
#' with \code{Factor.Information = TRUE}.
#'
#' @param x An object of class \code{"nparld_fit"}.
#' @param term Optional character vector specifying one or more model terms for
#'   which factor-specific relative effects and confidence intervals should be
#'   plotted. The requested terms require \code{Factor.Information = TRUE} in the
#'   original call to \code{nparLD()}. If \code{term = NULL}, the cell-level
#'   relative effects in \code{x$effects} are plotted.
#' @param xlab Optional x-axis label.
#' @param ylab Optional y-axis label. The default is \code{"Relative effect"}.
#' @param main Optional plot title.
#' @param legend.title Optional legend title.
#' @param ref.line Optional horizontal reference line. The default is \code{0.5}.
#' @param ref.lty Line type for the reference line.
#' @param ref.col Colour of the reference line.
#' @param ... Further arguments.
#'
#' @details
#' The default plot displays the estimated cell-level relative effects. If
#' \code{term} is supplied, the plot displays factor-specific relative effects
#' and confidence intervals for the selected main effects or interactions. This
#' requires that the model was fitted with \code{Factor.Information = TRUE}. 
#' The horizontal reference line at 0.5 indicates no tendency relative to the
#' reference distribution. Values above 0.5 indicate a tendency toward larger
#' responses, whereas values below 0.5 indicate a tendency toward smaller
#' responses.
#'
#' @examples
#' \donttest{
#' data(shoulder)
#'
#' fit <- nparLD(
#'   resp ~ group1 * group2 * time,
#'   data = shoulder,
#'   subject = "subject",
#'   hypothesis = "H0p",
#'   Factor.Information = TRUE
#' )
#'
#' ## Cell-level relative effects
#' plot(fit)
#'
#' ## Factor-specific relative effects and confidence intervals
#' plot(fit, term = "time")
#' plot(fit, term = "group1:time")
#' }
#'
#' @export
plot.nparld_fit <- function(x,
                            term = NULL,
                            xlab = NULL,
                            ylab = "Relative effect",
                            main = NULL,
                            legend.title = NULL,
                            ref.line = 0.5,
                            ref.lty = "dashed",
                            ref.col = "grey40",
                            ...) {

  if (is.null(term)) {
    return(.nparld_plot_cell_effects(
      x = x,
      xlab = xlab,
      ylab = ylab,
      main = main,
      legend.title = legend.title,
      ref.line = ref.line,
      ref.lty = ref.lty,
      ref.col = ref.col,
      ...
    ))
  }

  .nparld_plot_factor_information(
    x = x,
    term = term,
    reference = ref.line,
    ylab = ylab,
    xlab = xlab,
    main = main,
    ref.lty = ref.lty,
    ref.col = ref.col,
    ...
  )
}

.nparld_plot_cell_effects <- function(x,
                                      xlab = NULL,
                                      ylab = "Relative effect",
                                      main = NULL,
                                      legend.title = NULL,
                                      ref.line = 0.5,
                                      ref.lty = "dashed",
                                      ref.col = "grey40",
                                      ...) {
if (!requireNamespace("ggplot2", quietly = TRUE)) {
  stop("Package 'ggplot2' is required for plotting.", call. = FALSE)
}

  eff <- x$effects
  has_ci <- all(c("lower", "upper") %in% names(eff))

  non_factor_cols <- c(
    "n", "NAs",
    "Nsubj", "lambda", "Nobs", "Nmiss",
    "ScoreMean", "RTE", "std.error", "lower", "upper"
  )

  factor_cols <- setdiff(names(eff), non_factor_cols)
  nfac <- length(factor_cols)

  pd <- ggplot2::position_dodge(width = 0.3)

  p <- ggplot2::ggplot(eff, ggplot2::aes(y = RTE))

  # LD-F1
  if (nfac == 1) {
    p <- p +
      ggplot2::aes(x = .data[[factor_cols[1]]], group = 1) +
      ggplot2::geom_point(size = 3) +
      ggplot2::geom_line()

    if (is.null(xlab)) xlab <- factor_cols[1]
  }

  # two factors
  if (nfac == 2) {
    p <- p +
      ggplot2::aes(
        x = .data[[factor_cols[2]]],
        color = .data[[factor_cols[1]]],
        group = .data[[factor_cols[1]]]
      ) +
      ggplot2::geom_point(size = 3, position = pd) +
      ggplot2::geom_line(position = pd)

    if (is.null(xlab)) xlab <- factor_cols[2]
    if (is.null(legend.title)) legend.title <- factor_cols[1]
  }

  # three or more factors
  if (nfac >= 3) {
    p <- p +
      ggplot2::aes(
        x = .data[[factor_cols[2]]],
        color = .data[[factor_cols[3]]],
        group = .data[[factor_cols[3]]]
      ) +
      ggplot2::geom_point(size = 3, position = pd) +
      ggplot2::geom_line(position = pd) +
      ggplot2::facet_grid(
        rows = ggplot2::vars(.data[[factor_cols[1]]])
      )

    if (is.null(xlab)) xlab <- factor_cols[2]
    if (is.null(legend.title)) legend.title <- factor_cols[3]
  }

  if (has_ci) {
    p <- p +
      ggplot2::geom_errorbar(
        ggplot2::aes(ymin = lower, ymax = upper),
        width = 0.2,
        position = pd
      )
  }

  if (!is.null(ref.line)) {
    p <- p +
      ggplot2::geom_hline(
        yintercept = ref.line,
        linetype = ref.lty,
        colour = ref.col
      )
  }



  p +
    ggplot2::labs(
      title = main,
      x = xlab,
      y = ylab,
      color = legend.title
    ) +
    ggplot2::theme_bw()
}

.nparld_plot_factor_information <- function(x,
                                            term,
                                            reference = 0.5,
                                            ylab = "Relative effect",
                                            xlab = NULL,
                                            main = NULL,
                                            ref.lty = "dashed",
                                            ref.col = "grey40",
                                            ...) {

  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    stop("Package 'ggplot2' required for plotting.", call. = FALSE)
  }

  if (is.null(x$factor.info)) {
    stop(
      "No factor-specific information is available. ",
      "Refit the model with Factor.Information = TRUE.",
      call. = FALSE
    )
  }

  if (x$hypothesis != "H0p") {
    stop(
      "Factor-information confidence interval plots are available ",
      "only for hypothesis = 'H0p'.",
      call. = FALSE
    )
  }

  dat <- .nparld_collect_factor_information(x$factor.info, term)

estimate_col <- intersect(
  c("Rel.Effect", "RTE", "Estimate", "estimate"),
  names(dat)
)[1]

lower_col <- intersect(
  c("Lower", "lower", "lower.CL", "Lower.CL", "Lower.CI", "lower.CI"),
  names(dat)
)[1]

upper_col <- intersect(
  c("Upper", "upper", "upper.CL", "Upper.CL", "Upper.CI", "upper.CI"),
  names(dat)
)[1]
  if (is.na(estimate_col) || is.na(lower_col) || is.na(upper_col)) {
    stop(
      "Could not identify estimate and confidence interval columns in factor.info.",
      call. = FALSE
    )
  }

stat_cols <- c(
  "term", "Nsubj", "lambda", "Nobs", "Nmiss",
  "ScoreMean", "RTE", "Rel.Effect", "Estimate", "estimate",
  "std.error", "Std.Error", "SE",
  "lower", "Lower", "lower.CL", "Lower.CL", "Lower.CI", "lower.CI",
  "upper", "Upper", "upper.CL", "Upper.CL", "Upper.CI", "upper.CI"
)

if (!".label" %in% names(dat)) {
  label_cols <- setdiff(names(dat), stat_cols)

  if (length(label_cols) == 0L) {
    dat$.label <- seq_len(nrow(dat))
  } else {
    dat$.label <- apply(dat[label_cols], 1, paste, collapse = ":")
  }
}

  dat$.label <- factor(dat$.label, levels = unique(dat$.label))

  if (is.null(main)) {
  main <- paste("Estimated relative effects:", paste(term, collapse = ", "))
}

ggplot2::ggplot(
  dat,
  ggplot2::aes(
    x = .data[[".label"]],
    y = .data[[estimate_col]],
    ymin = .data[[lower_col]],
    ymax = .data[[upper_col]]
  )
) +
  ggplot2::geom_hline(
    yintercept = reference,
    linetype = ref.lty,
    colour = ref.col
  ) +
  ggplot2::geom_pointrange() +
  ggplot2::facet_wrap(ggplot2::vars(.data[["term"]]), scales = "free_x") +
    ggplot2::labs(
      x = xlab,
      y = ylab,
      title = main
    ) +
    ggplot2::coord_cartesian(ylim = c(0, 1)) +
    ggplot2::theme_bw()
}

.nparld_collect_factor_information <- function(factor.info, term) {

  if (is.list(factor.info) && !is.data.frame(factor.info)) {

    missing_terms <- setdiff(term, names(factor.info))

    if (length(missing_terms) > 0L) {
      stop(
        "The requested term is not available in factor.info: ",
        paste(missing_terms, collapse = ", "),
        call. = FALSE
      )
    }

dat <- do.call(rbind, lapply(term, function(tt) {
  z <- factor.info[[tt]]
  z <- as.data.frame(z)

  z$.label <- rownames(z)
  z$term <- tt

  z
}))

rownames(dat) <- NULL
return(dat)

    rownames(dat) <- NULL
    return(dat)
  }

  if (is.data.frame(factor.info)) {

    if (!"term" %in% names(factor.info)) {
      stop(
        "If factor.info is a data frame, it must contain a column named 'term'.",
        call. = FALSE
      )
    }

    dat <- factor.info[factor.info$term %in% term, , drop = FALSE]

    if (nrow(dat) == 0L) {
      stop(
        "The requested term is not available in factor.info: ",
        paste(term, collapse = ", "),
        call. = FALSE
      )
    }

    rownames(dat) <- NULL
    return(dat)
  }

  stop("Unsupported factor.info structure.", call. = FALSE)
}



##################################################################
# Plot simultaneous confidence intervals

#' Plot nparLD multiple contrast results
#'
#' Displays simultaneous confidence intervals for local contrasts from a
#' multiple contrast test procedure. The plot shows the contrast estimates
#' together with their simultaneous confidence limits and a vertical reference
#' line at zero. Intervals excluding zero are highlighted by default.
#'
#' @param x An object of class \code{"nparld_mctp"}.
#' @param xlab Label for the x-axis. The default is \code{"Contrast effect"}.
#' @param ylab Label for the y-axis. The default is an empty label.
#' @param main Optional plot title.
#' @param ref.line Optional vertical reference line. The default is \code{0}.
#' @param ref.lty Line type for the reference line.
#' @param ref.col Colour of the reference line.
#' @param pch Plotting character.
#' @param lwd Line width.
#' @param col Optional colors for confidence intervals and point estimates. If
#'   \code{NULL}, intervals excluding zero are shown in red and intervals
#'   including zero are shown in black.
#' @param ... Further graphical arguments.
#'
#' @details
#' Displays simultaneous confidence intervals for the local contrast results
#' returned by the multiple contrast test procedure. The vertical reference line
#' at zero indicates the null value for contrast effects.
#'
#' @examples
#' \donttest{
#' data(shoulder)
#'
#' fit <- nparLD(
#'   resp ~ group1 * group2 * time,
#'   data = shoulder,
#'   subject = "subject",
#'   hypothesis = "H0p",
#'   contrast = list("time", "Dunnett")
#' )
#'
#' plot(fit$MCTP)
#' }
#'
#' @export
plot.nparld_mctp <- function(x,
                             xlab = "Contrast effect",
                             ylab = "",
                             main = NULL,
                             ref.line = 0,
                             ref.lty = 2,
                             ref.col = "gray50",
                             pch = 19,
                             lwd = 2,
                             col = NULL,
                             ...) {

  if (is.null(x$Local.Results) || nrow(x$Local.Results) == 0L) {
    stop("No local contrast results available.", call. = FALSE)
  }

  res <- x$Local.Results

  if (!all(c("Estimate", "lower", "upper") %in% names(res))) {
    stop(
      "No simultaneous confidence intervals available for this object.",
      call. = FALSE
    )
  }

  est <- res$Estimate
  lower <- res$lower
  upper <- res$upper

  labs <- rownames(res)
  if (is.null(labs) || any(!nzchar(labs))) {
    labs <- paste("C", seq_len(nrow(res)))
  }

  k <- length(est)
  ypos <- rev(seq_len(k))

  if (is.null(col)) {
    sig <- !(lower <= ref.line & upper >= ref.line)
    col <- ifelse(sig, "red3", "black")
  }

  xr <- range(c(lower, upper, ref.line), na.rm = TRUE)

  if (is.null(main)) {
    main <- if (!is.null(x$Factor)) {
      paste("Simultaneous confidence intervals for", x$Factor)
    } else {
      "Simultaneous confidence intervals"
    }
  }

  oldpar <- graphics::par(no.readonly = TRUE)
  on.exit(graphics::par(oldpar), add = TRUE)

  graphics::plot(
    NA,
    xlim = xr,
    ylim = c(0.5, k + 0.5),
    yaxt = "n",
    ylab = ylab,
    xlab = xlab,
    main = main,
    ...
  )

  graphics::axis(2, at = ypos, labels = labs, las = 1)
  graphics::abline(v = ref.line, lty = ref.lty, col = ref.col)

  graphics::segments(lower, ypos, upper, ypos, lwd = lwd, col = col)

  cap <- 0.12
  graphics::segments(lower, ypos - cap, lower, ypos + cap, lwd = lwd, col = col)
  graphics::segments(upper, ypos - cap, upper, ypos + cap, lwd = lwd, col = col)

  graphics::points(est, ypos, pch = pch, col = col)

  invisible(x)
}

---
title: "Nonparametric Longitudinal Data Analysis with nparLD"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Nonparametric Longitudinal Data Analysis with nparLD}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```


```{r setup}
library(nparLD)
```


## Introduction

The `nparLD` package provides nonparametric methods for longitudinal and
repeated-measures data in factorial experiments. It implements inference for
hypotheses in marginal distribution functions and in unweighted relative
marginal effects. The redesigned interface uses a general formula syntax and
supports crossed factorial designs, incomplete observations, dependent
replicates, multiple contrast procedures, and simultaneous confidence intervals.

The main function is nparLD(). It supports crossed factorial designs with
whole-plot and subplot factors, incomplete observations, dependent replicate
measurements, rank- and pseudo-rank-based inference, tests in marginal
distribution functions, tests in unweighted relative marginal effects, and
graphical summaries of estimated effects and contrasts.

## Data structure and formula interface

The data must be supplied in long format. Each row corresponds to one observed
measurement. At minimum, the data frame contains a response variable, a subject
identifier, and one or more design factors. Repeated measurements are represented
by one or more within-subject factors. Whole-plot factors, such as treatment
groups or strata, vary between subjects.

A typical call has the form


```{r, eval=FALSE}

nparLD(response ~ factor1 * factor2,
       data = dat,
       subject = "subject")

```
       

The formula determines the factorial structure of the analysis. Interactions are
specified in the usual R formula notation. Factor ordering is controlled by the
levels of the corresponding variables in the input data. Thus, when a specific
ordering of time points, treatment groups, or ordinal categories is required,
the variables should be converted to factors with the desired level order before
calling `nparLD()`.

## Classical designs and the formula interface

Earlier versions of `nparLD` used design-specific functions for common
longitudinal layouts. In the redesigned version, these designs are specified
through the same formula interface allowing for missing values and possibly 
dependent replicates.

| Classical design | Formula                         |
| ---------------- | ------------------------------- |
| `LD-F1`          | `resp ~ time`                   |
| `F1-LD-F1`       | `resp ~ group * time`           |
| `LD-F2`          | `resp ~ time1 * time2`          |
| `F1-LD-F2`       | `resp ~ group * time1 * time2`  |
| `F2-LD-F1`       | `resp ~ group1 * group2 * time` |

Factor ordering is controlled by setting factor levels in the input data before
calling `nparLD()`.

## Marginal distributions and relative effects

For each factorial cell, the basic distributional object is the marginal
distribution of the response in that cell. The package uses the normalized
marginal distribution function

$$
\frac{1}{2}{F_{is}^{+}(x) + F_{is}^{-}(x)},
$$

where

\[
F_{is}^{+}(x) = P(X_{is} \le x),
\qquad
F_{is}^{-}(x) = P(X_{is} < x).
\]

This convention corresponds to mid-ranks and provides a common treatment of
continuous, discrete, ordinal, and tied responses.

Relative marginal Mann-Whitney effects describe the position of a marginal distribution
relative to a reference distribution. Values larger than 0.5 indicate a tendency
toward larger responses than the reference, while values smaller than 0.5
indicate a tendency toward smaller responses. In factorial longitudinal designs,
the choice of the reference distribution is important and leads to the distinction
between weighted and unweighted effects.


## Weighted and unweighted relative effects

Relative effects are defined with respect to a reference distribution. In a
factorial longitudinal design, this reference distribution can be formed in two
different ways.

The **weighted relative effect** uses the sample-size weighted reference
distribution. Marginal distributions from cells with larger sample sizes
therefore contribute more strongly to the reference distribution than cells with
smaller sample sizes. Weighted effects are closely connected to ordinary ranks:
they describe the relative position of a cell distribution within the empirical
pooled distribution induced by the observed sample sizes.

This makes weighted effects useful as descriptive summaries of the observed
data. However, they depend on the allocation of sample sizes across cells. In an
unbalanced design, the same set of marginal distributions could lead to different
weighted effects if the sample-size allocation were changed. For this reason,
weighted effects are not used as the target of hypotheses in relative marginal
effects.

The **unweighted relative effect** uses an equally weighted reference
distribution. Each factorial cell contributes equally to the reference
distribution, independently of its sample size. The resulting effects therefore
refer to the factorial marginal distributions themselves rather than to the
particular sample-size allocation of the study. This is the effect scale used
for hypotheses in relative marginal effects.

In `nparLD`, hypotheses specified by

```{r, eval = FALSE}
hypothesis = "H0p"
```

are hypotheses in unweighted relative marginal effects. These effects are
estimated by pseudo-ranks. Weighted effects, in contrast, are reported as
descriptive rank-based summaries and are naturally associated with hypotheses in
marginal distribution functions,

```{r, eval = FALSE}
hypothesis = "H0F"
```

when the interest lies in comparing the underlying marginal distributions.

The distinction is most important in unbalanced designs. In balanced designs,
weighted and unweighted reference distributions often lead to similar
interpretations. In unbalanced designs, however, weighted effects partly reflect
the empirical sample-size allocation, whereas unweighted effects keep the
factorial cells on equal footing.



## Hypotheses H0F and H0p


The argument hypothesis = "H0F" specifies hypotheses in marginal distribution
functions. In abstract notation, these hypotheses have the form

\[
H_0^F : \mathbf{C F} = \mathbf{0},
\]

where \(\mathbf{F}\) collects the marginal distribution functions and \(\mathbf{C}\) is a contrast
matrix. Such hypotheses compare complete marginal distributions.

The argument hypothesis = "H0p" specifies hypotheses in unweighted relative
marginal effects,

\[
H_0^p : \mathbf{C \psi} = \mathbf{0},
\]

where \(\mathbf{\psi}\) collects the unweighted relative marginal effects. These hypotheses
focus on the relative positions of the marginal distributions with respect to a
common unweighted reference distribution. They are especially useful for effect
interpretation, multiple contrasts, simultaneous confidence intervals, and
plots. Note that this null hypothesis does not imply equal marginal distributions and allows for 
unequal variances and/or other moments even under the null hypothesis. Therefore,
testing `"H0p"` is also known as the nonparametric Behrens-Fisher problem. 

The two hypothesis types therefore answer related but distinct questions.
H0F asks whether marginal distribution functions are equal in the sense
specified by the contrast matrix. H0p asks whether the corresponding
unweighted relative marginal effects satisfy the specified contrast.


## A one-factor longitudinal example

```{r dental-example}
data(dental)

fit_dental <- nparLD(
  resp ~ time,
  data = dental,
  subject = "subject",
  hypothesis = "H0p",
  covariance = TRUE
)

fit_dental
```

## A factorial longitudinal example

```{r shoulder-example, fig.width = 7, fig.height = 5}
data(shoulder)

fit_shoulder <- nparLD(
  resp ~ group1 * group2 * time,
  data = shoulder,
  subject = "subject",
  hypothesis = "H0p",
  contrast = list("group1:time")
)

fit_shoulder
plot(fit_shoulder)
plot(fit_shoulder$MCTP)
```

## Missing observations

Longitudinal studies often contain incomplete observation patterns. A subject may
miss one or more visits, or a measurement may be unavailable for a particular
condition. The redesigned `nparLD()` function allows such incomplete
subject-condition cells. Missing response values should be coded in the data set
in the usual R way, for example as `NA`.

If a subject has a missing response at a particular time point or condition, the
corresponding row should still contain the relevant factor-level information.
Thus, the subject identifier, time point, group, and any other design factors
should remain available in the data set, while only the response value is set to
`NA`. This allows `nparLD()` to recognize the intended longitudinal design and
the incomplete subject-condition cell automatically.

Observed measurements contribute to the estimation of the corresponding marginal
distributions and effects, while missing subject-condition cells simply do not
contribute observations for that cell. The analysis is still based on the
independent sampling units, that is, on subjects. Thus, incomplete subjects are
not automatically removed from the data as long as they contribute observations
to at least part of the design. This is useful in longitudinal applications
where complete-case analysis would discard a substantial amount of information.
The implemented inference procedures are valid under the missing completely at random
mechanism.


```{r missing-example}
set.seed(123)

dat_miss <- dental
dat_miss$resp[c(2, 7, 12)] <- NA

fit_miss <- nparLD(
  resp ~ time,
  data = dat_miss,
  subject = "subject",
  hypothesis = "H0p"
)

fit_miss
```
The descriptive table in the output lists the number of subjects and missing values for every condition.


## Dependent replicate measurements

In some longitudinal experiments, more than one measurement is available for the 
same subject under the same condition. Examples include repeated technical measurements, 
multiple regions or lesions within a patient, or several readings taken at the same visit. 
These measurements are replicate observations within a subject-condition cell. 
They should not be treated as independent subjects, because observations from the same subject are generally dependent. Replicate measurements are specified by the `replicate` argument. The replicate variable identifies repeated observations within the same subject-condition cell.

```{r, eval = FALSE}
nparLD(response ~ group * time,
       data = dat,
       subject = "subject",
       replicate = "replicate")
```

For hypotheses in relative marginal effects, the argument `cell.weights`
determines how replicate measurements contribute to the estimated effects.

With

```{r, eval = FALSE}
cell.weights = "subjects"
```

replicate scores are first averaged within each observed subject-condition cell.
The resulting effect describes a typical subject-condition cell. This is the
natural choice when subjects are the primary sampling units and replicate
measurements are repeated observations within those units.

With

```{r, eval = FALSE}
cell.weights = "observations"
```

all replicate observations contribute directly. Subject-condition cells with
more replicate observations therefore receive more weight. This targets a
typical replicate observation and is appropriate only when the replicate
observations themselves are part of the scientific target.

For hypotheses in marginal distribution functions, dependent replicates are
handled through subject-level score averaging. Thus, inference remains based on
the independent subject-level units.


```{r replicate-example}

data(brdu)

fit_brdu_subjects <- nparLD(
  resp ~ dose,
  data = brdu,
  subject = "culture",
  replicate = "replicate",
  hypothesis = "H0p",
  cell.weights = "subjects"
)


fit_brdu_observations <- nparLD(
  resp ~ dose,
  data = brdu,
  subject = "culture",
  replicate = "replicate",
  hypothesis = "H0p",
  cell.weights = "observations"
)

fit_brdu_subjects
fit_brdu_observations

```

For relative marginal effects, `cell.weights = "subjects"` targets a typical
subject-condition cell, while `cell.weights = "observations"` targets a typical
replicate observation. In the example, the numbers of replicates are identical 
and therefore, the results are identical.

## Term-specific effect plots

The default plot method displays the estimated cell-level relative effects.

```{r plot-cell-effects, fig.width=7, fig.height=4}
plot(fit_shoulder)
```

If factor-specific information is requested by setting `Factor.Information = TRUE`,
the plot method can also display estimated relative effects and confidence
intervals for selected model terms. This is useful when the scientific question
concerns a main effect or interaction rather than all factorial cells.


```{r plot-term effects, fig.width=7, fig.height=4}
fit_shoulder_info <- nparLD(
  resp ~ group1 * group2 * time,
  data = shoulder,
  subject = "subject",
  hypothesis = "H0p",
  Factor.Information = TRUE
)

plot(fit_shoulder_info, term = "group1:time")
```

Several terms can be displayed in one faceted plot.

```{r plot-cell-effectsseveral, fig.width=7, fig.height=4}
plot(fit_shoulder_info, term = c("time", "group1:time"))
```
The horizontal reference line at 0.5 marks the value corresponding to no tendency
relative to the reference distribution. Values above 0.5 indicate a tendency
toward larger responses, whereas values below 0.5 indicate a tendency toward
smaller responses. The plots are `ggplot2` plots and hence can be adapted (e.g., in crease font) in the usual way. 

## Test statistics and multiple contrasts

For each specified hypothesis, `nparLD()` returns Wald-type and ANOVA-type test
statistics. Both statistics are based on the estimated (relative marginal Mann-Whitney)
effect vector and its estimated covariance matrix, but they use this information differently.
Test procedures testing `"H0F"` estimate the covariance matrix under this null hypothesis, 
whereas all procedures testing `"H0p"` use a covariance matrix estimator that is consistent
under arbitrary (but fixed) alternatives.


The Wald-type statistic uses the full covariance structure of the estimated
effects. In general form, it can be written as

\[
Q_N =
N \widehat{\mathbf{\theta}}^\top C^\top
(\mathbf{C} \widehat{\mathbf{V}}_N\mathbf{C}^\top)^+
\mathbf{C} \widehat{\mathbf{\theta}},
\]

where \(\widehat{\mathbf{\theta}}\) denotes the estimated target vector, \(\widehat{\mathbf{V}}_N\) is the
estimated covariance matrix, \(\mathbf{C}\) is the contrast matrix defining the
hypothesis, and \((\cdot)^+\) denotes a generalized inverse.

The ANOVA-type statistic is based on a quadratic form using the projection
matrix associated with the hypothesis,

\[
A_N =
\frac{
N \widehat{\mathbf{\theta}}^\top T \widehat{\mathbf{\theta}}
}{
\operatorname{tr}(\mathbf{T} \widehat{\mathbf{V}}_N)}.
\]

It provides an alternative approximation that is often used in nonparametric
factorial methods, especially when sample sizes are small.

In addition to global tests for main effects and interactions, `nparLD()` can
compute multiple contrast procedures for selected factors or interactions. The
argument `contrast` specifies the effect for which local comparisons should be
computed. For a selected factor or interaction, let \(\mathbf{K}\) denote the contrast matrix used
for the multiple contrast procedure. The estimated local contrasts are

\[
\widehat{\mathbf{\delta}}
=
\mathbf{K} \widehat{\mathbf{\theta}},
\]

with covariance estimator

\[
\widehat{\mathbf{\Sigma}}_{\delta}
=
\mathbf{K} \widehat{\mathbf{V}}_N \mathbf{K}^\top .
\]

The corresponding studentized contrast statistics are

\[
T_\ell
=
\frac{
\sqrt{N}\widehat{\delta}_\ell
}{
\sqrt{\widehat{\Sigma}_{\delta,\ell\ell}}
},
\qquad
\ell = 1,\ldots,q,
\]

where \(q\) is the number of local contrasts. Simultaneous confidence intervals
are obtained from the joint distribution of these studentized statistics, using
the estimated correlation matrix derived from \(\widehat{\mathbf{\Sigma}}_{\delta}\).



For example,

```{r, eval = FALSE}
contrast = list("time", "Dunnett")
```

requests Dunnett-type contrasts for the levels of `time`, while

```{r, eval = FALSE}
contrast = list("group:time")
```

requests a multiple contrast procedure for the interaction between `group` and
`time`.

When multiple contrasts are requested, the output contains a global multiple
contrast test and local contrast estimates with standard errors, simultaneous
confidence intervals, test statistics, adjusted p-values, and degrees of
freedom. The contrast results can also be plotted:

```{r, eval = FALSE}
plot(fit$MCTP)
```

Note that all procedures testing the null hypothesis `"H0F"` can not yield 
confidence intervals for the relative marginal effects.


## A real-data example: shoulder tip pain

The `shoulder` data illustrate a factorial longitudinal design with two
whole-plot factors and one repeated-measures factor. The response variable
`resp` is a shoulder pain score measured repeatedly over time. The variables
`group1` and `group2` define between-subject factors, while `time` is the
within-subject factor.

```{r shoulder-data}
data(shoulder)

str(shoulder)
```

This corresponds to an `F2-LD-F1` design in the classical terminology: two
whole-plot factors and one longitudinal factor. In the redesigned interface, the
model is specified directly by the formula

```{r shoulder-h0f}
fit_shoulder_F <- nparLD(
  resp ~ group1 * group2 * time,
  data = shoulder,
  subject = "subject",
  hypothesis = "H0F"
)

fit_shoulder_F
```

The analysis with `hypothesis = "H0F"` tests hypotheses in the marginal
distribution functions. These tests compare the complete marginal distributions
defined by the factorial cells and may therefore detect differences beyond
location shifts.

For effect-based interpretation, we can formulate the analysis in terms of
unweighted relative marginal effects:

```{r shoulder-h0p}
fit_shoulder_p <- nparLD(
  resp ~ group1 * group2 * time,
  data = shoulder,
  subject = "subject",
  hypothesis = "H0p"
)

fit_shoulder_p
```

The estimated relative effects can be displayed graphically:

```{r shoulder-plot, fig.width = 7, fig.height = 5}
plot(fit_shoulder_p)
```

A value above 0.5 indicates a tendency toward larger responses relative to the
unweighted reference distribution, while a value below 0.5 indicates a tendency
toward smaller responses.

Finally, we can request multiple contrasts for an interaction. For example, the
following call computes contrasts for the interaction between `group1` and
`time`:

```{r shoulder-mctp}
fit_shoulder_contrast <- nparLD(
  resp ~ group1 * group2 * time,
  data = shoulder,
  subject = "subject",
  hypothesis = "H0p",
  contrast = list("group1:time")
)

fit_shoulder_contrast$MCTP
```

```{r shoulder-mctp-plot, fig.width = 7, fig.height = 5}
plot(fit_shoulder_contrast$MCTP)
```

This output contains both a global multiple contrast test and local contrast
results with simultaneous confidence intervals.

## Interpreting the output

The object returned by `nparLD()` contains several components. Printing the
object gives a compact summary of the design, the selected hypothesis type, the
estimated effects, and the available test results. The most commonly used
components are `effects`, `WTS`, `ATS`, and, when multiple contrasts are
requested, `MCTP`.

The component `effects` contains the estimated relative effects for the
factorial cells. It includes the factor-level combinations, the number of
contributing subjects and observations, the number of missing observations, the
estimated relative effect, and its standard error. For analyses with
`hypothesis = "H0p"`, confidence limits are also returned.

```{r effects-component}
head(fit_shoulder_p$effects)
```

The components `WTS` and `ATS` contain the global tests for the main effects and
interactions defined by the model formula.

```{r test-components}
fit_shoulder_p$WTS
fit_shoulder_p$ATS
```

The Wald-type statistic uses the estimated covariance matrix directly, whereas
the ANOVA-type statistic uses a trace-based approximation. In practice, both
statistics are useful summaries of the global hypotheses. Differences between
the two may occur, especially in small samples, unbalanced designs, or settings
with singular covariance structures.

When multiple contrasts are requested, the component `MCTP` contains the
corresponding contrast results.

```{r mctp-component}
fit_shoulder_contrast$MCTP
```

The local contrast table contains the estimated contrasts, standard errors,
simultaneous confidence limits, test statistics, adjusted p-values, and degrees
of freedom. These results are most naturally interpreted on the relative-effect
scale. A simultaneous confidence interval that does not contain zero indicates a
contrast that is significant after adjustment for the selected family of
comparisons. 

The estimated covariance matrix can be included in the output by setting
`covariance = TRUE`. This option is mainly intended for methodological checks,
advanced post-processing, or reproducibility of the reported test statistics.

For graphical summaries, `plot()` can be applied to the fitted object or to the
multiple contrast component.

```{r output-plots, eval = FALSE}
plot(fit_shoulder_p)
plot(fit_shoulder_contrast$MCTP)
```

## Practical recommendations

The redesigned `nparLD()` function provides several analysis options. The
following recommendations may help to choose an appropriate specification.

Use `hypothesis = "H0F"` when the primary question concerns equality of marginal
distribution functions. These hypotheses compare complete marginal
distributions. This option is useful when the
scientific question is distributional and not restricted to a single effect
measure. Classical rank procedures test null hypotheses in distribution functions.

Use `hypothesis = "H0p"` when the primary question concerns relative marginal
effects. These hypotheses are formulated for unweighted relative effects and are
therefore particularly useful for effect interpretation, graphical summaries,
multiple contrasts, and simultaneous confidence intervals. The resulting
estimates can be interpreted on the relative-effect scale: values above 0.5
indicate a tendency toward larger responses than the reference distribution,
whereas values below 0.5 indicate a tendency toward smaller responses.

Weighted relative effects should be interpreted as descriptive summaries. They
are based on the sample-size weighted reference distribution and are closely
connected to ordinary ranks. Because they depend on the observed sample-size
allocation, they are not used as the inferential target for hypotheses in
relative marginal effects.

Unweighted relative effects should be used when inference on relative effects is
desired. They are based on an equally weighted reference distribution over the
factorial cells and are estimated by pseudo-ranks. This keeps the interpretation
of the effects separate from the sample-size allocation, which is especially
important in unbalanced designs.

For incomplete longitudinal data, missing response values should be coded as
`NA`, while the subject identifier and the relevant factor-level information
should remain in the data set. This allows `nparLD()` to identify the incomplete
subject-condition cells automatically. Subjects with partially observed
trajectories are not discarded completely as long as they contribute observations
to at least part of the design.

For dependent replicate measurements, use the `replicate` argument to identify
multiple observations within the same subject-condition cell. With
`cell.weights = "subjects"`, replicate scores are averaged within
subject-condition cells, so that the analysis targets a typical
subject-condition cell. With `cell.weights = "observations"`, all replicate
observations contribute directly, so that the analysis targets a typical
replicate observation.

In most applications, a useful workflow is to first identify the primary
research questions of interest and translate them into suitable contrasts. The
multiple contrast procedures can then be used for selected factors or
interactions to obtain local comparisons and simultaneous confidence intervals.
Graphical displays of the estimated effects and simultaneous confidence
intervals can help to communicate the results.

Global tests in `WTS` and `ATS` are useful when the global null hypothesis is
the primary or sole object of interest. In contrast, when the scientific
question concerns specific group, time, or interaction comparisons, contrast
procedures usually provide the more directly interpretable results.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{panic}
\alias{panic}
\title{Panic Disorder Study I}
\format{
A data frame with variables:
\describe{
\item{resp}{Clinical Global Impression score.}
\item{time}{Visit or week of assessment.}
\item{subject}{Patient identifier.}
}
}
\usage{
panic
}
\description{
Clinical Global Impression scores from patients with panic disorder and
agoraphobia. The data illustrate a one-factor longitudinal design with
ordinal responses.
}
\examples{
data(panic)

\dontrun{
fit <- nparLD(resp ~ time,
              data = panic,
              subject = "subject",
              hypothesis = "H0F", contrast =list("time", "Dunnett"))
fit
}

\dontrun{
fit <- nparLD(resp ~ time,
              data = panic,
              subject = "subject",
              hypothesis = "H0p",
              contrast = list("time", "Dunnett"))
fit
plot(fit)
plot(fit$MCTP)
}

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Utility.R
\name{print.nparld_mctp}
\alias{print.nparld_mctp}
\title{Print nparLD multiple contrast results}
\usage{
\method{print}{nparld_mctp}(x, digits = 4, show.matrix = FALSE, ...)
}
\arguments{
\item{x}{An object of class \code{"nparld_mctp"}.}

\item{digits}{Number of digits used for printing numerical results.}

\item{show.matrix}{Logical. If \code{TRUE}, print the contrast matrix used in the
multiple contrast procedure.}

\item{...}{Further arguments.}
}
\description{
Prints the global and local results of a multiple contrast test procedure.
The local results include contrast estimates, standard errors, simultaneous
confidence limits, test statistics, adjusted p-values, and degrees of freedom.
The contrast matrix can optionally be displayed with
\code{show.matrix = TRUE}.
}
\details{
The method prints the global multiple contrast test and the local contrast
results, including estimates, standard errors, simultaneous confidence
limits, test statistics, adjusted p-values, and degrees of freedom. The
contrast matrix is stored in \code{x$Contrast.Matrix} and is printed only
when \code{show.matrix = TRUE}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{plasma}
\alias{plasma}
\title{Plasma-renin Activity Study}
\format{
A data frame with variables:
\describe{
\item{resp}{Plasma-renin activity measurement.}
\item{time}{Measurement time.}
\item{group}{Drug group.}
\item{subject}{Subject identifier.}
}
}
\usage{
plasma
}
\description{
Plasma-renin activity measurements from a randomized study of healthy
non-smokers. The data illustrate a factorial longitudinal design with one
whole-plot factor and one repeated-measures factor.
}
\examples{
data(plasma)

\dontrun{
fit <- nparLD(resp ~ group * time,
              data = plasma,
              subject = "subject",
              hypothesis = "H0F", effect="weighted")
fit
}

\dontrun{
fit <- nparLD(resp ~ group * time,
              data = plasma,
              subject = "subject",
              hypothesis = "H0p",
              contrast = list("group", "Tukey"))
fit
plot(fit)
plot(fit$MCTP)
}

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nparLD.R
\name{nparLD}
\alias{nparLD}
\title{Nonparametric Analysis of Longitudinal Data}
\usage{
nparLD(
  formula,
  data,
  subject,
  replicate = NULL,
  cell.weights = c("subjects", "observations"),
  effect = c("unweighted", "weighted"),
  hypothesis = c("H0F", "H0p"),
  contrast = NULL,
  sci.method = c("fisher", "multi.t"),
  Factor.Information = FALSE,
  CI.method = c("logit", "normal"),
  alpha = 0.05,
  covariance = FALSE,
  perm.test = FALSE,
  B = 1000
)
}
\arguments{
\item{formula}{A model formula of the form \code{response ~ factors}. Factors
on the right-hand side may contain crossed whole-plot and subplot factors
and their interactions.}

\item{data}{A data frame containing the response, subject variable, design
factors, and optionally a replicate variable.}

\item{subject}{Character string specifying the subject identifier.}

\item{replicate}{Optional character string specifying the replicate identifier.
Replicates can be dependent within subject-condition cells.}

\item{cell.weights}{Character string specifying how dependent replicates are
weighted. Use \code{"subjects"} for subject-level cell weighting and
\code{"observations"} for observation-level cell weighting.
For dependent replicates under \code{hypothesis = "H0F"},
\code{cell.weights = "subjects"} averages rank scores within
subject-condition cells before covariance estimation, whereas
\code{cell.weights = "observations"} uses the sums of the observed rank
scores within subject-condition cells and therefore gives more weight to
subjects with more observed replicate measurements.The default is
\code{"subjects"}.}

\item{effect}{Character string specifying whether weighted or unweighted
relative effects are used. Use \code{"weighted"} for effects based on
ordinary ranks and a sample-size weighted reference distribution. Use
\code{"unweighted"} for effects based on pseudo-ranks and an equally
weighted factorial-cell reference distribution. Weighted effects depend on
the sample-size allocation and are mainly descriptive. Unweighted effects
are the inferential target for \code{hypothesis = "H0p"}. In balanced
designs, both effect definitions coincide. The default is
\code{effect = "unweighted"}.}

\item{hypothesis}{Character string specifying the hypothesis type. Use
\code{"H0F"} for hypotheses in marginal distribution functions and
\code{"H0p"} for hypotheses in unweighted relative marginal effects.
The latter permits unequal variances or higher-order distributional
differences under the null hypothesis and is related to the nonparametric
Behrens-Fisher problem.}

\item{contrast}{Optional list specifying a factor or interaction term for
simultaneous inference. A one-element specification such as
\code{list("time")} uses the standard hypothesis matrix for the selected
model term, that is, the same contrast structure underlying the WTS and ATS
for that term. A two-element specification such as
\code{list("time", "Tukey")} or \code{list("time", "Dunnett")} applies the
named multiple contrast procedure to the marginal relative effects of the
selected term. A numeric vector or matrix may be supplied as the second
element for user-defined contrasts.}

\item{sci.method}{Character string specifying the method for simultaneous
confidence intervals. Available choices are \code{"fisher"} and \code{"multi.t"}.
The default is \code{"fisher"}.}

\item{Factor.Information}{Logical. If \code{TRUE}, factor-specific relative
effects, standard errors, and confidence intervals are returned. These
summaries can also be displayed with \code{plot(fit, term = ...)}.}

\item{CI.method}{Character string specifying the method for confidence intervals for
relative effects. Available choices are \code{"logit"} and \code{"normal"}. The default is \code{"logit"}.}

\item{alpha}{Significance level for tests and confidence intervals. The
default is \code{alpha = 0.05}.}

\item{covariance}{Logical. If \code{TRUE}, the estimated covariance matrix
used for the selected hypothesis is included in the output. The default is
\code{FALSE}.}

\item{perm.test}{Logical. Should the paired two-time-point permutation test be
used when applicable? This option is only available for paired designs with
two time points.}

\item{B}{Number of permutation samples used for the paired permutation test.}
}
\value{
An object of class \code{"nparld_fit"}. The object is a list containing the
results of the nonparametric longitudinal analysis. The main components are:
\itemize{
\item \code{Design}: character string describing the detected longitudinal
factorial design.
\item \code{wholeplots}: names of the whole-plot factors.
\item \code{subplots}: names of the subplot or repeated-measures factors.
\item \code{text.ranks}: character string describing the type of ranks used.
\item \code{text.hypotheses}: character string describing the type of
hypotheses tested.
\item \code{N.info}: information on the number of subjects and observations.
\item \code{effects}: a data frame with estimated relative effects. This
includes the factor-level combinations, the number of contributing
subjects and observations, the number of missing observations, the mean
rank or pseudo-rank score, the estimated relative effect, and its
standard error. For \code{hypothesis = "H0p"}, confidence limits are also
returned.
\item \code{factor.info}: optional factor-specific relative effects,
standard errors, and confidence limits for main effects and interactions,
returned when \code{Factor.Information = TRUE}. These summaries can be
displayed with \code{plot(fit, term = ...)}.
\item \code{WTS}: Wald-type test statistics, degrees of freedom, and
p-values for the tested main effects and interactions.
\item \code{ATS}: ANOVA-type test statistics, denominator degrees of
freedom, and p-values for the tested main effects and interactions.
\item \code{MCTP}: multiple contrast test results, returned when
\code{contrast} is specified. These include the selected factor or
interaction, the simultaneous confidence interval method, the global
multiple contrast test, local contrast estimates with standard errors,
simultaneous confidence limits, test statistics, p-values, degrees of
freedom, and the corresponding contrast matrix.
\item \code{covariance.info}: estimated covariance matrix used for the
selected hypothesis, returned when \code{covariance = TRUE}.
\item \code{perm}: permutation test results, returned when a permutation
test is requested and applicable.
\item \code{hypothesis}: the type of hypothesis tested, either \code{"H0F"}
for hypotheses in marginal distribution functions or \code{"H0p"} for
hypotheses in unweighted relative marginal effects.
\item \code{CI.method}: method used for confidence intervals.
\item \code{.internal}: internal objects used for computation and advanced
post-processing.
}

Some components may be \code{NULL} depending on the selected hypothesis,
contrast specification, and output options.
}
\description{
Performs nonparametric inference for longitudinal or repeated-measures data
from crossed factorial experiments. The function can be used to test hypotheses in
marginal distribution functions or hypotheses in unweighted relative
marginal effects. It allows crossed whole-plot and sub-plot factors,
missing values, and dependent replicate measurements.
}
\details{
The function provides rank- and pseudo-rank-based procedures for factorial
longitudinal data. The argument \code{hypothesis = "H0F"} specifies
hypotheses in marginal distribution functions. These hypotheses compare the
complete marginal distributions and are tested by rank-based procedures.

The argument \code{hypothesis = "H0p"} specifies hypotheses in unweighted
relative marginal effects. These effects describe the relative position of
each marginal distribution with respect to a common unweighted reference
distribution and are particularly useful for effect interpretation, multiple
contrast procedures, simultaneous confidence intervals, and graphical
summaries.

The \code{contrast} argument computes a multiple contrast test along with simultaneous
confidence intervals for the selected model terms. A one-element specification such as
\code{list("time")} uses the same hypothesis matrix as the global test procedures (Wald-
and ANOVA-type statistics) for the selected term. This
provides simultaneous inference for the components of the corresponding
global null hypothesis. A two-element specification such as
\code{list("time", "Tukey")} or \code{list("time", "Dunnett")} first forms
the marginal relative effects for the selected term and then applies the
requested multiple contrast procedure. Thus, \code{list("time", "Tukey")}
gives pairwise comparisons of the marginal time effects, whereas
\code{list("time", "Dunnett")} compares the marginal time effects with the
first time point. User-defined contrast vectors or matrices can be supplied
as the second list element.

Missing observations are allowed. Incomplete subject-condition cells
contribute where observations are available, and covariance estimation is
based on the independent subject-level units. Dependent replicate
measurements can be specified using the \code{replicate} argument. For
hypotheses in relative marginal effects, \code{cell.weights = "subjects"}
targets a typical subject-condition cell, whereas
\code{cell.weights = "observations"} targets a typical replicate observation.
For hypotheses in marginal distribution functions, dependent replicates are
handled by averaging rank or pseudo-rank scores within subject-condition
cells.
In case of bivariate data (e.g., before and after measurements), the function implements
a studentized permutation test for testing either \code{hypothesis = "H0F"} or
\code{hypothesis = "H0p"}.
}
\examples{
## One repeated-measures factor: dental data
data(dental)

fit_dental <- nparLD(
  resp ~ time,
  data = dental,
  subject = "subject",
  hypothesis = "H0p"
)

fit_dental

## Crossed factorial longitudinal design: shoulder data
data(shoulder)

\dontrun{
fit_shoulder <- nparLD(
  resp ~ group1 * group2 * time,
  data = shoulder,
  subject = "subject",
  hypothesis = "H0p"
)

fit_shoulder
}

## Multiple contrast procedure for an interaction

\dontrun{
fit_contrast <- nparLD(
  resp ~ group1 * group2 * time,
  data = shoulder,
  subject = "subject",
  hypothesis = "H0p",
  contrast = list("group1:time")
)

fit_contrast$MCTP
print(fit_contrast$MCTP, show.matrix = TRUE)
}

## Missing response values
\dontrun{
dat_miss <- dental
dat_miss$resp[c(2, 7, 12)] <- NA

fit_miss <- nparLD(
  resp ~ time,
  data = dat_miss,
  subject = "subject",
  hypothesis = "H0p"
)

fit_miss
}

## Dependent replicate measurements: BrdU data
data(brdu)

\dontrun{
fit_brdu <- nparLD(
  resp ~ dose,
  data = brdu,
  subject = "culture",
  replicate = "replicate",
  hypothesis = "H0p",
  cell.weights = "subjects"
)

fit_brdu
}

\dontrun{
## Graphical displays
plot(fit_dental)
plot(fit_shoulder)
plot(fit_contrast$MCTP)
}


}
\references{
Akritas, M. G., & Brunner, E. (1997). A unified approach to rank tests for mixed models.
Journal of Statistical Planning and Inference, 61(2), 249-277.

Brunner, E., Bathke, A.C., Konietschke, F. Rank and Pseudo-Rank Procedures
for Independent Observations in Factorial Designs. Springer International Publishing, 2018.

Brunner, E., Domhof, S., & Langer, F. (2002). Nonparametric analysis of longitudinal data in factorial experiments. Wiley, New York

Brunner, E., Konietschke, F., Pauly, M., Puri, M. L. (2017). Rank-based procedures in factorial designs:
Hypotheses about non-parametric treatment effects. Journal of the Royal Statistical Society: Series B
(Statistical Methodology), 79(5), 1463-1485.

Domhof, S., Brunner, E., Osgood, D. W. (2002). Rank procedures for repeated measures with missing values.
Sociological methods & research, 30(3), 367-393.

Konietschke, F., Bathke, A. C., Hothorn, L. A., & Brunner, E. (2010).
Testing and estimation of purely nonparametric effects in repeated measures designs.
Computational Statistics & Data Analysis, 54(8), 1895-1905.

Konietschke, F., Hothorn, L. A., Brunner, E. (2012). Rank-based multiple test procedures and simultaneous confidence intervals. Electronic Journal of Statistics,
6, 738-759.

Rubarth, K., Pauly, M., & Konietschke, F. (2022). Ranking procedures for repeated measures designs with missing data:
estimation, testing and asymptotic theory.
Statistical Methods in Medical Research, 31(1), 105-118.

Rubarth, K., Sattler, P., Zimmermann, H. G., & Konietschke, F. (2021).
Estimation and testing of Wilcoxon–Mann–Whitney effects in factorial clustered data designs.
Symmetry, 14(2), 244.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{dental}
\alias{dental}
\title{Dental Growth Study}
\format{
A data frame with variables:
\describe{
\item{resp}{Distance measurement in millimeters.}
\item{time}{Age at measurement.}
\item{subject}{Subject identifier.}
}
}
\usage{
dental
}
\description{
Measurements of distances between the center of the pituitary and the
pterygomaxillary fissure in boys. The data illustrate a one-factor
longitudinal design.
}
\examples{
data(dental)

dental$time <- factor(dental$time)

\dontrun{
fit <- nparLD(resp ~ time,
              data = dental,
              subject = "subject",
              hypothesis = "H0F")
fit
}

\dontrun{
fit <- nparLD(resp ~ time,
              data = dental,
              subject = "subject",
              hypothesis = "H0p",
              contrast = list("time", "Tukey"))
fit
plot(fit)
plot(fit$MCTP)
}

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{panic2}
\alias{panic2}
\title{Panic Disorder Study II}
\format{
A data frame with variables:
\describe{
\item{resp}{Panic and agoraphobia score.}
\item{time}{Visit or week of assessment.}
\item{group}{Agoraphobia group.}
\item{subject}{Patient identifier.}
}
}
\usage{
panic2
}
\description{
Panic and agoraphobia scores from patients with panic disorder, with or
without agoraphobia. The data illustrate a factorial longitudinal design with
one whole-plot factor and one repeated-measures factor.
}
\examples{
data(panic2)

\dontrun{
fit <- nparLD(resp ~ group * time,
              data = panic2,
              subject = "subject",
              hypothesis = "H0F", effect="unweighted")
fit
}

\dontrun{
fit <- nparLD(resp ~ group * time,
              data = panic2,
              subject = "subject",
              hypothesis = "H0p",
              contrast = list("group:time"),
              Factor.Information = TRUE,
              covariance=TRUE)
fit
plot(fit)
plot(fit$MCTP)
}

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Utility.R
\name{summary.nparld_fit}
\alias{summary.nparld_fit}
\title{Summarize an nparLD fit}
\usage{
\method{summary}{nparld_fit}(object, ...)
}
\arguments{
\item{object}{An object of class \code{"nparld_fit"}.}

\item{...}{Further arguments passed to or from other methods.}
}
\value{
An object of class \code{"summary.nparld_fit"} containing selected
components of the fitted model.
}
\description{
Extracts the main components of a fitted \code{nparLD()} object into a
structured summary object. The summary is useful for inspecting, printing, or
programmatically accessing the main results without working directly with all
internal components of the fitted object.
}
\details{
The summary method returns a structured list containing the main components
of the fitted object. In contrast to \code{print()}, which is mainly intended
for console display, \code{summary()} is useful for storing or extracting the
main results programmatically.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{amylase}
\alias{amylase}
\title{Alpha-amylase Saliva Study}
\format{
A data frame with variables:
\describe{
\item{resp}{Alpha-amylase measurement.}
\item{time1}{First repeated-measures factor.}
\item{time2}{Second repeated-measures factor.}
\item{subject}{Subject identifier.}
}
}
\usage{
amylase
}
\description{
Measurements of alpha-amylase levels in saliva from healthy volunteers.
The study illustrates a two-factor longitudinal design with two repeated
measures factors.
}
\examples{
data(amylase)

fit <- nparLD(resp ~ time1 * time2,
              data = amylase,
              subject = "subject",
              effect = "weighted",
              hypothesis = "H0F")
fit

# Dunnett-type contrasts for the levels of time2
fit <- nparLD(resp ~ time1 * time2,
              data = amylase,
              subject = "subject",
              hypothesis = "H0p",
              contrast = list("time2", "Dunnett"),
              Factor.Information = TRUE)
fit
plot(fit)
plot(fit$MCTP)

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Utility.R
\name{print.nparld_factorinfo}
\alias{print.nparld_factorinfo}
\title{Print nparLD factor information}
\usage{
\method{print}{nparld_factorinfo}(x, digits = 4, ...)
}
\arguments{
\item{x}{An object of class \code{"nparld_factorinfo"}.}

\item{digits}{Number of digits used for printing numerical results.}

\item{...}{Further arguments.}
}
\description{
Prints factor-specific relative effects, standard errors, and confidence
limits for the main effects and interactions of an \code{nparLD()} fit.
Factor information is returned when the model is fitted with
\code{Factor.Information = TRUE} and can also be displayed graphically with
\code{plot(fit, term = ...)}.
}
\details{
Factor information consists of term-specific relative effects, standard
errors, and confidence limits for main effects and interactions. These
summaries are returned when \code{Factor.Information = TRUE} and can be
visualized with \code{plot(fit, term = ...)}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{brdu}
\alias{brdu}
\title{BrdU incorporation in fibroblasts}
\format{
A data frame with 60 rows and 4 variables:
\describe{
\item{resp}{BrdU incorporation response.}
\item{dose}{Dose condition, with levels \code{0}, \code{0.1}, \code{1}, and \code{10}.}
\item{culture}{Culture identifier. This is the independent experimental unit.}
\item{replicate}{Replicate identifier within each culture-dose condition.}
}
}
\usage{
brdu
}
\description{
BrdU incorporation in fibroblast cultures measured under four dose
conditions. For each culture and dose condition, three replicate measurements
are available. The data set is provided in long format and can be used to
illustrate dependent replicate measurements in \code{nparLD()}.
}
\details{
The original variables were \code{CultureNo}, \code{Replic}, \code{Dose}, and \code{Brdu}.
In the package data set, the variables were renamed to \code{culture},
\code{replicate}, \code{dose}, and \code{resp} for consistency with the examples.
}
\examples{
\dontrun{
data(brdu)

fit <- nparLD(
  resp ~ dose,
  data = brdu,
  subject = "culture",
  replicate = "replicate",
  hypothesis = "H0p",
  cell.weights = "subjects"
)

fit
}

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Utility.R
\name{print.nparld_covarianceinfo}
\alias{print.nparld_covarianceinfo}
\title{Print nparLD covariance matrix}
\usage{
\method{print}{nparld_covarianceinfo}(x, digits = 4, ...)
}
\arguments{
\item{x}{An object of class \code{"nparld_covarianceinfo"}.}

\item{digits}{Number of digits used for printing numerical results.}

\item{...}{Further arguments.}
}
\description{
Displays the estimated covariance matrix of the relative effect estimator.
}
\details{
The printed matrix is the estimated covariance matrix used for the selected
hypothesis. It is returned when \code{covariance = TRUE}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Utility.R
\name{plot.nparld_fit}
\alias{plot.nparld_fit}
\title{Plot nparLD results}
\usage{
\method{plot}{nparld_fit}(
  x,
  term = NULL,
  xlab = NULL,
  ylab = "Relative effect",
  main = NULL,
  legend.title = NULL,
  ref.line = 0.5,
  ref.lty = "dashed",
  ref.col = "grey40",
  ...
)
}
\arguments{
\item{x}{An object of class \code{"nparld_fit"}.}

\item{term}{Optional character vector specifying one or more model terms for
which factor-specific relative effects and confidence intervals should be
plotted. The requested terms require \code{Factor.Information = TRUE} in the
original call to \code{nparLD()}. If \code{term = NULL}, the cell-level
relative effects in \code{x$effects} are plotted.}

\item{xlab}{Optional x-axis label.}

\item{ylab}{Optional y-axis label. The default is \code{"Relative effect"}.}

\item{main}{Optional plot title.}

\item{legend.title}{Optional legend title.}

\item{ref.line}{Optional horizontal reference line. The default is \code{0.5}.}

\item{ref.lty}{Line type for the reference line.}

\item{ref.col}{Colour of the reference line.}

\item{...}{Further arguments.}
}
\description{
Displays graphical summaries of estimated relative effects from an
\code{nparLD()} fit. By default, the method plots the cell-level relative
effects stored in \code{x$effects}. If \code{term} is supplied, it plots
factor-specific relative effects and confidence intervals for selected main
effects or interactions. Term-specific plots require that the model was fitted
with \code{Factor.Information = TRUE}.
}
\details{
The default plot displays the estimated cell-level relative effects. If
\code{term} is supplied, the plot displays factor-specific relative effects
and confidence intervals for the selected main effects or interactions. This
requires that the model was fitted with \code{Factor.Information = TRUE}.
The horizontal reference line at 0.5 indicates no tendency relative to the
reference distribution. Values above 0.5 indicate a tendency toward larger
responses, whereas values below 0.5 indicate a tendency toward smaller
responses.
}
\examples{
\donttest{
data(shoulder)

fit <- nparLD(
  resp ~ group1 * group2 * time,
  data = shoulder,
  subject = "subject",
  hypothesis = "H0p",
  Factor.Information = TRUE
)

## Cell-level relative effects
plot(fit)

## Factor-specific relative effects and confidence intervals
plot(fit, term = "time")
plot(fit, term = "group1:time")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{edema}
\alias{edema}
\title{Postoperative Edema Study}
\format{
A data frame with variables:
\describe{
\item{resp}{Skin temperature measurement.}
\item{time1}{Hand factor.}
\item{time2}{Day factor.}
\item{group}{Treatment group.}
\item{subject}{Subject identifier.}
}
}
\usage{
edema
}
\description{
Skin-temperature measurements from patients after hand surgery. The study
compares treatment groups and includes two repeated-measures factors.
}
\examples{
data(edema)

\dontrun{
fit <- nparLD(resp ~ group * time1 * time2,
              data = edema,
              subject = "subject",
              hypothesis = "H0F")
fit
}

\dontrun{
fit <- nparLD(resp ~ group * time1 * time2,
              data = edema,
              subject = "subject",
              hypothesis = "H0p",
              contrast = list("group", "Tukey"),
              Factor.Information = TRUE)
fit
plot(fit)
plot(fit$MCTP)
}

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Utility.R
\name{print.nparld_fit}
\alias{print.nparld_fit}
\title{Print nparLD fit}
\usage{
\method{print}{nparld_fit}(x, digits = 4, ...)
}
\arguments{
\item{x}{An object of class \code{"nparld_fit"}.}

\item{digits}{Number of digits used for printed numerical results.}

\item{...}{Further arguments.}
}
\description{
Prints the main results of a fitted \code{nparLD()} model, including the
detected design, hypothesis type, ranking method, estimated relative effects,
and global WTS and ATS results. Optional components such as factor-specific
information, multiple contrast results, covariance matrices, and permutation
tests are displayed when available.
}
\details{
The print method displays the detected design, sample size information,
hypothesis type, ranking method, estimated relative effects, and global WTS
and ATS results. Optional components are printed when available, including
factor-specific information, multiple contrast results, covariance matrices,
and permutation test results.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Utility.R
\name{print.summary.nparld_fit}
\alias{print.summary.nparld_fit}
\title{Print summary of an nparLD fit}
\usage{
\method{print}{summary.nparld_fit}(x, digits = 4, ...)
}
\arguments{
\item{x}{An object of class \code{"summary.nparld_fit"}.}

\item{digits}{Number of digits used for printing numerical results.}

\item{...}{Further arguments.}
}
\description{
Prints the main components of a summarized \code{nparLD()} fit, including
design information, hypothesis type, estimated relative effects, and global
WTS and ATS results. Optional components such as factor-specific information,
multiple contrast results, covariance matrices, and permutation tests are
displayed when available.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Utility.R
\name{plot.nparld_mctp}
\alias{plot.nparld_mctp}
\title{Plot nparLD multiple contrast results}
\usage{
\method{plot}{nparld_mctp}(
  x,
  xlab = "Contrast effect",
  ylab = "",
  main = NULL,
  ref.line = 0,
  ref.lty = 2,
  ref.col = "gray50",
  pch = 19,
  lwd = 2,
  col = NULL,
  ...
)
}
\arguments{
\item{x}{An object of class \code{"nparld_mctp"}.}

\item{xlab}{Label for the x-axis. The default is \code{"Contrast effect"}.}

\item{ylab}{Label for the y-axis. The default is an empty label.}

\item{main}{Optional plot title.}

\item{ref.line}{Optional vertical reference line. The default is \code{0}.}

\item{ref.lty}{Line type for the reference line.}

\item{ref.col}{Colour of the reference line.}

\item{pch}{Plotting character.}

\item{lwd}{Line width.}

\item{col}{Optional colors for confidence intervals and point estimates. If
\code{NULL}, intervals excluding zero are shown in red and intervals
including zero are shown in black.}

\item{...}{Further graphical arguments.}
}
\description{
Displays simultaneous confidence intervals for local contrasts from a
multiple contrast test procedure. The plot shows the contrast estimates
together with their simultaneous confidence limits and a vertical reference
line at zero. Intervals excluding zero are highlighted by default.
}
\details{
Displays simultaneous confidence intervals for the local contrast results
returned by the multiple contrast test procedure. The vertical reference line
at zero indicates the null value for contrast effects.
}
\examples{
\donttest{
data(shoulder)

fit <- nparLD(
  resp ~ group1 * group2 * time,
  data = shoulder,
  subject = "subject",
  hypothesis = "H0p",
  contrast = list("time", "Dunnett")
)

plot(fit$MCTP)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{rat}
\alias{rat}
\title{Rat Growth Study}
\format{
A data frame with variables:
\describe{
\item{resp}{Body weight in grams.}
\item{time}{Week of measurement.}
\item{group}{Treatment group.}
\item{subject}{Rat identifier.}
}
}
\usage{
rat
}
\description{
Body-weight measurements from rats observed over a five-week period. The data
illustrate longitudinal growth curves in several treatment groups.
}
\examples{

\dontrun{
data(rat)

fit <- nparLD(resp ~ group * time,
              data = rat,
              subject = "subject",
              hypothesis = "H0F")
fit
}

\dontrun{
fit <- nparLD(resp ~ group * time,
              data = rat,
              subject = "subject",
              hypothesis = "H0p",
              contrast = list("group:time"))
fit
plot(fit)
plot(fit$MCTP)
}

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{respiration}
\alias{respiration}
\title{Respiratory Disorder Study}
\format{
A data frame with variables:
\describe{
\item{resp}{Ordinal health-status response.}
\item{time}{Visit number.}
\item{center}{Study center.}
\item{treatment}{Treatment group.}
\item{patient}{Patient identifier.}
}
}
\usage{
respiration
}
\description{
Ordinal health-status measurements from patients with a respiratory disorder.
The data include study center, treatment group, and repeated visits.
}
\examples{

\dontrun{
data(respiration)

fit <- nparLD(resp ~ center * treatment * time,
              data = respiration,
              subject = "patient",
              hypothesis = "H0F")
fit
}

\dontrun{
fit <- nparLD(resp ~ center * treatment * time,
              data = respiration,
              subject = "patient",
              hypothesis = "H0p",
              contrast = list("treatment", "Tukey"))
fit
plot(fit)
plot(fit$MCTP)
}

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{tree}
\alias{tree}
\title{Vitality of Treetops}
\format{
A data frame with variables:
\describe{
\item{resp}{Vitality score.}
\item{time}{Year or measurement occasion.}
\item{group}{Experimental area.}
\item{subject}{Tree identifier.}
}
}
\usage{
tree
}
\description{
Repeated vitality scores of treetops from three experimental areas. The data
illustrate an ordinal longitudinal response in several groups.
}
\examples{
data(tree)

\dontrun{
fit <- nparLD(resp ~ group * time,
              data = tree,
              subject = "subject",
              hypothesis = "H0F")
fit
}


\dontrun{
fit <- nparLD(resp ~ group * time,
              data = tree,
              subject = "subject",
              hypothesis = "H0p",
              contrast = list("group:time"))
fit
plot(fit)
plot(fit$MCTP)
}

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{shoulder}
\alias{shoulder}
\title{Shoulder Tip Pain Study}
\format{
A data frame with variables:
\describe{
\item{resp}{Shoulder pain score.}
\item{time}{Measurement occasion.}
\item{group1}{Treatment group.}
\item{group2}{Gender group.}
\item{subject}{Patient identifier.}
}
}
\usage{
shoulder
}
\description{
Shoulder pain scores from patients after laparoscopic abdominal surgery. The
study includes treatment group, gender group, and repeated pain measurements.
}
\examples{
data(shoulder)

\dontrun{
fit <- nparLD(resp ~ group1 * group2 * time,
              data = shoulder,
              subject = "subject",
              hypothesis = "H0F")
fit
}

\dontrun{
fit <- nparLD(resp ~ group1 * group2 * time,
              data = shoulder,
              subject = "subject",
              hypothesis = "H0p",
              contrast = list("group1:time"),
              Factor.Information=TRUE)
fit
plot(fit)
plot(fit$MCTP)
}

}
\keyword{datasets}
